/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.biz.api.analyse.node;

import com.abssqr.plat.common.facade.enums.AnalyseServiceEnum;
import com.abssqr.plat.common.facade.model.report.PlanReportItem;
import com.abssqr.plat.common.facade.model.report.ReportField;
import com.abssqr.plat.common.facade.model.report.wk.WkAstReportFields;
import com.abssqr.plat.common.facade.result.analyse.AnalyseChartItemVO;
import com.abssqr.plat.core.service.util.AnalyseReportUtil;
import com.google.common.collect.Lists;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 资金情况流动
 * @author hanfei
 * @version $Id: AnalyseChartAna205Node.java, v 0.1 2019-10-09 11:26 PM hanfei Exp $
 */
@Component
public class AnalyseChartAna205Node extends AnalyseChartNodeBase {
    
    /**
     * 资金情况流动
     资金流出项	图表：资金流出明细
     资金流入项	图表：资金流入明细
     转让贷款资金发生额
     兑付流出	兑付流出-启鑫、兑付流出-汇鑫……
     金融服务费	金融服务费-启鑫、金融服务费-汇鑫……
     新增贷款投放	启鑫放款
     * @param context
     * @return
     */
    @Override
    protected Void handle(AnalyseChartContext context) {
        List<ReportField> statFields = Lists.newArrayList(
                // 新增贷款投放  负
                WkAstReportFields.WK111,
                // 还款资金发生额  正
                WkAstReportFields.WK004,
                // 兑付流出     负
                WkAstReportFields.WK005,
                // 放款冲正金额   正
                WkAstReportFields.WK101,
                // 金融服务费    负
                WkAstReportFields.WK108,
                // 其他流入     正
                WkAstReportFields.WK007,
                // 其他流出     负
                WkAstReportFields.WK006
                
        );
        // 查询计划对应统计项数据
        Map<String, PlanReportItem> itemMap =
                analyseSpvReportManager.querySpvStatPlanDayItem(context.getEndDate(),
                        context.getPlanBase().getPlanNo(), statFields, true);
        List<AnalyseChartItemVO> itemVOS = new ArrayList<>();
//        // 总流出
//        AnalyseChartItemVO outTotal = new AnalyseChartItemVO();
//        outTotal.setEntry(WkAstReportFields.WK008.getCode());
//        outTotal.setItem(WkAstReportFields.WK008.getName());
//        outTotal.setValue(AnalyseReportUtil.getItemBig(itemMap, CollectionUtil.newArrayList(
//                // 流出
//                // 兑付流出
//                WkAstReportFields.WK005,
//                // 金融服务费
//                WkAstReportFields.WK108,
//                // 新增贷款投放
//                WkAstReportFields.WK111
//        )));
//        itemVOS.add(outTotal);
        // 正负切换
        int sign = 1;
        // 各个分项
        for (ReportField field : statFields) {
            sign = sign * -1;
            AnalyseChartItemVO itemVO = new AnalyseChartItemVO();
            itemVO.setEntry(String.valueOf(sign));
            itemVO.setItem(field.getName());
            // 处理统计值
            itemVO.setValue(AnalyseReportUtil.getItemBig(itemMap, field).multiply(new BigDecimal(sign)));
            itemVOS.add(itemVO);
        }
        context.addItems(itemVOS);
        return null;
    }
    
    
    @Override
    protected AnalyseServiceEnum getServiceEnum(){
        return AnalyseServiceEnum.DAY_FLOW;
    }
    
}
