/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.api.ast;

import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import com.abssqr.plat.biz.support.QryTemplate;
import com.abssqr.plat.biz.support.ServiceSupport;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.AstScreenRuleDO;
import com.abssqr.plat.common.dal.mysql.auto.paging.ScreenRulePage;
import com.abssqr.plat.common.facade.model.rule.AstScreenRule;
import com.abssqr.plat.common.facade.param.ast.AstScreenRuleEditParam;
import com.abssqr.plat.common.facade.param.ast.AstScreenRulePageQryParam;
import com.abssqr.plat.common.facade.param.ast.AstScreenRuleQryParam;
import com.abssqr.plat.common.facade.result.BasePage;
import com.abssqr.plat.common.facade.result.ast.AstScreenRuleVO;
import com.abssqr.plat.common.facade.service.ast.AstScreenRuleService;
import com.abssqr.plat.common.facade.utils.CalcUtil;
import com.abssqr.plat.common.facade.validation.Valid;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.common.model.utils.ApproveChangeUtil;
import com.abssqr.plat.core.service.rule.AstScreenRuleManager;
import com.general.system.bpm.enums.ApproveStatusEnum;
import com.general.system.facade.current.IdentityUtil;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @author bangis.wangdf
 * @version com.abssqr.plat.biz.api.ast: AstScreenRuleServiceImpl.java, v 0.1 2019-06-01 21:02 bangis.wangdf Exp $
 */
@Service
public class AstScreenRuleServiceImpl extends ServiceSupport implements AstScreenRuleService {
    @Autowired
    PlanRepo             planRepo;
    @Autowired
    AstScreenRuleManager astScreenRuleManager;

    @Override
    public BasePage<AstScreenRuleVO> getAstScreenList(AstScreenRulePageQryParam param) {
        return this.doQry(param, () -> {
            ScreenRulePage screenRulePage = astScreenRuleManager.getScreenRuleList(param, IdentityUtil.getUserCode());
            BasePage<AstScreenRuleVO> result = new BasePage<>();
            result.setCurrPageNo(screenRulePage.getCurrPageNo());
            result.setTotal(screenRulePage.getTotal());
            result.setLimit(screenRulePage.getLimit());

            if (CollectionUtils.isNotEmpty(screenRulePage.getDatas())) {
                Set<String> lenderOrgCodes = screenRulePage.getDatas().stream().map(AstScreenRuleDO::getLenderOrgCode).collect(
                    Collectors.toSet());
                Map<String, String> planNameMap = planRepo.getPlanNames(Lists.newArrayList(lenderOrgCodes));

                List<AstScreenRuleVO> datas = screenRulePage.getDatas().stream().map(astScreenRuleDO -> {
                    AstScreenRuleVO vo = new AstScreenRuleVO();
                    vo.setGmtCreate(astScreenRuleDO.getGmtCreate());
                    vo.setGmtModified(astScreenRuleDO.getGmtModified());
                    vo.setScreenRuleNo(astScreenRuleDO.getScreenRuleNo());
                    vo.setScreenRuleName(astScreenRuleDO.getScreenRuleName());
                    vo.setProdCode(astScreenRuleDO.getProdCode());
                    vo.setProdName(astScreenRuleDO.getProdName());
                    vo.setLenderOrgCode(astScreenRuleDO.getLenderOrgCode());
                    vo.setLenderOrgName(planNameMap.get(astScreenRuleDO.getLenderOrgCode()));
                    vo.setApproveStatus(ApproveStatusEnum.getEnum(astScreenRuleDO.getApproveStatus()));
                    //可编辑判断
                    vo.setOperate(ApproveChangeUtil.allowEdit(vo.getApproveStatus()));
                    return vo;
                }).collect(Collectors.toList());

                result.setDatas(datas);
            }
            return result;
        });
    }

    /**
     * 新增Or编辑资产筛选
     *
     * @param param
     * @return
     */
    @Override
    public AstScreenRule addOrUpdate(@Valid AstScreenRuleEditParam param) {
        return this.doOpr(param, () -> {
            String astScreeningNo = astScreenRuleManager.addOrUpdate(param, IdentityUtil.getUserCode(), IdentityUtil.getUserOrgCode());
            return astScreenRuleManager.getScreenRuleByNo(astScreeningNo);
        });
    }
    
    /**
     * 查询资产筛选规则
     * @param param
     * @return
     */
    @Override
    public AstScreenRule getAstScreenRule(AstScreenRuleQryParam param) {
        if (param.getWork()){
            return this.doQry(param, new QryTemplate<AstScreenRule>() {
                @Override
                public AstScreenRule query() {
                  return astScreenRuleManager.getDataChangeScreenRuleBy(param.getScreenRuleNo());
                }

                @Override
                public AstScreenRule afterProcess(AstScreenRule ret) {
                    calcRate(ret);
                    return ret;
                }
            });
        }else{
            return this.doQry(param, new QryTemplate<AstScreenRule>() {
                @Override
                public AstScreenRule query() {
                    return astScreenRuleManager.getScreenRuleByNo(param.getScreenRuleNo());
                }

                @Override
                public AstScreenRule afterProcess(AstScreenRule ret) {
                    calcRate(ret);
                    return ret;
                }
            });
        }
    }

    private void calcRate(AstScreenRule ret) {
        if(ret!=null) {
            if(ret.getSingleMaxInterRate()!=null) {
                ret.setSingleMaxInterRate(CalcUtil.calcPercRate(ret.getSingleMaxInterRate()));
            }
            if(ret.getSingleMinInterRate()!=null) {
                ret.setSingleMinInterRate(CalcUtil.calcPercRate(ret.getSingleMinInterRate()));
            }
        }
    }
}
