/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.api.plan;

import com.abssqr.plat.biz.support.ServiceSupport;
import com.abssqr.plat.common.dal.mysql.auto.paging.MonitorRecordPage;
import com.abssqr.plat.common.facade.enums.PlanIndicatorEnum;
import com.abssqr.plat.common.facade.model.plan.PlanMonitor;
import com.abssqr.plat.common.facade.model.plan.PlanMonitorRecord;
import com.abssqr.plat.common.facade.param.base.CommonPlanQryParam;
import com.abssqr.plat.common.facade.param.plan.monitor.PlanEventMonitorQryParam;
import com.abssqr.plat.common.facade.param.plan.monitor.PlanMonitorDelParam;
import com.abssqr.plat.common.facade.param.plan.monitor.PlanMonitorOprParam;
import com.abssqr.plat.common.facade.param.plan.monitor.PlanMonitorQryParam;
import com.abssqr.plat.common.facade.result.BasePage;
import com.abssqr.plat.common.facade.service.plan.PlanMonitorService;
import com.abssqr.plat.common.facade.validation.Valid;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.repo.plan.PlanMonitorRecordRepo;
import com.abssqr.plat.common.model.repo.plan.PlanMonitorRepo;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.MessageUtil;
import com.general.system.common.util.VarChecker;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @author zhenxuan.luo
 * @version com.abssqr.plat.biz.api.plan: PlanMonitorServiceImpl.java, v 0.1 2019-05-28 15:23 zhenxuan.luo Exp $
 */
@Component
public class PlanMonitorServiceImpl extends ServiceSupport implements PlanMonitorService {
    
    private final static Logger LOGGER = LoggerFactory.getLogger(PlanMonitorServiceImpl.class);
    
    @Autowired
    private PlanMonitorRepo       planMonitorRepo;
    @Autowired
    private PlanMonitorRecordRepo planMonitorRecordRepo;
    
    @Override
    public PlanMonitor editMonitor(@Valid PlanMonitorOprParam param) {
        
        return this.doOpr(param, () -> {
            
            PlanMonitor existPlanMonitor = planMonitorRepo.getByIndicator(param.getPlanNo(), param.getIndicator());
            if (param.getAdd()) {
                if (existPlanMonitor!=null) {
                    throw new AbssqrBizException(
                            MessageUtil.formatMsg("存在[{0}]的{1}监控指标，无法新增", param.getPlanNo(), PlanIndicatorEnum.getByCode(param.getIndicator()).getDesc()));
                }
                planMonitorRepo.insert(constructDomain(param));
            } else {
                planMonitorRepo.updateByIndicator(constructDomain(param));
            }
            
            return planMonitorRepo.getByIndicator(param.getPlanNo(), param.getIndicator());
        });
    }
    
    @Override
    public Void deletePlanMonitor(@Valid PlanMonitorDelParam param) {
        return this.doOpr(param, () -> {
            int refCnt = planMonitorRepo.deleteByIndicator(param.getPlanNo(), param.getIndicator());
            LogUtil.info(LOGGER, "删除计划指标项[{0}]，实际删除[{1}]条数据", param, refCnt);
            return null;
        });
    }
    
    /**
     * 事件列表
     *
     * @param param
     * @return
     */
    @Override
    public BasePage<PlanMonitorRecord> queryEventMonitorList(@Valid PlanEventMonitorQryParam param) {
            return this.doQry(param, () -> {
                MonitorRecordPage monitorRecordPage=planMonitorRecordRepo.getEventByPlan(param);
                BasePage<PlanMonitorRecord> result = new BasePage<>();
                
                result.setCurrPageNo(monitorRecordPage.getCurrPageNo());
                result.setTotal(monitorRecordPage.getTotal());
                result.setLimit(monitorRecordPage.getLimit());
    
                if (CollectionUtils.isNotEmpty(monitorRecordPage.getDatas())) {
        
                    List<PlanMonitorRecord> datas = monitorRecordPage.getDatas().stream().map(planMonitorRecordDO -> {
                        
                        PlanMonitorRecord vo = new PlanMonitorRecord();
                        vo.setGmtCreate(planMonitorRecordDO.getGmtCreate());
                        vo.setMonitorName(planMonitorRecordDO.getMonitorName());
                        vo.setExcessValue(planMonitorRecordDO.getExcessValue());
                        vo.setId(planMonitorRecordDO.getId());
                        vo.setIndicator(PlanIndicatorEnum.getByCode(planMonitorRecordDO.getIndicator()));
                        vo.setPlanNo(planMonitorRecordDO.getPlanNo());
                        vo.setValue(planMonitorRecordDO.getValue());
                        
                        return vo;
                    }).collect(Collectors.toList());
        
                    result.setDatas(datas);
                }
                return result;
            }
       );
    }
    
    @Override
    public PlanMonitor queryMonitorDtl(@Valid PlanMonitorQryParam param) {
        return this.doQry(param, () -> {
            return planMonitorRepo.getByIndicator(param.getPlanNo(), param.getIndicator());
        });
    }
    
    @Override
    public List<PlanMonitor> queryMonitorList(@Valid CommonPlanQryParam param) {
        return this.doQry(param, () -> planMonitorRepo.getByPlan(param.getPlanNo()));
    }
    
    private PlanMonitor constructDomain(PlanMonitorOprParam param) {
        PlanMonitor planMonitor = new PlanMonitor();
        planMonitor.setPlanNo(param.getPlanNo());
        planMonitor.setIndicator(PlanIndicatorEnum.getByCode(param.getIndicator()));
        planMonitor.setName(param.getName());
        planMonitor.setValue(param.getValue());
        return planMonitor;
    }
}