/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.api.plan;

import cn.hutool.core.collection.CollectionUtil;
import com.abssqr.plat.biz.support.OprTemplate;
import com.abssqr.plat.biz.support.QryTemplate;
import com.abssqr.plat.biz.support.ServiceSupport;
import com.abssqr.plat.biz.validate.ValidateUtil;
import com.abssqr.plat.biz.validate.ValidatorError;
import com.abssqr.plat.common.dal.mysql.auto.paging.PlanListPage;
import com.abssqr.plat.common.facade.base.BaseParam;
import com.abssqr.plat.common.facade.enums.HandleTypeEnum;
import com.abssqr.plat.common.facade.enums.PlanStatusEnum;
import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.model.org.OrganizationEntity;
import com.abssqr.plat.common.facade.model.plan.FeeStructure;
import com.abssqr.plat.common.facade.model.plan.InvestStructure;
import com.abssqr.plat.common.facade.model.plan.PlanAst;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.model.prod.LoanProduct;
import com.abssqr.plat.common.facade.param.plan.PlanDelOprParam;
import com.abssqr.plat.common.facade.param.plan.PlanOprParam;
import com.abssqr.plat.common.facade.param.plan.PlanPageQryParam;
import com.abssqr.plat.common.facade.param.plan.PlanQryParam;
import com.abssqr.plat.common.facade.result.BasePage;
import com.abssqr.plat.common.facade.result.plan.PlanListVO;
import com.abssqr.plat.common.facade.result.plan.PlanVO;
import com.abssqr.plat.common.facade.service.plan.PlanService;
import com.abssqr.plat.common.facade.utils.CalcUtil;
import com.abssqr.plat.common.facade.validation.Valid;
import com.abssqr.plat.common.model.exception.AbssqrInvalidParamException;
import com.abssqr.plat.common.model.repo.plan.PlanAstRepository;
import com.abssqr.plat.common.model.repo.prod.LoanProdRepository;
import com.abssqr.plat.common.model.utils.ApproveChangeUtil;
import com.abssqr.plat.common.util.utils.BeanCopierUtil;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.plan.PlanManager;
import com.general.system.bpm.enums.ApproveStatusEnum;
import com.general.system.common.model.Money;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.IEnumUtil;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.VarChecker;
import com.general.system.facade.current.IdentityUtil;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 资产转让项目计划管理
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.biz.api.plan: PlanServiceImpl.java, v 0.1 2019-05-28 15:23 zhenxuan.luo Exp $
 */
@Component
public class PlanServiceImpl extends ServiceSupport implements PlanService {
    private static final Logger LOGGER = LoggerFactory.getLogger(PlanServiceImpl.class);

    @Autowired
    PlanManager planManager;
    @Autowired
    LoanProdRepository loanProdRepository;
    @Autowired
    PlanAstRepository planAstRepository;
    @Autowired
    private OrgAcctDayComponent orgAcctDayComponent;
    /**
     * 产品分页查询
     *
     * @param param
     * @return
     */
    @Override
    public BasePage<PlanListVO> getPlanList(@Valid PlanPageQryParam param, boolean ast) {
        return this.doQry(param, () -> {
            // 增加计划运行状态条件
            List<String> status = null;
            // 资产列表 限定为 存续中 和 已清算
            if (ast || HandleTypeEnum.getByCode(param.getHandleType()) == HandleTypeEnum.REPORT) {
                status = CollectionUtil
                        .newArrayList(PlanStatusEnum.ONGOING.getCode(), PlanStatusEnum.COMPLETED.getCode());
            }
            PlanListPage planList = planManager.getPlanList(param, status, IdentityUtil.getUserCode());

            BasePage<PlanListVO> result = new BasePage();

            List<PlanListVO> datas = planList.getDatas().stream().map(planDO -> {
                PlanListVO vo = new PlanListVO();
                vo.setPlanName(planDO.getPlanName());
                vo.setPlanNo(planDO.getPlanNo());
                vo.setBeginDate(planDO.getBeginDate());
                vo.setEndDate(planDO.getEndDate());
                vo.setStatus(IEnumUtil.getEnum(PlanStatusEnum.class, planDO.getStatus()));
                vo.setApproveStatus(IEnumUtil.getEnum(ApproveStatusEnum.class, planDO.getApproveStatus()));
                //可编辑判断
                vo.setOperate(ApproveChangeUtil.allowEditPlan(vo.getApproveStatus(), vo.getStatus(), PlanTypeEnum.getByCode(planDO.getPlanType())));
                vo.setStopPayDate(planDO.getStopPayDate());
                if (ast) {
                    //模型增加  当前资产规模  当前余额
                    buildByReport(planDO.getPlanNo(), vo);
                }

                if (planDO.getTotalAmt() != null) {
                    Money money = new Money();
                    money.setCent(planDO.getTotalAmt());
                    vo.setTotalAmt(money);
                }
                return vo;
            }).collect(Collectors.toList());

            result.setDatas(datas);
            result.setLimit(planList.getLimit());
            result.setTotal(planList.getTotal());
            result.setCurrPageNo(planList.getCurrPageNo());

            return result;
        });
    }

    /**
     * 新增OrUpdate
     *
     * @param param
     * @return
     */
    @Override
    public String edit(PlanOprParam param) {
        Date sysDate = orgAcctDayComponent.getTrfAcctDay().getStandardDate();
        return this.doOpr(param, new OprTemplate<String>() {
            @Override
            public String execute() {
                // 受让计划 投资结构起息日等于 项目起息日
                if (param.getPlanType()==null ||param.getPlanType().equals(PlanTypeEnum.PLAN.getCode())){
                    if (CollectionUtils.isNotEmpty(param.getInvestStructureList())) {
                        param.getInvestStructureList().forEach(item -> item.setEffectDate(param.getEffectDate()));
                    }
                }
                // 参数校验
                if (param.getDraft() == false) {
                    ValidatorError errors = ValidateUtil.validate(param);
                    if (errors.hasErrors()) {
                        String errorMsg = ValidateUtil.collectErrorsLog(param, errors);
                        LogUtil.warn(LOGGER, "非法的请求参数：" + errorMsg);
                        throw new AbssqrInvalidParamException("非法的请求参数：" + errorMsg);
                    }
                    if (param instanceof BaseParam) {
                        ((BaseParam) param).checkParam();
                    }
                    if (param.getAdd()) {
                        param.getInvestStructureList().forEach(item -> {
                            Date standardDate = DateTimeUtil.getStandardDate(item.getEffectDate());
                            VarChecker.checkArgument(DateTimeUtil.calcDayDiff(standardDate, sysDate) > 0,
                                    "[{0}]非法的请求参数: 投资结构起息日[{1}]不能小于等于当前日期[{2}]", param.getPlanNo(), standardDate, sysDate);
                        });
                    } else {
                        param.getInvestStructureList().stream().filter(item -> item.getAppend() == true).forEach(item -> {
                            Date standardDate = DateTimeUtil.getStandardDate(item.getEffectDate());
                            VarChecker.checkArgument(DateTimeUtil.calcDayDiff(standardDate, sysDate) > 0,
                                    "[{0}]非法的请求参数: 投资结构起息日[{1}]不能小于等于当前日期[{2}]", param.getPlanNo(), standardDate, sysDate);
                        });
                    }
                } else {
                    VarChecker.checkNotEmpty(param.getPlanNo(), "非法的请求参数: 计划编号不能为空");
                    VarChecker.checkNotEmpty(param.getPlanName(), "非法的请求参数: 计划名称不能为空");
                }

                return planManager.addOrUpdatePlan(param, IdentityUtil.getUserCode(), IdentityUtil.getUserOrgCode());
            }
        });
    }


    /**
     * 删除草稿箱
     *
     * @param param
     */
    @Override
    public void deleteDraftPlan(@Valid PlanDelOprParam param) {
        this.doOpr(param, () -> planManager.deleteDraftPlan(param.getPlanNo()));

    }

    /**
     * 查询计划
     *
     * @param param
     * @return
     */
    @Override
    public PlanVO getPlan(@Valid PlanQryParam param) {

        return this.doQry(param, new QryTemplate<PlanVO>() {
            @Override
            public PlanVO query() {
                PlanBase planBase = null;
                if (param.getWork()) {
                    planBase = planManager.getDataChangePlanBy(param.getPlanNo());
                } else {
                    planBase = planManager.getPlanFullByNo(param.getPlanNo());
                }

                return convertPlanVO(planBase);
            }

            @Override
            public PlanVO afterProcess(PlanVO ret) {
                return convertPlanRate(ret);
            }
        });

    }

    /**
     * 转换模型
     *
     * @param planBase
     * @return
     */
    private PlanVO convertPlanVO(PlanBase planBase) {
        PlanVO result = BeanCopierUtil.copy(PlanVO.class, planBase);
        //prodList->planProd
        if (planBase.getPlanType() == PlanTypeEnum.SPV && planBase instanceof OrganizationEntity) {
            OrganizationEntity entity = (OrganizationEntity) planBase;
            List<String> prodCodeList = entity.getProdCodeList();
            List<LoanProduct> productList = loanProdRepository.getBybssqrProdCodes(prodCodeList);
            result.setProducts(productList);
        }
        return result;
    }

    /**
     * 计算层级利率、费率
     *
     * @param ret
     * @return
     */
    private PlanVO convertPlanRate(PlanVO ret) {
        ret.setVatRate(CalcUtil.calcPercRate(ret.getVatRate()));
        for (InvestStructure investStructure : ret.getInvestStructureList()) {
            investStructure.setLayerIntr(CalcUtil.calcPercRate(investStructure.getLayerIntr()));
        }
        for (FeeStructure feeStructure : ret.getFeeStructureList()) {
            feeStructure.setFeeRate(CalcUtil.calcPercRate(feeStructure.getFeeRate()));
        }
        ret.setFinSerFeeRate(CalcUtil.calcPercRate(ret.getFinSerFeeRate()));

        return ret;
    }

    /**
     * 构建资产规模数据
     *
     * @param planNo
     * @param planListVO
     * @return
     */
    private PlanListVO buildByReport(String planNo, PlanListVO planListVO) {

        PlanAst planAst = planAstRepository.getByPlanNo(planNo);

        Money ZERO = new Money();
        //资产当前规模
        planListVO.setAstCurrentAmt(planAst == null ? ZERO : planAst.getAccGrant());
        //资产当前余额
        planListVO.setAstCurrentBal(planAst == null ? ZERO : planAst.getCash());

        return planListVO;
    }

}