/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */
package com.abssqr.plat.biz.validate.validator;

import com.abssqr.plat.biz.validate.AFieldValidator;
import com.abssqr.plat.biz.validate.FieldError;
import com.abssqr.plat.biz.validate.ValidateService;
import com.abssqr.plat.common.facade.validation.VldNotNull;

/**
 * 注解@{@link VldNotNull}的校验类实现
 *
 * @author zhenxuan.luo
 * @version $Id: NotNullValidator.java, v 0.1 2015年5月19日 上午11:03:19 zhenxuan.luo Exp $
 */
public final class NotNullValidator extends AFieldValidator<VldNotNull> {
    /**
     * 默认构造器
     */
    public NotNullValidator() {
        super(VldNotNull.class);
    }
    
    @Override
    public boolean isValid(ValidateService validateService, VldNotNull annotation, Object object,
                           FieldError fieldError) {
        boolean isValid = true;
        try {
            return isValid = validate(object);
        } finally {
            if (!isValid) {
                fieldError.setErrorMsg(annotation.message());
            }
        }
    }
    
    /**
     * 校验
     * @param object 被校验的字段对象
     * @return true:校验通过,反之返回false
     */
    private boolean validate(Object object) {
        if (object != null) {
            return true;
        }
        return false;
    }
}
