/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.dispatcher;

import com.abssqr.plat.biz.shared.interceptor.TaskDigestLogUtil;
import com.abssqr.plat.common.model.domain.task.DispatcherNode;
import com.abssqr.plat.common.util.loggers.CommonLoggers;
import com.abssqr.plat.common.util.loggers.LogInfo;
import com.abssqr.plat.common.util.thread.ThreadLocalCleaner;
import com.general.system.common.model.exception.BaseException;
import com.general.system.common.util.LogUtil;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.UncategorizedSQLException;
import org.springframework.transaction.CannotCreateTransactionException;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallback;
import org.springframework.transaction.support.TransactionTemplate;

import static com.general.system.common.util.MessageUtil.formatMsg;

/**
 *
 * @author zhenxuan.luo
 * @version $Id: DispatcherExecutorTemp.java, v 0.1 2017-07-02 上午11:13 zhenxuan.luo Exp $
 */
public abstract class DispatcherExecutorTemp<T extends DispatcherNode>
        implements DispatcherExecutor<T> {
    
    private final static Logger LOG = CommonLoggers.COMMON_DISPATCHER;
    
    @Autowired
    private TransactionTemplate transactionTemplate;
    
    @Override
    public void execute(final T node) {
        
        Throwable ex = null;
        T aftNode = null;
        LogInfo logInfo = TaskDigestLogUtil.initDigestLogCallerInfo(getName());
        try {
            LogUtil.info(LOG, "[DispatcherExecutor]start execute {0}", node);
            if (node.needTrans()) {
                aftNode = transactionTemplate.execute(new TransactionCallback<T>() {
                    @Override
                    public T doInTransaction(TransactionStatus status) {
                        return doExecute(node);
                    }
                });
            } else {
                aftNode = doExecute(node);
            }
            LogUtil.info(LOG, "[DispatcherExecutor]success execute {0}", node);
            
        } catch (BaseException e) {
            ex = e;
            LogUtil.error(LOG, e, "[DispatcherExecutor]error execute {0} in {1}-{2}",
                    node, e.getClass().getSimpleName(), e.getErrCode().getCode());
            String msg = formatMsg("{0}@{1}@{2}", e.getErrCode(), e.getClass().getSimpleName(), e.getMessage());
            if (e.isCanRetry()) {
                aftNode = this.markException(node, msg);
            } else {
                aftNode = this.markFail(node, msg);
            }
        } catch (CannotCreateTransactionException e) {
            ex = e;
            LogUtil.error(LOG, e, "[DispatcherExecutor]success execute {0}", node);
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e1) {
                LogUtil.error(LOG, e, "[DispatcherExecutor]success execute {0}", node);
            }
            aftNode = this.markException(node,
                formatMsg("unknown@{0}@{1}", e.getClass().getSimpleName(), e.getMessage()));
        } catch (UncategorizedSQLException e) {
            ex = e;
            LogUtil.error(LOG, e, "[DispatcherExecutor]success execute {0}", node);
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e1) {
                LogUtil.error(LOG, e, "[DispatcherExecutor]success execute {0}", node);
            }
            aftNode = this.markException(node,
                formatMsg("unknown@{0}@{1}", e.getClass().getSimpleName(), e.getMessage()));
        } catch (Throwable e) {
            ex = e;
            String msg = formatMsg("unknown@{0}@{1}", e.getClass().getSimpleName(), e.getMessage());
            LogUtil.error(LOG, e, "[DispatcherExecutor]error execute {0} in {1}}", node, msg);
            aftNode = this.markException(node, msg);
        } finally {
            TaskDigestLogUtil.logDigest(logInfo, ex, aftNode);
            ThreadLocalCleaner.clean();
        }
        
    }
    
    protected abstract T doExecute(T node);
    
    /**
     * 标记成功
     *
     * @param node
     */
    protected abstract T markSuccess(T node);
    
    /**
     * 标记失败
     *
     * @param node
     */
    protected abstract T markFail(T node, String failMessage);
    
    /**
     * 标记异常
     *
     * @param node
     */
    protected abstract T markException(T node, String failMessage);
    
    protected abstract String getName();
    
}