/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.dispatcher.transfer;

import com.abssqr.plat.biz.shared.dispatcher.DispatcherExecutorTemp;
import com.abssqr.plat.common.dal.mysql.auto.dao.TrTaskDAO;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.model.domain.task.TrTask;
import com.abssqr.plat.common.model.repo.tr.TrTaskRepo;
import com.abssqr.plat.core.service.ast.LoanAstTrManager;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.SystemDateUtil;
import com.general.system.common.util.LogUtil;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;

/**
 * 资产转让执行器
 *
 * @author bangis.wangdf
 * @version com.abssqr.plat.biz.shared.task.task: AstTrTaskExecutor.java, v 0.1 2018-01-08 02:15 hongwei.wang Exp $
 */
@Component
public class AstTrTaskExecutor extends DispatcherExecutorTemp<TrTask> {
    // 单位（秒）
    private final static int    DEFAULT_STEP = 5;
    private final static Logger logger       = LoggerFactory.getLogger(AstTrTaskExecutor.class);
    
    @Autowired
    private TrTaskDAO        trTaskDAO;
    @Autowired
    private LoanAstTrManager loanAstTrManager;
    @Autowired
    private TrTaskRepo       trTaskRepo;

    @Override
    public TrTask doExecute(TrTask node) {
        
        logger.info(node.toString());
        boolean rs = false;
        Date befDt = SystemDateUtil.getSystemDate();
        try {
            TrTask lockedNode = trTaskRepo.lockByIdNowait(node.getId());

            if (null == lockedNode || lockedNode.getTaskStatus() != TaskStatusEnum.EXECUTING) {
                LogUtil.warn(logger, "转让任务[{0}]状态[{1}]不为执行中，可能存在并发问题", node.getId(),
                        (lockedNode != null) ? lockedNode.getTaskStatus() : "-");
                return lockedNode;
            }
            // 执行转让操作
            rs = loanAstTrManager.transfer(node);
            if (rs) {
                return markSuccess(node);
            } else {
                return markFail(node, "转让校验失败");
            }
        } finally {
            Date endDt = SystemDateUtil.getSystemDate();
            logger.info("[AstTrTaskDigest] astId=[" + node.getAstNo() + "], res=" + rs + ", elipse=" + (endDt.getTime()
                                                                                                        - befDt.getTime())
                        + "ms");
        }
    }
    
    /**
     * 标记成功
     *
     * @param node
     */
    @Override
    protected TrTask markSuccess(TrTask node) {
        node.setTaskStatus(TaskStatusEnum.SUCCESS);
        trTaskDAO.updateTaskStatus(node.getId(), "", TaskStatusEnum.SUCCESS.getCode());
        return node;
    }
    
    /**
     * 标记失败
     *
     * @param node
     * @param failMessage
     */
    @Override
    protected TrTask markFail(TrTask node, String failMessage) {
        node.setTaskStatus(TaskStatusEnum.FAILURE);
        trTaskDAO.updateTaskStatus(node.getId(), failMessage, TaskStatusEnum.FAILURE.getCode());
        return node;
    }
    
    /**
     * 标记异常
     *
     * @param node
     * @param failMessage
     */
    @Override
    protected TrTask markException(TrTask node, String failMessage) {
        if (StringUtils.isBlank(failMessage)) {
            failMessage = "转让执行失败";
        }
        node.setTaskStatus(TaskStatusEnum.INIT);
        trTaskDAO.updateTaskExep(DateTimeUtil.addSecond(SystemDateUtil.getSystemDate(), DEFAULT_STEP),
                node.getId(),
                failMessage.length() > 2000 ? failMessage.substring(0, 1990) : failMessage);
        return node;
    }
    
    @Override
    protected String getName() {
        return "AstTrTaskExecutor";
    }
}
