/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.scheduler;

import com.abssqr.plat.biz.shared.interceptor.TaskDigestLogUtil;
import com.abssqr.plat.common.model.domain.job.JobControl;
import com.abssqr.plat.common.model.repo.job.JobControlRepository;
import com.abssqr.plat.common.util.loggers.LogInfo;
import com.general.system.common.util.LogUtil;
import org.quartz.CronScheduleBuilder;
import org.quartz.CronTrigger;
import org.quartz.Job;
import org.quartz.JobBuilder;
import org.quartz.JobDetail;
import org.quartz.JobExecutionContext;
import org.quartz.TriggerBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;

/**
 *
 * @author zhenxuan.luo
 * @version $Id: BaseJobInitializer.java, v 0.1 2017-06-29 下午18:46 zhenxuan.luo Exp $ 
 */
public abstract class BaseQuartzJob implements InitializingBean, Job {
    
    protected final Logger LOGGER = LoggerFactory.getLogger(getClass());
    
    @Autowired
    protected AbssqrJobCollector abssqrJobCollector;
    
    @Autowired
    protected JobControlRepository jobControlRepository;
    
    public JobDetail buildJobDetail() {
        Class<? extends Job> clz = this.getClass();
        JobDetail jobDetail = JobBuilder.newJob(clz).withIdentity(getJobName(), getJobName())
                                      .requestRecovery(isRequestRecovery())
                                      .build();
        
        return jobDetail;
    }
    
    public CronTrigger buildJobTrigger() {
        
        String cronExpr = this.getCronExpr();
        
        // 表达式调度构建器
        CronScheduleBuilder scheduleBuilder = CronScheduleBuilder.cronSchedule(cronExpr)
                                                      .withMisfireHandlingInstructionDoNothing();
        // 构建一个trigger
        TriggerBuilder<CronTrigger> triggerBuilder = TriggerBuilder.newTrigger()
                                                             .withIdentity(getJobName()).withSchedule(scheduleBuilder);
        
        CronTrigger trigger = triggerBuilder.build();
    
        LogUtil.info(LOGGER, "创建定时任务[{0}]执行时间配置[{1}]", getJobName(), getCronExpr());
        return trigger;
    }
    
    @Override
    public void execute(JobExecutionContext context) {
        Throwable ex = null;
        LogInfo logInfo = TaskDigestLogUtil.initDigestLogCallerInfo(this.getJobName());
        try {
            JobControl jobControl = jobControlRepository.selectByJobName(this.getJobName());
            if (jobControl==null||!jobControl.getIsEff().isYes() ) {
                LogUtil.warn(LOGGER, "忽略[{0}--{1}]定时任务", getJobName(), jobControl);
                return;
            }
            LogUtil.info(LOGGER, "执行定时任务[{0}]开始", getJobName());
            doExecute();
            LogUtil.info(LOGGER, "执行定时任务[{0}]结束", getJobName());
        } catch (Throwable e) {
            ex = e;
            LogUtil.error(LOGGER, e, "执行定时任务[{0}]失败", getJobName());
        } finally {
            TaskDigestLogUtil.logDigest(logInfo, ex, null);
        }
    }
    
    @Override
    public void afterPropertiesSet() throws Exception {
        abssqrJobCollector.registerJob(this);
    }
    
    protected boolean isRequestRecovery() {
        return false;
    }
    
    public String getJobName() {
        return this.getClass().getName();
    }
    
    protected abstract String getCronExpr();
    
    protected abstract void doExecute();
    
}