/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.scheduler.job.keeper.node;

import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.model.domain.job.JobControl;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.repo.job.JobControlRepository;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.notice.AbssqrNoticeManager;
import com.general.system.common.model.IEnum;
import com.general.system.util.adapter.pipeline.AbstractMultiplePipelineNode;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.Calendar;
import java.util.Date;
import java.util.Set;

/**
 * 任务监控执行器节点 基类
 *
 * @author hanfei
 * @version $Id: AbstractKeeperNode.java, v 0.1 2019-07-26 2:55 PM hanfei Exp $
 */
public abstract class AbstractKeeperNode extends AbstractMultiplePipelineNode<Date> {

    protected final Logger LOGGER = LoggerFactory.getLogger(getClass());

    @Autowired
    private JobControlRepository jobControlRepository;

    @Autowired
    protected AbssqrNoticeManager abssqrNoticeManager;
    @Autowired
    protected OrgAcctDayComponent orgAcctDayComponent;
    /**
     * 执行次数
     */
    protected Long execTimes = 1L;


    @Override
    protected Void handle(Date systemDate) {
        if (this.judge(getWatchJob())){
            doHandle(systemDate);
        }
        return null;
    }


    /**
     * 判断监控job是否启用
     * @param watchJob
     * @return
     */
    private boolean judge(String watchJob) {
        if (StringUtils.isNotBlank(watchJob)){
            JobControl jobControl = jobControlRepository.selectByJobName(watchJob);
            if (jobControl.getIsEff().isYes()){
                return true;
            }
            return false;
        }
        return true;
    }

    public void sendBussinessNotice(PlanBase planEntity,
                                    String bizCode, String content, Set<String> receivers) {
        abssqrNoticeManager.sendBusinessNotice(planEntity, bizCode, getBizType(), content, receivers);

    }
    public void sendSystemNotice(String bizTitle,
                                 String bizCode, String content, Set<String> receivers) {
        abssqrNoticeManager.sendSystemNotice(bizTitle, bizCode, getBizType(), content, receivers);

    }


    public Set<String> getAdminReceivers() {
        return abssqrNoticeManager.getAdminReceivers();
    }

    /**
     * 获取指定时间
     * @param currentTime
     * @param deadline y M d h m s/h m s
     * @return
     */
    protected Date getDeadLineTime(Date currentTime, String deadline) {
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(currentTime);
        String[] times = deadline.split(" ");
        if (times.length == 6) {
            calendar.set(Calendar.YEAR, Integer.valueOf(times[0]));
            calendar.set(Calendar.MONTH, Integer.valueOf(times[1]) - 1);
            calendar.set(Calendar.DAY_OF_MONTH, Integer.valueOf(times[2]));
            calendar.set(Calendar.HOUR_OF_DAY, Integer.valueOf(times[3]));
            calendar.set(Calendar.MINUTE, Integer.valueOf(times[4]));
            calendar.set(Calendar.SECOND, Integer.valueOf(times[5]));
        } else if (times.length == 3) {
            calendar.set(Calendar.HOUR_OF_DAY, Integer.valueOf(times[0]));
            calendar.set(Calendar.MINUTE, Integer.valueOf(times[1]));
            calendar.set(Calendar.SECOND, Integer.valueOf(times[2]));
        } else {
            throw new AbssqrBizException("deadline格式不正确");
        }

        return calendar.getTime();
    }

    /**
     * 获取当前的标准日期
     * @return
     */
    public Date getCurrentStanderDate(){
        return Date.from(LocalDate.now()
                .atStartOfDay(ZoneId.systemDefault())
                .toInstant());
    }

    protected abstract IEnum getBizType();

    protected abstract String getWatchJob();

    protected abstract Void doHandle(Date date);


}
