package com.abssqr.plat.biz.shared.scheduler.job.keeper.node;

import com.abssqr.plat.common.facade.enums.AbssqrNoticeTypeEnum;
import com.abssqr.plat.common.facade.enums.PlanTaskTypeEnum;
import com.abssqr.plat.common.facade.enums.SyncTaskTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.model.domain.plan.PlanTaskLog;
import com.abssqr.plat.common.model.domain.task.ctr.AstSyncTaskCtrLog;
import com.abssqr.plat.common.model.enums.OrgCodeEnum;
import com.abssqr.plat.common.model.enums.OrgTypeEnum;
import com.abssqr.plat.common.model.repo.plan.PlanTaskLogRepo;
import com.abssqr.plat.common.model.repo.task.AstSyncTaskCtrRepo;
import com.general.system.common.model.IEnum;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.SystemDateUtil;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 模型转换完成通知
 */
@Component
public class AstModCompleteCheckKeeper extends AbstractKeeperNode {
    @Autowired
    PlanTaskLogRepo    planTaskLogRepo;
    @Autowired
    AstSyncTaskCtrRepo astSyncTaskCtrRepo;

    @Override
    protected Void doHandle(Date date) {
        for (OrgCodeEnum orgCodeEnum : OrgCodeEnum.values()) {
            if (orgCodeEnum.getType() == OrgTypeEnum.ABS) {
                continue;
            }
            LogUtil.info(LOGGER, "模型转换完成监控-start");
            Date execDate = orgAcctDayComponent.getTrfAcctDay().getStandardDate();
            List<PlanTaskLog> tasks = planTaskLogRepo.getTasks(execDate, PlanTaskTypeEnum.AST_MOD_COMPLETE_CHECK);
            List<String> orgCodes = tasks.stream().map(PlanTaskLog::getPlanNo).collect(Collectors.toList());
            if (CollectionUtils.isEmpty(orgCodes) || !orgCodes.contains(orgCodeEnum.getCode())) {
                AstSyncTaskCtrLog lastTaskLog = astSyncTaskCtrRepo.getTaskLogByExecDate(execDate, SyncTaskTypeEnum.AST_MOD, orgCodeEnum
                                                                                                                              .getCode());
                if (lastTaskLog != null) {
                    //如果同步完成
                    if (TaskStatusEnum.SUCCESS.getCode().equals(lastTaskLog.getStatus().getCode())) {
                        PlanTaskLog planTaskLog = new PlanTaskLog();
                        planTaskLog.setExecDate(execDate);
                        planTaskLog.setPlanNo(orgCodeEnum.getCode());
                        planTaskLog.setTaskType(PlanTaskTypeEnum.AST_MOD_COMPLETE_CHECK);
                        planTaskLogRepo.createTask(planTaskLog);
                        Set<String> receivers = super.getAdminReceivers();
                        Date compelteDate = SystemDateUtil.getSystemDate();
                        PlanBase planBase = new PlanBase();
                        sendBussinessNotice(planBase, orgCodeEnum.getCode(), buildNoticeMessage(execDate, compelteDate, orgCodeEnum), receivers);
                    }
                }
            }
        }

        LogUtil.info(LOGGER, "模型转换完成监控-end");
        return null;
    }

    private String buildNoticeMessage(Date currDate, Date completeDate, OrgCodeEnum orgCodeEnum) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        SimpleDateFormat sdf2 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String currAccDateStr = sdf.format(currDate);
        String completeDateStr = sdf2.format(completeDate);
        StringBuilder sb = new StringBuilder();
        sb.append("机构[");
        sb.append(orgCodeEnum.getCode());
        sb.append("]");
        sb.append(currAccDateStr);
        sb.append("模型转换任务已完成 完成时间为:");
        sb.append(completeDateStr);
        return sb.toString();
    }


    @Override
    protected IEnum getBizType() {
        return AbssqrNoticeTypeEnum.AstModCompleteCheck;
    }

    @Override
    protected String getWatchJob() {
        return "Wk1009AstSyncDispatcher";
    }


}
