package com.abssqr.plat.common.dal.mysql.auto.dao;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import java.util.ArrayList;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.AstScreenRuleDO;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.LinkedHashMap;
import java.util.ArrayList;
import java.util.HashSet;
import com.abssqr.plat.common.dal.mysql.auto.paging.ScreenRulePage;
import com.abssqr.plat.common.dal.mysql.auto.mapper.AstScreenRuleDOMapper;

/**
* The Table abs_ast_screen_rule.
* 资产筛选规则
*/
@Repository
public class AstScreenRuleDAO{

    @Autowired
    private AstScreenRuleDOMapper astScreenRuleDOMapper;

    /**
     * desc:插入表:abs_ast_screen_rule.<br/>
     * @param entity entity
     * @return int
     */
    
    public int insert(AstScreenRuleDO entity){
        return astScreenRuleDOMapper.insert(entity);
    }
    /**
     * desc:批量插入表:abs_ast_screen_rule.<br/>
     * @param list list
     * @return int
     */
    
    public int insertBatch(List<AstScreenRuleDO> list){
        astScreenRuleDOMapper.insertBatch(list);
        return  list.size(); 
    }
    /**
     * desc:根据唯一约束ScreenRuleNo更新表:abs_ast_screen_rule.<br/>
     * @param entity entity
     * @return int
     */
    
    public int updateByScreenRuleNo(AstScreenRuleDO entity){
        return astScreenRuleDOMapper.updateByScreenRuleNo(entity);
    }
    /**
     * desc:根据唯一约束ScreenRuleNo获取数据:abs_ast_screen_rule.<br/>
     * @param screenRuleNo screenRuleNo
     * @return AstScreenRuleDO
     */
    
    public AstScreenRuleDO getByScreenRuleNo(String screenRuleNo){
        return astScreenRuleDOMapper.getByScreenRuleNo(screenRuleNo);
    }
     /**
     * desc:根据唯一约束ScreenRuleNo获取数据:abs_ast_screen_rule.<br/>
     * @param screenRuleNos screenRuleNos
     * @return Map<String,AstScreenRuleDO>
     */
    
    public List<AstScreenRuleDO> getByScreenRuleNosList(List<String> screenRuleNos){
        return astScreenRuleDOMapper.getByScreenRuleNos(screenRuleNos);
    }
    /**
     * desc:根据唯一约束ScreenRuleNo获取数据:abs_ast_screen_rule.<br/>
     * @param screenRuleNos screenRuleNos
     * @return Map<String,AstScreenRuleDO>
     */
    
    public Map<String,AstScreenRuleDO> getByScreenRuleNos(List<String> screenRuleNos){
    if(screenRuleNos!=null && screenRuleNos.size()>201){throw new RuntimeException("screenRuleNos 超出201长度限制");}
        Map<String,AstScreenRuleDO> result = new LinkedHashMap();
        List<AstScreenRuleDO> resultList = astScreenRuleDOMapper.getByScreenRuleNos(screenRuleNos);
        if(resultList!=null && !resultList.isEmpty()){
            for(AstScreenRuleDO entity:resultList){
                result.put(entity.getScreenRuleNo(),entity);
            }
        }
        return result;
    }
    /**
     * desc:分页查询资产筛选规则.<br/>
     * @param screenRule screenRule
     * @return ScreenRulePage
     */
    
    public ScreenRulePage getScreenRuleList(ScreenRulePage screenRule){
        int total = astScreenRuleDOMapper.getScreenRuleListCount(screenRule);
        if(total>0){
            screenRule.setDatas(astScreenRuleDOMapper.getScreenRuleListResult(screenRule));
        }else{
            screenRule.setDatas(new ArrayList());
        }
        screenRule.setTotal(total);
        return screenRule;
    }
    /**
     * desc:获取同名筛选规则数量，防止重名存在.<br/>
     * @param screenRuleNo screenRuleNo
     * @param screenRuleName screenRuleName
     * @return int
     */
    
    public int getSameRuleCnt(String screenRuleNo,String screenRuleName){
        return astScreenRuleDOMapper.getSameRuleCnt(screenRuleNo, screenRuleName);
    }
}
