package com.abssqr.plat.common.dal.mysql.auto.dao;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import java.util.ArrayList;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.OrganizationDO;
import com.abssqr.plat.common.dal.mysql.auto.paging.OrgListPage;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.LinkedHashMap;
import java.util.ArrayList;
import java.util.HashSet;
import com.abssqr.plat.common.dal.mysql.auto.mapper.OrganizationDOMapper;

/**
* The Table abs_organization.
* 机构信息
*/
@Repository
public class OrganizationDAO{

    @Autowired
    private OrganizationDOMapper organizationDOMapper;

    /**
     * desc:insertOrUpdate.<br/>
     * @param entity entity
     * @return int
     */
    
    public int insert(OrganizationDO entity){
        return organizationDOMapper.insert(entity);
    }
    /**
     * desc:判断是否存在同名机构.<br/>
     * @param entity entity
     * @return Integer
     */
    
    public Integer getSameNameOrgCnt(OrganizationDO entity){
        return organizationDOMapper.getSameNameOrgCnt(entity);
    }
    /**
     * desc:根据唯一约束OrgCode更新表:abs_organization.<br/>
     * @param entity entity
     * @return int
     */
    
    public int updateByOrgCode(OrganizationDO entity){
        return organizationDOMapper.updateByOrgCode(entity);
    }
    /**
     * desc:根据唯一约束OrgCode获取数据:abs_organization.<br/>
     * @param orgCode orgCode
     * @return OrganizationDO
     */
    
    public OrganizationDO getByOrgCode(String orgCode){
        return organizationDOMapper.getByOrgCode(orgCode);
    }
    /**
     * desc:查询机构列表.<br/>
     * @param orgList orgList
     * @return OrgListPage
     */
    
    public OrgListPage queryOrgListForPage(OrgListPage orgList){
        int total = organizationDOMapper.queryOrgListForPageCount(orgList);
        if(total>0){
            orgList.setDatas(organizationDOMapper.queryOrgListForPageResult(orgList));
        }else{
            orgList.setDatas(new ArrayList());
        }
        orgList.setTotal(total);
        return orgList;
    }
     /**
     * desc:getOrgNames.<br/>
     * @param orgCodes orgCodes
     * @return Map<String,String>
     */
    
    public List<OrganizationDO> getOrgNamesList(List<String> orgCodes){
        return organizationDOMapper.getOrgNames(orgCodes);
    }
    /**
     * desc:getOrgNames.<br/>
     * @param orgCodes orgCodes
     * @return Map<String,String>
     */
    
    public Map<String,String> getOrgNames(List<String> orgCodes){
    if(orgCodes!=null && orgCodes.size()>201){throw new RuntimeException("orgCodes 超出201长度限制");}
        Map<String,String> result = new LinkedHashMap();
        List<OrganizationDO> resultList = organizationDOMapper.getOrgNames(orgCodes);
        if(resultList!=null && !resultList.isEmpty()){
            for(OrganizationDO entity:resultList){
                result.put(entity.getOrgCode(),entity.getOrgName());
            }
        }
        return result;
    }
}
