package com.abssqr.plat.common.dal.mysql.auto.dao;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import java.util.ArrayList;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanCapitalOrderDO;
import java.util.List;
import java.util.Date;
import java.lang.Long;
import com.abssqr.plat.common.dal.mysql.auto.paging.PlanCapitalOrderPage;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.CapitalOrderAmt;
import com.abssqr.plat.common.dal.mysql.auto.mapper.PlanCapitalOrderDOMapper;

/**
* The Table abs_plan_capital_order.
* 计划资金交易订单
*/
@Repository
public class PlanCapitalOrderDAO{

    @Autowired
    private PlanCapitalOrderDOMapper planCapitalOrderDOMapper;

    /**
     * desc:插入表:ABS_PLAN_CAPITAL_ORDER.<br/>
     * @param entity entity
     * @return int
     */
    
    public int insert(PlanCapitalOrderDO entity){
        return planCapitalOrderDOMapper.insert(entity);
    }
    /**
     * desc:插入表:ABS_PLAN_CAPITAL_ORDER.<br/>
     * @param entity entity
     * @return int
     */
    
    public int insertHis(PlanCapitalOrderDO entity){
        return planCapitalOrderDOMapper.insertHis(entity);
    }
    /**
     * desc:批量插入表:ABS_PLAN_CAPITAL_ORDER.<br/>
     * @param list list
     * @return int
     */
    
    public int insertBatch(List<PlanCapitalOrderDO> list){
        planCapitalOrderDOMapper.insertBatch(list);
        return  list.size(); 
    }
    /**
     * desc:根据唯一约束AbsPlanCapitalOrdr更新表:ABS_PLAN_CAPITAL_ORDER.<br/>
     * @param entity entity
     * @return int
     */
    
    public int updateProcess(PlanCapitalOrderDO entity){
        return planCapitalOrderDOMapper.updateProcess(entity);
    }
    /**
     * desc:失败的交易重新发起.<br/>
     * @param nextExecTime nextExecTime
     * @param seqNo seqNo
     * @return PlanCapitalOrderDO
     */
    
    public PlanCapitalOrderDO restoreCapitalOrder(Date nextExecTime,String seqNo){
        return planCapitalOrderDOMapper.restoreCapitalOrder(nextExecTime, seqNo);
    }
    /**
     * desc:根据唯一约束AbsPlanCapitalOrdr删除数据:ABS_PLAN_CAPITAL_ORDER.<br/>
     * @param seqNo seqNo
     * @param planNo planNo
     * @return int
     */
    
    public int deleteBySeqno(String seqNo,String planNo){
        return planCapitalOrderDOMapper.deleteBySeqno(seqNo, planNo);
    }
    /**
     * desc:根据唯一约束AbsPlanCapitalLog获取数据:abs_plan_capital_log.<br/>
     * @param seqNo seqNo
     * @return PlanCapitalOrderDO
     */
    
    public PlanCapitalOrderDO lockBySeqno(String seqNo){
        return planCapitalOrderDOMapper.lockBySeqno(seqNo);
    }
    /**
     * desc:根据唯一约束AbsPlanCapitalLog获取数据:abs_plan_capital_log.<br/>
     * @param accountDate accountDate
     * @param planNo planNo
     * @param transType transType
     * @return PlanCapitalOrderDO
     */
    
    public PlanCapitalOrderDO lockActiveByPlanNo(Date accountDate,String planNo,String transType){
        return planCapitalOrderDOMapper.lockActiveByPlanNo(accountDate, planNo, transType);
    }
    /**
     * desc:根据唯一约束AbsPlanCapitalLog获取数据:abs_plan_capital_log.<br/>
     * @param planNo planNo
     * @return List<PlanCapitalOrderDO>
     */
    
    public List<PlanCapitalOrderDO> getInitListByPlanNo(String planNo){
        return planCapitalOrderDOMapper.getInitListByPlanNo(planNo);
    }
    /**
     * desc:查询存活支付单条数.<br/>
     * @param accountDate accountDate
     * @param planNo planNo
     * @param transType transType
     * @param list list
     * @return Long
     */
    
    public Long getActiveCntByCondition(Date accountDate,String planNo,String transType,List<String> list){
    if(list!=null && list.size()>201){throw new RuntimeException("list 超出201长度限制");}
        return planCapitalOrderDOMapper.getActiveCntByCondition(accountDate, planNo, transType, list);
    }
    /**
     * desc:根据状态分页查询订单.<br/>
     * @param planCapitalOrder planCapitalOrder
     * @return PlanCapitalOrderPage
     */
    
    public PlanCapitalOrderPage getCapitalOrderByStatus(PlanCapitalOrderPage planCapitalOrder){
        int total = planCapitalOrderDOMapper.getCapitalOrderByStatusCount(planCapitalOrder);
        if(total>0){
            planCapitalOrder.setDatas(planCapitalOrderDOMapper.getCapitalOrderByStatusResult(planCapitalOrder));
        }else{
            planCapitalOrder.setDatas(new ArrayList());
        }
        planCapitalOrder.setTotal(total);
        return planCapitalOrder;
    }
    /**
     * desc:查询执行次数较多的交易单.<br/>
     * @param accountDate accountDate
     * @param execTimes execTimes
     * @param list list
     * @return List<PlanCapitalOrderDO>
     */
    
    public List<PlanCapitalOrderDO> queryFailCapitalOrder(Date accountDate,Long execTimes,List<String> list){
    if(list!=null && list.size()>201){throw new RuntimeException("list 超出201长度限制");}
        return planCapitalOrderDOMapper.queryFailCapitalOrder(accountDate, execTimes, list);
    }
    /**
     * desc:查询失败数量.<br/>
     * @param execTimes execTimes
     * @return Long
     */
    
    public Long getFailCount(Long execTimes){
        return planCapitalOrderDOMapper.getFailCount(execTimes);
    }
    /**
     * desc:根据外部交易编码查询交易单用来做mock数据.<br/>
     * @param status status
     * @param noList noList
     * @return List<PlanCapitalOrderDO>
     */
    
    public List<PlanCapitalOrderDO> queryOrderListByOutAcptNo(String status,List<String> noList){
    if(noList!=null && noList.size()>201){throw new RuntimeException("noList 超出201长度限制");}
        return planCapitalOrderDOMapper.queryOrderListByOutAcptNo(status, noList);
    }
    /**
     * desc:根据计划编号查询支付金额和交易金额.<br/>
     * @param accountDate accountDate
     * @param planNo planNo
     * @param transType transType
     * @return CapitalOrderAmt
     */
    
    public CapitalOrderAmt getHisAmtByPlanNoAndActDateAndTransCode(Date accountDate,String planNo,String transType){
        return planCapitalOrderDOMapper.getHisAmtByPlanNoAndActDateAndTransCode(accountDate, planNo, transType);
    }
}
