package com.abssqr.plat.common.dal.mysql.auto.dao;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import java.util.ArrayList;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanDO;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.LinkedHashMap;
import java.util.ArrayList;
import java.util.HashSet;
import com.abssqr.plat.common.dal.mysql.auto.paging.PlanListPage;
import java.lang.Long;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.AstServiceReport;
import java.util.Date;
import com.abssqr.plat.common.dal.mysql.auto.mapper.PlanDOMapper;

/**
* The Table abs_plan.
* 计划信息
*/
@Repository
public class PlanDAO{

    @Autowired
    private PlanDOMapper planDOMapper;

    /**
     * desc:插入表:abs_plan.<br/>
     * @param entity entity
     * @return int
     */
    
    public int insert(PlanDO entity){
        return planDOMapper.insert(entity);
    }
    /**
     * desc:根据唯一约束PlanNo更新表:abs_plan.<br/>
     * @param entity entity
     * @return int
     */
    
    public int updateByPlanNo(PlanDO entity){
        return planDOMapper.updateByPlanNo(entity);
    }
    /**
     * desc:根据唯一约束PlanNo更新表:abs_plan.<br/>
     * @param planNo planNo
     * @param status status
     * @param gmtModifier gmtModifier
     * @param approveStatus approveStatus
     * @return int
     */
    
    public int updateStatusByPlanNo(String planNo,String status,String gmtModifier,String approveStatus){
        return planDOMapper.updateStatusByPlanNo(planNo, status, gmtModifier, approveStatus);
    }
    /**
     * desc:根据唯一约束PlanNo修改状态:abs_plan.<br/>
     * @param planNo planNo
     * @param passed passed
     * @return int
     */
    
    public int updatePassed(String planNo,Boolean passed){
        return planDOMapper.updatePassed(planNo, passed);
    }
    /**
     * desc:根据唯一约束PlanNo删除数据:abs_plan.<br/>
     * @param planNo planNo
     * @return int
     */
    
    public int deleteByPlanNo(String planNo){
        return planDOMapper.deleteByPlanNo(planNo);
    }
    /**
     * desc:根据唯一约束PlanNo获取数据:abs_plan.<br/>
     * @param planNo planNo
     * @return PlanDO
     */
    
    public PlanDO getByPlanNo(String planNo){
        return planDOMapper.getByPlanNo(planNo);
    }
    /**
     * desc:根据唯一约束PlanNo获取数据:abs_plan.<br/>
     * @param planNo planNo
     * @return PlanDO
     */
    
    public PlanDO lockByPlanNo(String planNo){
        return planDOMapper.lockByPlanNo(planNo);
    }
     /**
     * desc:根据唯一约束PlanNo获取数据:abs_plan.<br/>
     * @param list list
     * @return Map<String,String>
     */
    
    public List<PlanDO> queryByPlanNoList(List<String> list){
        return planDOMapper.queryByPlanNo(list);
    }
    /**
     * desc:根据唯一约束PlanNo获取数据:abs_plan.<br/>
     * @param list list
     * @return Map<String,String>
     */
    
    public Map<String,String> queryByPlanNo(List<String> list){
    if(list!=null && list.size()>201){throw new RuntimeException("list 超出201长度限制");}
        Map<String,String> result = new LinkedHashMap();
        List<PlanDO> resultList = planDOMapper.queryByPlanNo(list);
        if(resultList!=null && !resultList.isEmpty()){
            for(PlanDO entity:resultList){
                result.put(entity.getPlanNo(),entity.getPlanName());
            }
        }
        return result;
    }
    /**
     * desc:新增判断计划是否存在.<br/>
     * @param planNo planNo
     * @param planName planName
     * @return Integer
     */
    
    public Integer planExist(String planNo,String planName){
        return planDOMapper.planExist(planNo, planName);
    }
    /**
     * desc:编辑判断计划是否存在.<br/>
     * @param planNo planNo
     * @param planName planName
     * @return Integer
     */
    
    public Integer getSamePlan(String planNo,String planName){
        return planDOMapper.getSamePlan(planNo, planName);
    }
    /**
     * desc:分页查询计划.<br/>
     * @param planList planList
     * @return PlanListPage
     */
    
    public PlanListPage getPlanList(PlanListPage planList){
        int total = planDOMapper.getPlanListCount(planList);
        if(total>0){
            planList.setDatas(planDOMapper.getPlanListResult(planList));
        }else{
            planList.setDatas(new ArrayList());
        }
        planList.setTotal(total);
        return planList;
    }
    /**
     * desc:根据状态数组获取计划列表.<br/>
     * @param planType planType
     * @param planNos planNos
     * @param statusList statusList
     * @return List<PlanDO>
     */
    
    public List<PlanDO> getPlansByStatusList(String planType,List<String> planNos,List<String> statusList){
    if(planNos!=null && planNos.size()>201){throw new RuntimeException("planNos 超出201长度限制");}
    if(statusList!=null && statusList.size()>201){throw new RuntimeException("statusList 超出201长度限制");}
        return planDOMapper.getPlansByStatusList(planType, planNos, statusList);
    }
    /**
     * desc:根据状态数组获取计划列表.<br/>
     * @param status status
     * @param planType planType
     * @return Long
     */
    
    public Long getAllPlanAmt(String status,String planType){
        return planDOMapper.getAllPlanAmt(status, planType);
    }
    /**
     * desc:查询资产服务报告.<br/>
     * @param planNo planNo
     * @return AstServiceReport
     */
    
    public AstServiceReport getAstServiceReport(String planNo){
        return planDOMapper.getAstServiceReport(planNo);
    }
    /**
     * desc:根据服务机构获取存续期受让计划.<br/>
     * @param managerOrgCode managerOrgCode
     * @return List<PlanDO>
     */
    
    public List<PlanDO> getSpvByManagerOrgCode(String managerOrgCode){
        return planDOMapper.getSpvByManagerOrgCode(managerOrgCode);
    }
    /**
     * desc:查询每日任务失败的数量.<br/>
     * @param curDate curDate
     * @return Long
     */
    
    public Long getFailDailyCount(Date curDate){
        return planDOMapper.getFailDailyCount(curDate);
    }
    /**
     * desc:更新定价规则.<br/>
     * @param planNo planNo
     * @param priceType priceType
     * @return int
     */
    
    public int updatePriceType(String planNo,String priceType){
        return planDOMapper.updatePriceType(planNo, priceType);
    }
    /**
     * desc:根据类型获取所有的计划.<br/>
     * @param planType planType
     * @return List<PlanDO>
     */
    
    public List<PlanDO> getAllPlanByType(String planType){
        return planDOMapper.getAllPlanByType(planType);
    }
}
