/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.facade.model;

import com.abssqr.plat.common.facade.enums.CalcIntrTypeEnum;
import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.enums.FundTypeEnum;
import com.abssqr.plat.common.facade.enums.PlanPayFreqEnum;
import com.abssqr.plat.common.facade.enums.PlanStatusEnum;
import com.abssqr.plat.common.facade.model.account.Account;
import com.abssqr.plat.common.facade.model.plan.FeeStructure;
import com.abssqr.plat.common.facade.model.plan.InvestStructure;
import com.general.system.bpm.enums.ApproveStatusEnum;
import com.general.system.common.model.Money;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * 放款池计划 与 承接池计划 抽象模型
 *
 * @author hanfei
 * @version $Id: CommonPlanEntity.java, v 0.1 2019-08-26 4:23 PM hanfei Exp $
 */
public class CommonPlanEntity {
    /**
     * PLAN("PLAN", "计划"),
     * SPV("SPV", "SPV"),
     */
    private PlanTypeEnum entityType;
    /**
     * totalAmount 总投入资本 发行总规模.
     */
    private Money        totalAmt;
    /**
     * planNo 计划编号.
     */
    private String       entityNo;
    /**
     * planName 计划名称.
     */
    private String       entityName;
    /**
     * beginDate 计划设立日（信托设立日）.
     */
    private Date         beginDate;
    /**
     * effectDate 计划起息日.
     */
    private Date         effectDate;
    /**
     * 增值税率
     */
    private BigDecimal   vatRate;
    /**
     * 募集方式
     * PUBLIC
     * PRIVATE
     * 公募
     * 私募.
     */
    private FundTypeEnum fundType;
    /**
     * initDate 初始起算日.
     */
    private Date         initCalcDate;
    /**
     * 计划状态
     * PREPARE
     * CLEARED
     * SUBSIST
     * 准备中
     * 已清算
     * 存续中.
     */
    private PlanStatusEnum    status;
    /**
     * planApproveStatus 计划审批状态
     * OUTOFDATE
     * PASSED
     * FAILED
     * PENDING
     * 已过时
     * 已通过
     * 已失败
     * 审批中.
     */
    private ApproveStatusEnum approveStatus;
    /**
     * 计息方式
     * ACT/360
     * ACT/365
     */
    private CalcIntrTypeEnum  calcIntrType;
    /**
     * endDate 计划到期日（信托终止日）计划截止日.
     */
    private Date              endDate;
    /**
     * firstPayDay 首次结算日.
     */
    private Date                  firstClearDate;
    /**
     * firstPayDay 首次偿付日 首次兑付日.
     */
    private Date                  firstPayDate;
    /**
     * stopTrDate 停止转让日 持续购买终止日,放款日.
     */
    private Date                  stopTrPayDate;
    /**
     * 兑付频率
     * M
     * Q
     * 半年
     * Y
     * Month
     * Season
     * HalfYear
     * Year
     */
    private PlanPayFreqEnum       payFreq;
    /**
     * 账户信息
     */
    private List<Account>         accounts;
    /**
     * 投资结构
     */
    private List<InvestStructure> investStructureList;
    /**
     * 费用结构 费用清单
     */
    private List<FeeStructure>    feeStructureList;
    /**
     * 最近更新日期
     */
    private Date                  lastUpdateDate;

    public String getEntityNo() {
        return entityNo;
    }

    public void setEntityNo(String entityNo) {
        this.entityNo = entityNo;
    }

    public String getEntityName() {
        return entityName;
    }

    public void setEntityName(String entityName) {
        this.entityName = entityName;
    }

    public Date getEffectDate() {
        return effectDate;
    }

    public void setEffectDate(Date effectDate) {
        this.effectDate = effectDate;
    }

    public PlanStatusEnum getStatus() {
        return status;
    }

    public void setStatus(PlanStatusEnum status) {
        this.status = status;
    }

    public ApproveStatusEnum getApproveStatus() {
        return approveStatus;
    }

    public void setApproveStatus(ApproveStatusEnum approveStatus) {
        this.approveStatus = approveStatus;
    }

    public CalcIntrTypeEnum getCalcIntrType() {
        return calcIntrType;
    }

    public void setCalcIntrType(CalcIntrTypeEnum calcIntrType) {
        this.calcIntrType = calcIntrType;
    }

    public PlanTypeEnum getEntityType() {
        return entityType;
    }

    public void setEntityType(PlanTypeEnum entityType) {
        this.entityType = entityType;
    }

    public Date getLastUpdateDate() {
        return lastUpdateDate;
    }

    public void setLastUpdateDate(Date lastUpdateDate) {
        this.lastUpdateDate = lastUpdateDate;
    }

    public BigDecimal getVatRate() {
        return vatRate;
    }

    public void setVatRate(BigDecimal vatRate) {
        this.vatRate = vatRate;
    }

    public Date getBeginDate() {
        return beginDate;
    }

    public void setBeginDate(Date beginDate) {
        this.beginDate = beginDate;
    }

    public Date getEndDate() {
        return endDate;
    }

    public void setEndDate(Date endDate) {
        this.endDate = endDate;
    }

    public Date getFirstClearDate() {
        return firstClearDate;
    }

    public void setFirstClearDate(Date firstClearDate) {
        this.firstClearDate = firstClearDate;
    }

    public Date getFirstPayDate() {
        return firstPayDate;
    }

    public void setFirstPayDate(Date firstPayDate) {
        this.firstPayDate = firstPayDate;
    }

    public PlanPayFreqEnum getPayFreq() {
        return payFreq;
    }

    public void setPayFreq(PlanPayFreqEnum payFreq) {
        this.payFreq = payFreq;
    }

    public List<Account> getAccounts() {
        return accounts;
    }

    public void setAccounts(List<Account> accounts) {
        this.accounts = accounts;
    }

    public List<InvestStructure> getInvestStructureList() {
        return investStructureList;
    }

    public void setInvestStructureList(
        List<InvestStructure> investStructureList) {
        this.investStructureList = investStructureList;
    }

    public List<FeeStructure> getFeeStructureList() {
        return feeStructureList;
    }

    public void setFeeStructureList(List<FeeStructure> feeStructureList) {
        this.feeStructureList = feeStructureList;
    }

    public Date getStopTrPayDate() {
        return stopTrPayDate;
    }

    public void setStopTrPayDate(Date stopTrPayDate) {
        this.stopTrPayDate = stopTrPayDate;
    }

    public Money getTotalAmt() {
        return totalAmt;
    }

    public void setTotalAmt(Money totalAmt) {
        this.totalAmt = totalAmt;
    }

    public FundTypeEnum getFundType() {
        return fundType;
    }

    public void setFundType(FundTypeEnum fundType) {
        this.fundType = fundType;
    }

    public Date getInitCalcDate() {
        return initCalcDate;
    }

    public void setInitCalcDate(Date initCalcDate) {
        this.initCalcDate = initCalcDate;
    }
}
