/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.facade.param.ast;

import com.abssqr.plat.common.facade.base.BaseOprParam;
import com.abssqr.plat.common.facade.enums.EventCodeEnum;
import com.abssqr.plat.common.facade.enums.ScreenTypeEnum;
import com.abssqr.plat.common.facade.param.ast.astScreenRuleArea.RuleArea;
import com.abssqr.plat.common.facade.validation.VldString;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

/**
 * @author bangis.wangdf
 * @version com.abssqr.plat.common.facade.param.ast: AstScreenRuleEditParam.java, v 0.1 2019-06-02 20:19 bangis.wangdf Exp $
 */
public class AstScreenRuleEditParam extends BaseOprParam {
    private static final long serialVersionUID = -2081223545779548054L;

    /**
     * screenRuleNo 资产筛选规则编号.
     */
    private String screenRuleNo;

    /**
     * screenRuleName 资产筛选规则名.
     */
    @VldString(message = "资产筛选规则名不能为空")
    private String screenRuleName;

    /**
     * singleMaxAmount 单笔最大金额.
     */
    private Long           singleMaxAmount;
    /**
     * singleMinAmount 单笔最小金额.
     */
    private Long           singleMinAmount;
    /**
     * region 地区.
     */
    private List<RuleArea> region;
    /**
     * lenderOrgCode 原始权益机构.
     */
    @VldString(message = "原始权益机构不能为空")
    private String         lenderOrgCode;
    /**
     * abssqrProdCode 内部产品码.
     */
    @VldString(message = "产品不能为空")
    private String         abssqrProdCode;
    /**
     * assetStatus 资产状态,|分割，对应五级分类.
     */
    private String         assetStatus;

    /**
     * maxAge 最大年龄.
     */
    private Integer    maxAge;
    /**
     * minAge 最小年龄.
     */
    private Integer    minAge;
    /**
     * maxTerm 最大期次.
     */
    private Integer    maxTerm;
    /**
     * minTerm 最小期次.
     */
    private Integer    minTerm;
    /**
     * maxRemainDays 最大剩余天数.
     */
    private Integer    maxRemainDays;
    /**
     * minRemainDays 最小剩余天数.
     */
    private Integer    minRemainDays;
    /**
     * latestEndDate 最晚到期日.
     */
    private Date       latestEndDate;
    /**
     * earliestEndDate 最早到期日.
     */
    private Date       earliestEndDate;
    /**
     * singleMaxInterRate 单笔最高利率.
     */
    private BigDecimal singleMaxInterRate;
    /**
     * singleMinInterRate 单笔最单利率.
     */
    private BigDecimal singleMinInterRate;
    /**
     * slice 是否切片
     */
    private Boolean    slice;

    /**
     * 筛选类型
     */
    private String screenType = ScreenTypeEnum.AST_TRF.getCode();

    @Override
    public EventCodeEnum getEventCode() {
        return EventCodeEnum.ADMIN_EDIT;
    }

    public String getScreenRuleNo() {
        return screenRuleNo;
    }

    public void setScreenRuleNo(String screenRuleNo) {
        this.screenRuleNo = screenRuleNo;
    }

    public String getScreenRuleName() {
        return screenRuleName;
    }

    public void setScreenRuleName(String screenRuleName) {
        this.screenRuleName = screenRuleName;
    }

    public Long getSingleMaxAmount() {
        return singleMaxAmount;
    }

    public void setSingleMaxAmount(Long singleMaxAmount) {
        this.singleMaxAmount = singleMaxAmount;
    }

    public Long getSingleMinAmount() {
        return singleMinAmount;
    }

    public void setSingleMinAmount(Long singleMinAmount) {
        this.singleMinAmount = singleMinAmount;
    }

    public List<RuleArea> getRegion() {
        return region;
    }

    public void setRegion(List<RuleArea> region) {
        this.region = region;
    }

    public String getLenderOrgCode() {
        return lenderOrgCode;
    }

    public void setLenderOrgCode(String lenderOrgCode) {
        this.lenderOrgCode = lenderOrgCode;
    }

    public String getAbssqrProdCode() {
        return abssqrProdCode;
    }

    public void setAbssqrProdCode(String abssqrProdCode) {
        this.abssqrProdCode = abssqrProdCode;
    }

    public String getAssetStatus() {
        return assetStatus;
    }

    public void setAssetStatus(String assetStatus) {
        this.assetStatus = assetStatus;
    }

    public Integer getMaxAge() {
        return maxAge;
    }

    public void setMaxAge(Integer maxAge) {
        this.maxAge = maxAge;
    }

    public Integer getMinAge() {
        return minAge;
    }

    public void setMinAge(Integer minAge) {
        this.minAge = minAge;
    }

    public Integer getMaxTerm() {
        return maxTerm;
    }

    public void setMaxTerm(Integer maxTerm) {
        this.maxTerm = maxTerm;
    }

    public Integer getMinTerm() {
        return minTerm;
    }

    public void setMinTerm(Integer minTerm) {
        this.minTerm = minTerm;
    }

    public Integer getMaxRemainDays() {
        return maxRemainDays;
    }

    public void setMaxRemainDays(Integer maxRemainDays) {
        this.maxRemainDays = maxRemainDays;
    }

    public Integer getMinRemainDays() {
        return minRemainDays;
    }

    public void setMinRemainDays(Integer minRemainDays) {
        this.minRemainDays = minRemainDays;
    }

    public Date getLatestEndDate() {
        return latestEndDate;
    }

    public void setLatestEndDate(Date latestEndDate) {
        this.latestEndDate = latestEndDate;
    }

    public Date getEarliestEndDate() {
        return earliestEndDate;
    }

    public void setEarliestEndDate(Date earliestEndDate) {
        this.earliestEndDate = earliestEndDate;
    }

    public BigDecimal getSingleMaxInterRate() {
        return singleMaxInterRate;
    }

    public void setSingleMaxInterRate(BigDecimal singleMaxInterRate) {
        this.singleMaxInterRate = singleMaxInterRate;
    }

    public BigDecimal getSingleMinInterRate() {
        return singleMinInterRate;
    }

    public void setSingleMinInterRate(BigDecimal singleMinInterRate) {
        this.singleMinInterRate = singleMinInterRate;
    }

    public Boolean getSlice() {
        return slice;
    }

    public void setSlice(Boolean slice) {
        this.slice = slice;
    }

    public String getScreenType() {
        return screenType;
    }

    public void setScreenType(String screenType) {
        this.screenType = screenType;
    }
}
