/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.common.facade.result.plan;

import com.abssqr.plat.common.facade.constants.AbssqrConstants;
import com.abssqr.plat.common.facade.enums.FundTypeEnum;
import com.abssqr.plat.common.facade.enums.PlanAllocStatusEnum;
import com.abssqr.plat.common.facade.model.plan.PlanAlloc;
import com.abssqr.plat.common.facade.model.plan.PlanAllocItem;
import com.abssqr.plat.common.facade.model.plan.PlanAst;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.result.ApproveBaseEntity;
import com.general.system.common.model.Money;

import java.util.Date;
import java.util.List;

/**
 *
 * @author hanfei
 * @version $Id: PlanAllocVO.java, v 0.1 2019-06-26 11:26 AM hanfei Exp $ 
 */
public class PlanAllocVO extends ApproveBaseEntity {
    
    private static final long   serialVersionUID = 6904901111954362632L;
    /**
     * planNo 计划编号.
     */
    private String              planNo;
    /**
     * planName 计划名称.
     */
    private String              planName;
    /**
     * totalAmount 总投入资本 发行总规模.
     */
    private Money               totalAmt;
    /**
     * beginDate 计划设立日（信托设立日）.
     */
    private Date                beginDate;
    /**
     * endDate 计划到期日（信托终止日）计划截止日.
     */
    private Date                endDate;
    /**
     * 募集方式
     * PUBLIC
     * PRIVATE
     * 公募
     * 私募.
     */
    private FundTypeEnum        fundType;
    /**
     * initDate 初始起算日.
     */
    private Date                initCalcDate;
    /**
     * effectDate 计划起息日.
     */
    private Date                effectDate;
    /**
     * firstPayDay 首次结算日.
     */
    private Date                firstClearDate;
    /**
     * firstPayDay 首次偿付日 首次兑付日.
     */
    private Date                firstPayDate;
    /**
     * allocNo 分配编号.
     */
    private String              allocNo;
    /**
     * clearDate 结算日.
     */
    private Date                clearDate;
    /**
     * payDate 兑付日.
     */
    private Date                payDate;
    /**
     * 分配状态
     */
    private PlanAllocStatusEnum status;
    /**
     * allocAmt 本期分配总额.
     */
    private Money allocAmt = new Money();
    /**
     * 累计转让规模  水位中获取
     */
    private Money accAmt = new Money();
    /**
     * 剩余累计规模
     */
    private Money remainAmt = new Money();
    /**
     * 当前可用余额   水位中获取
     */
    private Money balAmt = new Money();
    
    /**
     * approveStatus 审批状态
     * OUTOFDATE("OUTOFDATE", "已过时"),
     * PASSED("PASSED", "已通过"),
     * FAILED("FAILED", "未通过"),
     * PENDING("PENDING", "审批中"),
     * UNSUBMIT("UNSUBMIT", "未提交"),
     * UNWANTED("UNWANTED", "无审批"),.
     */
    private String              approveStatus;
    /**
     * 分配事项
     */
    private List<PlanAllocItem> planAllocItemList;
    /**
     * 是否有提交按钮. Y 有提交， N 无提交
     */
    private String              submit = AbssqrConstants.NO_FLAG;
    /**
     * 是否有 试算 按钮. Y 有， N 无
     */
    private String              calc = AbssqrConstants.NO_FLAG;
    /**
     * 是否可支付 按钮. Y 有， N 无
     */
    private String              pay = AbssqrConstants.NO_FLAG;
    /**
     * 是否为第一期
     */
    private boolean     first;
    
    public void convert(PlanBase plan, PlanAst planAst , PlanAlloc alloc) {
        this.setBalAmt(planAst.containClearAbleBal());
        this.setAccAmt(planAst.getAccGrant());
        this.setPlanNo(plan.getPlanNo());
        this.setPlanName(plan.getPlanName());
        this.setTotalAmt(plan.getTotalAmt());
        this.setBeginDate(plan.getBeginDate());
        this.setEndDate(plan.getEndDate());
        this.setFundType(plan.getFundType());
        this.setInitCalcDate(plan.getInitCalcDate());
        this.setEffectDate(plan.getEffectDate());
        this.setFirstClearDate(plan.getFirstClearDate());
        this.setFirstPayDate(plan.getFirstPayDate());
        this.setAllocNo(alloc.getAllocNo());
        this.setClearDate(alloc.getClearDate());
        this.setPayDate(alloc.getPayDate());
        this.setStatus(alloc.getStatus());
        this.setAllocAmt(alloc.getAllocAmt());
        this.setPlanAllocItemList(alloc.getPlanAllocItemList());
        if (alloc.getApproveStatus()!=null){
            this.setApproveStatus(alloc.getApproveStatus().getCode());
        }
    }
    
    public String getPlanNo() {
        return planNo;
    }
    
    public void setPlanNo(String planNo) {
        this.planNo = planNo;
    }
    
    public String getPlanName() {
        return planName;
    }
    
    public void setPlanName(String planName) {
        this.planName = planName;
    }
    
    public Money getTotalAmt() {
        return totalAmt;
    }
    
    public void setTotalAmt(Money totalAmt) {
        this.totalAmt = totalAmt;
    }
    
    public Date getBeginDate() {
        return beginDate;
    }
    
    public void setBeginDate(Date beginDate) {
        this.beginDate = beginDate;
    }
    
    public Date getEndDate() {
        return endDate;
    }
    
    public void setEndDate(Date endDate) {
        this.endDate = endDate;
    }
    
    public FundTypeEnum getFundType() {
        return fundType;
    }
    
    public void setFundType(FundTypeEnum fundType) {
        this.fundType = fundType;
    }
    
    public Date getInitCalcDate() {
        return initCalcDate;
    }
    
    public void setInitCalcDate(Date initCalcDate) {
        this.initCalcDate = initCalcDate;
    }
    
    public Date getEffectDate() {
        return effectDate;
    }
    
    public void setEffectDate(Date effectDate) {
        this.effectDate = effectDate;
    }
    
    public Date getFirstClearDate() {
        return firstClearDate;
    }
    
    public void setFirstClearDate(Date firstClearDate) {
        this.firstClearDate = firstClearDate;
    }
    
    public Date getFirstPayDate() {
        return firstPayDate;
    }
    
    public void setFirstPayDate(Date firstPayDate) {
        this.firstPayDate = firstPayDate;
    }
    
    public String getAllocNo() {
        return allocNo;
    }
    
    public void setAllocNo(String allocNo) {
        this.allocNo = allocNo;
    }
    
    public Date getClearDate() {
        return clearDate;
    }
    
    public void setClearDate(Date clearDate) {
        this.clearDate = clearDate;
    }
    
    public Date getPayDate() {
        return payDate;
    }
    
    public void setPayDate(Date payDate) {
        this.payDate = payDate;
    }
    
    public PlanAllocStatusEnum getStatus() {
        return status;
    }
    
    public void setStatus(PlanAllocStatusEnum status) {
        this.status = status;
    }
    
    public Money getAllocAmt() {
        return allocAmt;
    }
    
    public void setAllocAmt(Money allocAmt) {
        this.allocAmt = allocAmt;
    }
    
    public Money getAccAmt() {
        return accAmt;
    }
    
    public void setAccAmt(Money accAmt) {
        this.accAmt = accAmt;
    }
    
    public Money getRemainAmt() {
        return remainAmt;
    }
    
    public void setRemainAmt(Money remainAmt) {
        this.remainAmt = remainAmt;
    }
    
    public Money getBalAmt() {
        return balAmt;
    }
    
    public void setBalAmt(Money balAmt) {
        this.balAmt = balAmt;
    }
    
    public String getApproveStatus() {
        return approveStatus;
    }
    
    public void setApproveStatus(String approveStatus) {
        this.approveStatus = approveStatus;
    }
    
    public List<PlanAllocItem> getPlanAllocItemList() {
        return planAllocItemList;
    }
    
    public void setPlanAllocItemList(List<PlanAllocItem> planAllocItemList) {
        this.planAllocItemList = planAllocItemList;
    }
    
    public String getSubmit() {
        return submit;
    }
    
    public void setSubmit(String submit) {
        this.submit = submit;
    }
    
    public String getCalc() {
        return calc;
    }
    
    public void setCalc(String calc) {
        this.calc = calc;
    }
    
    public String getPay() {
        return pay;
    }
    
    public void setPay(String pay) {
        this.pay = pay;
    }
    
    public boolean isFirst() {
        return first;
    }
    
    public void setFirst(boolean first) {
        this.first = first;
    }
}
