/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2020 All Rights Reserved. 
 */
package com.abssqr.plat.common.model.convertor;

import com.abssqr.plat.common.dal.mysql.auto.dataobject.AcctDateDO;
import com.abssqr.plat.common.model.domain.accounting.AcctDate;
import com.abssqr.plat.common.model.enums.OrgCodeEnum;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;

import java.util.List;

/**
 * 会计日转换
 * @author hanfei
 * @version $Id: AcctDateConvert.java, v 0.1 2020-03-06 11:17 AM hanfei Exp $ 
 */
public class AcctDateConvert {
    
    public static List<AcctDateDO> convert2DOList(List<AcctDate> param) {
        List<AcctDateDO> result = Lists.newArrayList();
        if (param == null) {
            return null;
        }
        param.forEach(item -> result.add(convert2DO(item)));
        return result;
    }
    
    public static List<AcctDate> convert2DomainList(List<AcctDateDO> param) {
        List<AcctDate> result = Lists.newArrayList();
        if (param == null) {
            return null;
        }
        param.forEach(item -> result.add(convert2Domain(item)));
        return result;
    }
    
    public static AcctDateDO convert2DO(AcctDate param) {
        if (param == null) {
            return null;
        }
        VarChecker.checkNotNull(param.getOrgCode(), "不匹配的机构编码={0}", param);
        AcctDateDO result = new AcctDateDO();
        result.setOrgCode(param.getOrgCode().getCode());
        result.setOrgType(param.getOrgCode().getType().getCode());
        result.setEff(param.getEff());
        result.setAcctDate(param.getAcctDate());
        return result;
    }
    
    public static AcctDate convert2Domain(AcctDateDO param) {
        if (param == null) {
            return null;
        }
        OrgCodeEnum org = OrgCodeEnum.getByCode(param.getOrgCode());
        VarChecker.checkNotNull(org, "不匹配的机构编码={0}", org);
        AcctDate result = new AcctDate();
        result.setOrgCode(org);
        result.setAcctDate(param.getAcctDate());
        result.setEff(param.getEff());
        return result;
    }
}
