/**
 * abssqr.comInc.
 * Copyright(c)2019-2019AllRightsReserved.
 */
package com.abssqr.plat.common.model.convertor;

import com.abssqr.plat.common.facade.enums.*;
import com.abssqr.plat.common.facade.model.account.Account;
import com.abssqr.plat.common.facade.model.org.OrganizationEntity;
import com.abssqr.plat.common.facade.model.plan.Plan;
import com.abssqr.plat.common.facade.model.plan.PlanAstPriceRule;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.model.plan.PlanTransferRule;
import com.abssqr.plat.common.facade.model.rule.AstScreenRule;
import com.abssqr.plat.common.facade.param.acct.AccountParam;
import com.abssqr.plat.common.facade.param.org.OrgEntityOprParam;
import com.abssqr.plat.common.facade.param.plan.PlanOprParam;
import com.abssqr.plat.common.facade.param.plan.sub.PlanAstPriceRuleParam;
import com.abssqr.plat.common.facade.param.plan.sub.PlanTransferRuleParam;
import com.abssqr.plat.common.facade.utils.CalcUtil;
import com.abssqr.plat.common.model.enums.IdTypeEnum;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import com.general.system.common.model.Money;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;

import java.util.List;
import java.util.Optional;

/**
 * @author yangcheng
 * @version PlanParamConvert.java, v0.12019-06-1416:50 yangchengExp$
 */
public class PlanParamConvert {

    public static PlanBase convert2PlanBase(PlanOprParam param) {
        PlanBase result = null;
        if (PlanTypeEnum.getByCode(param.getPlanType()) == PlanTypeEnum.SPV) {
            result = new OrganizationEntity();
        } else {
            result = new Plan();
        }

        result.setPlanType(PlanTypeEnum.getByCode(param.getPlanType()));
        //totalAmt 发行规模
        result.setTotalAmt(Optional.ofNullable(param.getTotalAmt()).map(Money::new).map((item) -> item.multiply(10000L)).orElse(null));
        //planNo 计划编号
        result.setPlanNo(param.getPlanNo().trim());
        //fundType 募集方式
        result.setFundType(FundTypeEnum.getByCode(param.getFundType()));
        //planName 计划名
        result.setPlanName(param.getPlanName().trim());
        //managerOrgCode 管理机构
        result.setPlanMgrOrgCode(param.getPlanMgrOrgCode());
        //sponsorOrgCode 发起机构
        result.setSponsorOrgCode(param.getSponsorOrgCode());
        //endDate 计划到期日（信托终止日）计划截止日
        result.setEndDate(param.getEndDate());
        //beginDate 计划设立日（信托设立日）
        result.setBeginDate(param.getBeginDate());
        //effectDate 计划起息日
        result.setEffectDate(param.getEffectDate());
        //firstPayDate 首次偿付日 首次兑付日
        result.setFirstPayDate(param.getFirstPayDate());
        //initCalcDate 初始起算日
        result.setInitCalcDate(param.getInitCalcDate());
        //firstClearDate 首次结算日
        result.setFirstClearDate(param.getFirstClearDate());
        //payFrequency 兑付频率
        result.setPayFreq(PlanPayFreqEnum.getByCode(param.getPayFreq()));
        //countCalcType 计息方式
        result.setCalcIntrType(CalcIntrTypeEnum.getByCode(param.getCalcIntrType()));
        //vatRate 增指税率
        result.setVatRate(CalcUtil.calcRealRate(param.getVatRate()));
        //status 计划状态
        if (result.getStatus() == null) {
            result.setStatus(PlanStatusEnum.PREPARING);
        }
        if (result.getPlanType() == PlanTypeEnum.SPV && result instanceof OrganizationEntity) {
            OrganizationEntity entity = (OrganizationEntity) result;
            //用户转换
            entity.setAccounts(convert2Account(param.getAccounts()));
            //结构
            entity.setInvestStructureList(
                    InvestStructureConvert.convert2InvestStructureList(param.getInvestStructureList()));
            //费用
            entity.setFeeStructureList(FeeStructureConvert.convert2FeeStructureList(param.getFeeStructureList()));
            OrgEntityOprParam entityQryParam = (OrgEntityOprParam) param;
            entity.setNeedAlloc(YNEnum.getByCode(entityQryParam.getNeedAlloc()));
            entity.setProdCodeList(entityQryParam.getProdCodes());
            entity.setStopPayDate(entityQryParam.getStopPayDate());
        } else {
            Plan plan = (Plan) result;
            //转让规则转换
            plan.setPlanTransferRule(convert2TransferRule(param.getPlanNo(), param.getTransferRule()));
            //用户转换
            plan.setAccounts(convert2Account(param.getAccounts()));
            //结构
            plan.setInvestStructureList(
                    InvestStructureConvert.convert2InvestStructureList(param.getInvestStructureList()));
            //费用
            plan.setFeeStructureList(FeeStructureConvert.convert2FeeStructureList(param.getFeeStructureList()));
            // 定价类型
            plan.setPriceType(PriceTypeEnum.getByCode(param.getPriceType()));
            // 定价规则列表
            plan.setPlanAstPriceRuleList(convert2PlanAstPriceRuleList(param.getPlanAstPriceRuleList()));
        }
        result.setFinSerFeeRate(CalcUtil.calcRealRate(param.getFinSerFeeRate()));
        return result;
    }


    private static List<Account> convert2Account(List<AccountParam> accounts) {
        List<Account> result = Lists.newArrayList();
        if (CollectionUtils.isNotEmpty(accounts)) {
            for (AccountParam account : accounts) {
                Account mdl = new Account();
                mdl.setActAttr(AccountAttributeEnum.getByCode(account.getActAttr()));
                mdl.setActBank(account.getActBank());
                mdl.setActCity(account.getActCity());
                mdl.setActName(account.getActName());
                mdl.setActNo(account.getActNo());
                mdl.setActProv(account.getActProv());
                mdl.setActType(AccountTypeEnum.getByCode(account.getActType()));
                mdl.setChannelNo(account.getChannelNo());
                result.add(mdl);
            }

        }
        return result;
    }

    private static PlanTransferRule convert2TransferRule(String planNo, PlanTransferRuleParam param) {

        if (param == null) {
            return null;
        }
        PlanTransferRule result = new PlanTransferRule();
        result.setPlanNo(planNo);
        //transferNo 转让规则       自己产生
        result.setTransferNo(param.getTransferNo());
        if (StringUtils.isBlank(result.getTransferNo())) {
            result.setTransferNo(SequenceUtil.genId(IdTypeEnum.TRANSFERRULE));
        }
        //resAmt 预留金额
        result.setResAmt(Optional.ofNullable(param.getResAmt()).map(Money::new).map((item) -> item.multiply(10000L)).orElse(null));
        //trCycle 转让周期
        result.setTrCycle(TrCycleEnum.getByCode(param.getTrCycle()));
        //cycle 是否循环注水
        result.setCycle(param.getCycle());
        if (result.getTrCycle() == TrCycleEnum.STATIC) {
            result.setCycle(false);
        }
        result.setCycle(param.getCycle());
        //slice 是否切片
        result.setSlice(param.getSlice());
        //stopTrDate 停止转让日 持续购买终止日
        result.setStopTrDate(param.getStopTrDate());
        //startTrDate 开始转让日 持续购买开始日
        result.setStartTrDate(param.getStartTrDate());
        if (CollectionUtils.isNotEmpty(param.getScreeningRules())) {
            List<AstScreenRule> astScreenRuleList = Lists.newArrayList();
            for (String screeningRuleNo : param.getScreeningRules()) {
                AstScreenRule astScreenRule = new AstScreenRule();
                astScreenRule.setScreenRuleNo(screeningRuleNo);
                astScreenRuleList.add(astScreenRule);
            }
            result.setAstScreenRuleList(astScreenRuleList);
        }
        return result;
    }
    
    /**
     * 分期定价规则
     * @param param
     * @return
     */
    public static List<PlanAstPriceRule> convert2PlanAstPriceRuleList(List<PlanAstPriceRuleParam> param) {
        List<PlanAstPriceRule> result = Lists.newArrayList();
        if (param == null) {
            return null;
        }
        param.forEach(item -> result.add(convert2PlanAstPriceRule(item)));
        return result;
    }
    
    private static PlanAstPriceRule convert2PlanAstPriceRule(PlanAstPriceRuleParam param) {
        if (param == null) {
            return null;
        }
        PlanAstPriceRule result = new PlanAstPriceRule();
        result.setPlanNo(param.getPlanNo());
        result.setTermNo(param.getTermNo());
        result.setIntrDiscountRate(param.getIntrDiscountRate());
        return result;
    }
}
