/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.common.model.convertor.vo;

import cn.hutool.core.util.IdcardUtil;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.LoanDO;
import com.abssqr.plat.common.facade.enums.LoanStatusEnum;
import com.abssqr.plat.common.facade.enums.OwnStatusEnum;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.result.ast.LoanAstDetail;
import com.abssqr.plat.common.facade.result.ast.LoanAstProfitDetail;
import com.abssqr.plat.common.facade.result.ast.LoanContractDetail;
import com.abssqr.plat.common.facade.result.ast.LoanDetail;
import com.abssqr.plat.common.facade.result.ast.LoanInstalDetail;
import com.abssqr.plat.common.facade.result.ast.LoanListVO;
import com.abssqr.plat.common.facade.utils.BigDecimalFormatUtil;
import com.abssqr.plat.common.facade.utils.CalcUtil;
import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.domain.ast.LoanAst;
import com.abssqr.plat.common.model.domain.ast.LoanAstProfit;
import com.abssqr.plat.common.model.domain.ast.LoanAstProfitGroup;
import com.abssqr.plat.common.model.domain.ast.LoanInstalDomain;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncRepayPlan;
import com.abssqr.plat.common.model.domain.contract.LoanContract;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.utils.DescUtil;
import com.general.system.common.model.Money;
import com.general.system.tool.util.ToolUtil;
import com.google.common.collect.Lists;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 资产相关模型转换
 * @author hanfei
 * @version $Id: LoanVOConvert.java, v 0.1 2019-12-12 6:01 PM hanfei Exp $
 */
public class LoanVOConvert {
    
    /**
     * 资产列表
     * @param datas
     * @return
     */
    public static List<LoanListVO> buildDetailList(List<LoanDO> datas, List<LoanContract> contracts) {
        List<LoanListVO> listVOS = Lists.newArrayList();
        if (ToolUtil.isOneEmpty(datas, contracts)){
            return listVOS;
        }
        Map<String, LoanContract> contractMap = contracts.stream().collect(
                Collectors.toMap(LoanContract::getAstNo, item -> item));
        for (LoanDO domain : datas){
            listVOS.add(convert(domain, contractMap.get(domain.getAstNo())));
        }
        return listVOS;
    }
    
    private static LoanListVO convert(LoanDO loan, LoanContract loanContract) {
        boolean isContNull = loanContract == null;
        Money zeroMoney = new Money(0);
        LoanListVO listVO = new LoanListVO();
        listVO.setAstNo(loan.getAstNo());
        
        listVO.setOutAstNo(loan.getOutAstNo());
        listVO.setOutProdNo(loan.getOutProdNo());
        listVO.setManagerOrgCode(loan.getManagerOrgCode());
        listVO.setLenderOrgCode(loan.getLenderOrgCode());
        listVO.setInDate(loan.getInDate());
        listVO.setTermCnt(loan.getTermCnt());
        listVO.setCurTerm(loan.getCurTerm());
        listVO.setPrinBal(loan.getPrinBal().add(loan.getOvdPrinBal()));
        listVO.setIntrBal(loan.getIntrBal().add(loan.getOvdIntrBal()));
        listVO.setPenalBal(loan.getPrinPenalBal().add(loan.getIntrPenalBal()));
        listVO.setOvdDays(loan.getOvdDays());
        listVO.setStatus(LoanStatusEnum.getByCode(loan.getStatus()));
        listVO.setOwnStatus(OwnStatusEnum.getByCode(loan.getOwnStatus()));
        // 合同上信息
        listVO.setRateDesc(DescUtil.buildRateDesc(loanContract));
        listVO.setContractNo(isContNull ? "" : loanContract.getOutContNo());
        listVO.setDeptrName(isContNull ? "" : loanContract.getLoanCustInfo().getName());
        listVO.setDistAmout(isContNull ? zeroMoney : loanContract.getDisbAmount());
        listVO.setRepayMethod(isContNull ? null : loanContract.getLoanProperty().getRepayMethod());
        listVO.setStartDate(isContNull ? null : loanContract.getContractPeriod().getStartDate());
        listVO.setEndDate(isContNull ? null : loanContract.getContractPeriod().getEndDate());
        listVO.setLayerName(loan.getLayerName());
        return listVO;
    }

    /**
     * 资产详情
     * @param loan
     * @return
     */
    public static LoanDetail buildDetail(Loan loan) {
        if (ToolUtil.isOneEmpty(loan, loan.getContract(), loan.getLoanInstalList())){
            throw new AbssqrBizException("错误的贷款数据");
        }
        LoanDetail detail = new LoanDetail();
        // 合同
        LoanContractDetail loanContractDetail = convert(loan);
        // 分期
        List<LoanInstalDetail> loanInstalDetails = new ArrayList<>();
        for (LoanInstalDomain domain : loan.getLoanInstalList()){
            // 分期详情
            loanInstalDetails.add(convert(domain));
        }
        detail.setAstNo(loan.getAstNo());
        detail.setLoanContractDetail(loanContractDetail);
        detail.setLoanInstalDetails(loanInstalDetails);
        return detail;
    }
    
    /**
     * 合同详情
     * @param loan
     * @return
     */
    private static LoanContractDetail convert(Loan loan) {
        LoanContract contract = loan.getContract();
        LoanContractDetail detail = new LoanContractDetail();
        detail.setContNo(contract.getContNo());
        detail.setOutContNo(contract.getOutContNo());
        detail.setProdNo(contract.getProdNo());
        detail.setContractType(contract.getContractType());
        detail.setStartDate(contract.getContractPeriod().getStartDate());
        detail.setEndDate(contract.getContractPeriod().getEndDate());
        detail.setEffectDate(contract.getContractPeriod().getEffectDate());
        detail.setLenderOrgCode(contract.getLenderOrgCode());
        detail.setAstNo(contract.getAstNo());
        detail.setUserId(contract.getUserId());
        detail.setOutAstNo(contract.getOutAstNo());
        detail.setDisbAmount(contract.getDisbAmount());
        detail.setDisbDate(contract.getDisbDate());
        detail.setName(contract.getLoanCustInfo().getName());
        detail.setCertType(contract.getLoanCustInfo().getCertType());
        detail.setCertNo(contract.getLoanCustInfo().getCertNo());
        detail.setMobileNo(contract.getLoanCustInfo().getMobileNo());
        if (ToolUtil.isNotEmpty(contract.getLoanCustInfo().getProvince())){
            String province = IdcardUtil.getProvinceByIdCard(
                    contract.getLoanCustInfo().getProvince() + "1234567890123456");
            detail.setProvince(province);
        }
        detail.setCity(contract.getLoanCustInfo().getCity());
        detail.setRepayMethod(contract.getLoanProperty().getRepayMethod());
        detail.setRepayPeriodType(contract.getLoanProperty().getRepayPeriodType());
        detail.setRate(contract.getLoanProperty().getRate());
        detail.setRateDesc(DescUtil.buildRateDesc(contract));
        detail.setRateType(contract.getLoanProperty().getRateType());
        detail.setTermCnt(contract.getLoanProperty().getTermCnt());
        detail.setLoanType(contract.getLoanProperty().getLoanType());
        // 资产转出状态
        detail.setOwnStatus(loan.getAstOrg().getAstOwnership().getOwnStatus());
        detail.setStatus(loan.getStatus());
        return detail;
    }
    
    /**
     * 分期详情
     * @param domain
     * @return
     */
    private static LoanInstalDetail convert(LoanInstalDomain domain){
        LoanInstalDetail detail = new LoanInstalDetail();
        detail.setStartDate(domain.getAstPeriod().getStartDate());
        detail.setEndDate(domain.getAstPeriod().getEndDate());
        detail.setClearDate(domain.getAstPeriod().getClearDate());
        detail.setAstNo(domain.getAstNo());
        detail.setOutAstNo(domain.getOutAstNo());
        detail.setTermNo(domain.getTermNo());
        detail.setBal(domain.getBal());
        detail.setPaidAmt(domain.getPaidAmt());
        detail.setRmtAmt(domain.getRmtAmt());
        detail.setPaidFundAmt(domain.getPaidFundAmt());
        detail.setStatus(domain.getStatus());
        detail.setOvdDays(domain.getOvdDays());
        if (ToolUtil.isNotEmpty(domain.getLoanAsts())
            && domain.getLoanAsts().size() > 1){
            detail.setOwnStatus(OwnStatusEnum.TRANS_OUT);
        }else{
            detail.setOwnStatus(OwnStatusEnum.INIT);
        }
        return detail;
    }
    
    /**
     * 分期资产详情含收益
     * @param loan
     * @param termNo
     * @return
     */
    public static LoanInstalDetail buildDetail(Loan loan, Integer termNo){
        if (ToolUtil.isOneEmpty(loan, loan.getLoanInstalList())){
            throw new AbssqrBizException("错误的贷款数据");
        }
        // 当前分期
        LoanInstalDomain domain = null;
        // 对应机构下的收益列表
        for (LoanInstalDomain one : loan.getLoanInstalList()){
            // 取特定分期
            if (ToolUtil.equals(one.getTermNo(), termNo)){
                domain = one;
            }
        }
        if (domain == null){
            throw new AbssqrBizException("没有匹配的分期"+termNo);
        }
        // 组装分期
        LoanInstalDetail detail = convert(domain);
        // 当前分期收益列表
        detail.setLoanAstDetails(convertByLoanAst(domain.getLoanAsts(),detail.getEndDate()));
        // 当前分期分摊收益列表
        detail.setRefAstDetails(convertByProfit(domain.getSourcedProfits(),detail.getEndDate()));
        return detail;
    }
    
    private static List<LoanAstDetail> convertByLoanAst(List<LoanAst> loanAsts, Date endDate){
        if (ToolUtil.isEmpty(loanAsts)){
            return Lists.newArrayList();
        }
        // 根据所属机构分组 机构资产
        Map<String, LoanAstDetail> ownOrgMap = loanAsts.stream().collect(
                Collectors.toMap(LoanAst::getLoanAstNo, item -> convert(item)));
        
        for (LoanAst one : loanAsts){
            Map<Integer, LoanAstProfitGroup> profitGroupMap = one.getLoanAstProfits();
            for(Map.Entry<Integer, LoanAstProfitGroup> entry : profitGroupMap.entrySet()){
                // 所属收益
                if (ToolUtil.isNotEmpty(entry.getValue().getOwnAstProfit())
                    && ToolUtil.isNotEmpty(ownOrgMap.get(entry.getValue().getOwnAstProfit().getLoanAstNo()))){
                    // 组装收益
                    ownOrgMap.get(entry.getValue().getOwnAstProfit().getLoanAstNo())
                            .addProfit(convert(entry.getValue().getOwnAstProfit()));
                }
                // 分润收益
                if (ToolUtil.isNotEmpty(entry.getValue().getSplitAstProfit())
                    && ToolUtil.isNotEmpty(ownOrgMap.get(entry.getValue().getSplitAstProfit().getLoanAstNo()))) {
                    // 组装收益
                    ownOrgMap.get(entry.getValue().getSplitAstProfit().getLoanAstNo())
                            .addProfit(convert(entry.getValue().getSplitAstProfit()));
                }
            }
        }
        List<LoanAstDetail> loanAstDetails = Lists.newArrayList(ownOrgMap.values());
        for (LoanAstDetail astDetail : loanAstDetails){
            astDetail.setEndDate(endDate);
            Collections.sort(astDetail.getProfitDetails(), Comparator.comparingInt(LoanAstProfitDetail::getTermNo));
        }
        Collections.sort(loanAstDetails, Comparator.comparingInt(LoanAstDetail::getTermNo).thenComparing(LoanAstDetail::getOwnOrg));
        return loanAstDetails;
    }
    
    private static List<LoanAstDetail> convertByProfit(List<LoanAstProfit> profits,Date endDate) {
        if (ToolUtil.isEmpty(profits)){
            return Lists.newArrayList();
        }
        // 根据所属机构分组
        Map<String, LoanAstDetail> ownOrgMap = new HashMap<>();
        // 获取对应机构的收益
        for (LoanAstProfit profit : profits){
            if (ToolUtil.isEmpty(ownOrgMap.get(profit.getLoanAstNo()))){
                ownOrgMap.put(profit.getLoanAstNo(), convert(profit.getRefLoanAst()));
            }
            ownOrgMap.get(profit.getLoanAstNo()).addProfit(convert(profit));
        }
        List<LoanAstDetail> loanAstDetails = Lists.newArrayList(ownOrgMap.values());
        for (LoanAstDetail astDetail : loanAstDetails){
            astDetail.setEndDate(endDate);
            Collections.sort(astDetail.getProfitDetails(), Comparator.comparingInt(LoanAstProfitDetail::getTermNo));
        }
        Collections.sort(loanAstDetails, Comparator.comparingInt(LoanAstDetail::getTermNo).thenComparing(LoanAstDetail::getOwnOrg));
        return loanAstDetails;
    }
    
    /**
     * 机构资产
     * @param loanAst
     * @return
     */
    private static LoanAstDetail convert(LoanAst loanAst){
        LoanAstDetail detail = new LoanAstDetail();
        detail.setLoanAstNo(loanAst.getLoanAstNo());
        detail.setAstNo(loanAst.getAstNo());
        detail.setOutAstNo(loanAst.getOutAstNo());
        detail.setOwnOrg(loanAst.getOwnOrg());
        detail.setLenderOrgCode(loanAst.getLenderOrgCode());
        detail.setTermNo(loanAst.getTermNo());
        detail.setPrinBal(loanAst.getPrinBal());
        detail.setPaidPrinAmt(loanAst.getPaidPrinAmt());
        detail.setCostAmt(loanAst.getCostAmt());
        detail.setAstPrice(loanAst.getAstPrice());
        if (BigDecimal.ZERO.compareTo(loanAst.getAstPrice()) == 0){
            detail.setAstPriceDesc("--");
        }else{
            detail.setAstPriceDesc(BigDecimalFormatUtil.formatRate(loanAst.getAstPrice()) + "%");
        }
        detail.setAstPriceType(loanAst.getAstPriceType());
        detail.setInRule(loanAst.getInRule());
        detail.setInDate(loanAst.getInDate());
        detail.setAstStatus(loanAst.getAstStatus());
        detail.setClearDate(loanAst.getClearDate());
        return detail;
    }
    
    /**
     * 收益
     * @param profit
     * @return
     */
    private static LoanAstProfitDetail convert(LoanAstProfit profit){
        LoanAstProfitDetail detail = new LoanAstProfitDetail();
        detail.setAstProfitNo(profit.getAstProfitNo());
        detail.setLoanAstNo(profit.getLoanAstNo());
        detail.setProfitType(profit.getProfitType());
        detail.setAstNo(profit.getAstNo());
        detail.setOutAstNo(profit.getOutAstNo());
        detail.setTermNo(profit.getTermNo());
        detail.setSrcTermNo(profit.getSrcTermNo());
        detail.setOwnOrg(profit.getOwnOrg());
        detail.setSplitRate(profit.getSplitRate());
        if (BigDecimal.ZERO.compareTo(profit.getSplitRate()) == 0){
            detail.setSplitRateDesc("--");
        }else{
            detail.setSplitRateDesc(BigDecimalFormatUtil.formatRate(profit.getSplitRate()) + "%");
        }
        detail.setBal(profit.getBal());
        detail.setRmtAmt(profit.getRmtAmt());
        detail.setFundAmt(profit.getFundAmt());
        detail.setPaidAmt(profit.getPaidAmt());
        detail.setEntryAmt(profit.getEntryAmt());
        detail.setIntrEntryDec(profit.getIntrEntryDec());
        detail.setIntrPenalEntryDec(profit.getIntrPenalEntryDec());
        detail.setPrinPenalEntryDec(profit.getPrinPenalEntryDec());
        detail.setFeeEntryDec(profit.getFeeEntryDec());
        detail.setFeePenalEntryDec(profit.getFeePenalEntryDec());
        return detail;
    }
    
    
}
