/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2017 All Rights Reserved. 
 */
package com.abssqr.plat.common.model.domain.ast;

import com.general.system.common.model.BaseEntity;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.VarChecker;

import java.util.Date;

/**
 * 资产周期
 *
 * @author zhenxuan.luo
 * @version com.abssqr.common.model.domain.asset: AstPeriod.java, v 0.1 2017-12-05 下午16:00 zhenxuan.luo Exp $
 */
public class AstPeriod extends BaseEntity {
    
    /**
     * 开始日期
     */
    private Date startDate;
    /**
     * 结束日期
     */
    private Date endDate;
    /**
     * 结清日期
     */
    private Date clearDate;
    /**
     * 资产在当前所属机构形成时间（比如转入日）
     */
    private Date inDate;
    /**
     * 宽限期结束日期
     */
    private Date graceEndDate;
    /**
     * 放款日期
     */
    private Date disbDate;
    
    public void cloneTo(AstPeriod astPeriod) {
        this.startDate = astPeriod.startDate;
        this.endDate = astPeriod.endDate;
        this.clearDate = astPeriod.clearDate;
        this.inDate = astPeriod.inDate;
        this.graceEndDate = astPeriod.graceEndDate;
        this.disbDate = astPeriod.disbDate;
    }
    
    public void checkSelf() {
        VarChecker.checkArgument(DateTimeUtil.calcDayDiff(endDate, startDate) >= 0, "结束日期不能早于开始日期");
    }
    
    public boolean isInTerm(Date date) {
        return DateTimeUtil.calcDayDiff(date, this.startDate) >= 0
               && DateTimeUtil.calcDayDiff(date, this.endDate) <= 0;
    }
    
    public Date getStartDate() {
        return startDate;
    }
    
    public void setStartDate(Date startDate) {
        this.startDate = startDate;
    }
    
    public Date getEndDate() {
        return endDate;
    }
    
    public void setEndDate(Date endDate) {
        this.endDate = endDate;
    }
    
    public Date getClearDate() {
        return clearDate;
    }
    
    public void setClearDate(Date clearDate) {
        this.clearDate = clearDate;
    }
    
    public Date getInDate() {
        return inDate;
    }
    
    public void setInDate(Date inDate) {
        this.inDate = inDate;
    }
    
    public Date getGraceEndDate() {
        return graceEndDate;
    }
    
    public void setGraceEndDate(Date graceEndDate) {
        this.graceEndDate = graceEndDate;
    }
    
    public Date getDisbDate() {
        return disbDate;
    }
    
    public void setDisbDate(Date disbDate) {
        this.disbDate = disbDate;
    }
}