/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */
package com.abssqr.plat.common.model.domain.plan;

import com.abssqr.plat.common.facade.enums.AstFlowTypeEnum;
import com.abssqr.plat.common.facade.enums.StageChangeTypeEnum;
import com.abssqr.plat.common.facade.enums.TransCodeEnum;
import com.abssqr.plat.common.facade.model.plan.PlanAst;
import com.abssqr.plat.common.model.enums.TraderTypeEnum;
import com.abssqr.plat.common.model.exception.AbssqrInvalidParamException;
import com.abssqr.plat.common.model.utils.PlanAstChangeCheckerUtil;
import com.general.system.common.model.BaseEntity;
import com.general.system.common.model.Money;
import com.general.system.common.util.DateTimeUtil;

import java.util.Date;

/**
 * @author hanfei
 * @version $Id: PlanAstLog.java, v 0.1 2018-01-05 下午2:54 hanfei Exp $
 */
public class PlanAstLog extends BaseEntity {
    
    /**
     * id ID.
     */
    private Long                id;
    /**
     * planNo 计划编号.
     */
    private String              planNo;
    /**
     * transBizNo 关联业务编号
     */
    private String              transBizNo;
    /**
     * 交易对象类型
     */
    private TraderTypeEnum      traderType;
    /**
     * transCode 交易场景码.
     */
    private TransCodeEnum       transCode;
    /**
     * changeType 水位变更类型.  {@link StageChangeTypeEnum}
     */
    private StageChangeTypeEnum changeType;
    /**
     * setlDt 会记日.
     */
    private Date                accountDate;
    /**
     * 发生额，不区分发生类型
     */
    private Money happenAmt = new Money();
    /**
     * stage 水位变更发生金额.
     */
    private Money cashAmt        = new Money();
    /**
     * stage 当前水位.
     */
    private Money cash                = new Money();
    /**
     * lastStage 变更前水位.
     */
    private Money lastCash            = new Money();
    /**
     * frozenStage 当前冻结水位.
     */
    private Money frozenCash          = new Money();
    /**
     * frozenStage 冻结水位变更发生金额.
     */
    private Money frozenCashAmt       = new Money();
    /**
     * lastFrozenStage 变更前冻结水位.
     */
    private Money lastFrozenCash      = new Money();
    /**
     * 累计投放
     */
    private Money accGrant            = new Money();
    /**
     * 累计投放
     */
    private Money accGrantAmt         = new Money();
    /**
     * 变更前累计投放
     */
    private Money lastAccGrant        = new Money();
    /**
     * 当前流出在途金额
     */
    private Money payOnWay            = new Money();
    /**
     * 变更前流出在途金额
     */
    private Money lastPayOnWay        = new Money();
    /**
     * 当前流入在途金额
     */
    private Money repoOnWay           = new Money();
    /**
     * 变更前流入在途金额
     */
    private Money lastRepoOnWay  = new Money();
    /**
     * 支付待结算金额
     */
    private Money payClear       = new Money();
    /**
     * 还款待结算金额
     */
    private Money repoClear      = new Money();
    /**
     * 变更前支付待结算金额
     */
    private Money lastPayClear   = new Money();
    /**
     * 变更前还款待结算金额
     */
    private Money lastRepoClear  = new Money();
    /**
     * 校验水位金额
     */
    private Money checkCash    = new Money();
    /**
     * 校验水位金额
     */
    private Money lastCheckCash    = new Money();
    /**
     * 当前银行账户余额
     */
    private Money bankBal        = new Money();
    /**
     * 变更前银行账户余额
     */
    private Money lastBankBal    = new Money();
    /**
     * extInf 扩展字段.
     */
    private String extInf;
    
    public static PlanAstLog buildAstLog(Date accountDate, PlanAst planAst, String transBizNo, TraderTypeEnum traderType,
                                         TransCodeEnum transCode, StageChangeTypeEnum changeType) {
        PlanAstLog planAstLog = new PlanAstLog();
        planAstLog.setPlanNo(planAst.getPlanNo());
        planAstLog.setTransBizNo(transBizNo);
        planAstLog.setTraderType(traderType);
        planAstLog.setTransCode(transCode);
        planAstLog.setChangeType(changeType);
        planAstLog.setAccountDate(DateTimeUtil.getStandardDate(accountDate));
        planAstLog.setCash(planAst.getCash().clone());
        planAstLog.setLastCash(planAst.getCash().clone());
        planAstLog.setFrozenCash(planAst.getFrozenCash().clone());
        planAstLog.setLastFrozenCash(planAst.getFrozenCash().clone());
        planAstLog.setAccGrant(planAst.getAccGrant().clone());
        planAstLog.setLastAccGrant(planAst.getAccGrant().clone());
        // 在途金额
        planAstLog.setPayOnWay(planAst.getPayOnWay().clone());
        planAstLog.setRepoOnWay(planAst.getRepoOnWay().clone());
        planAstLog.setLastPayOnWay(planAst.getPayOnWay().clone());
        planAstLog.setLastRepoOnWay(planAst.getRepoOnWay().clone());
        // 待结算金额
        planAstLog.setPayClear(planAst.getPayClear().clone());
        planAstLog.setRepoClear(planAst.getRepoClear().clone());
        planAstLog.setLastPayClear(planAst.getPayClear().clone());
        planAstLog.setLastRepoClear(planAst.getRepoClear().clone());
        // 水位校验金额
        planAstLog.setCheckCash(planAst.getCheckCash().clone());
        planAstLog.setLastCheckCash(planAst.getCheckCash().clone());
        // 银行余额
        planAstLog.setBankBal(planAst.getBankBal().clone());
        planAstLog.setLastBankBal(planAst.getLastBankBal().clone());
        
        return planAstLog;
    }
    
    /**
     * 变更水位金额变化
     * @param planAst       需要变更的水位
     * @param happenAmt     实际发生额
     * @param extInf
     */
    public void changeStageAndLog(PlanAst planAst, AstFlowTypeEnum flowType, Money happenAmt, Money clearToWayAmt, String extInf) {
        if (happenAmt == null) {
            throw new AbssqrInvalidParamException("当前发生金额不能为空");
        }
        // 不区分类型记录发生额
        this.setHappenAmt(happenAmt);
        switch (changeType) {
            case ON_WAY:
                // 流入流出金额在途记录
                this.changeOnWay(planAst, flowType, happenAmt, 1);
                // 扣减清算对应的 待结算金额
                this.changeClear(planAst, flowType, clearToWayAmt, -1);
                break;
            case SUB:
                // 扣减资金余额
                this.changeCash(planAst, happenAmt, -1);
                // 扣减流出在途金额
                this.changeOnWay(planAst, flowType, happenAmt, -1);
                break;
            case ADD:
                // 增加资金余额
                this.changeCash(planAst, happenAmt, 1);
                // 使用原在途金额 扣减流入在途金额
                this.changeOnWay(planAst, flowType, happenAmt, -1);
                break;
            // 清算流水 转结算单
            case CLEAR:
                // 待结算金额变更 增加
                this.changeClear(planAst, flowType, happenAmt, 1);
                break;
            default:
                // 无变化
        }
        this.setExtInf(extInf);
        /**
         * 校验金额相等
         */
        PlanAstChangeCheckerUtil.me().checkEqualFund(planAst, this);
    }
    
    /**
     * 水位金额变化
     */
    private void changeCash(PlanAst planAst, Money happenAmt, int sign) {
        if (happenAmt.isZero()){
            return;
        }
        planAst.getCash().addTo(happenAmt.multiply(sign));
        this.setCashAmt(happenAmt.clone());
        this.setCash(planAst.getCash().clone());
        
        // 判断是否是资产流入 累计转让金额变化
        if (this.getTransCode().getAstFlow() == AstFlowTypeEnum.FLOW_IN) {
            planAst.getAccGrant().addTo(happenAmt);
            this.setAccGrantAmt(happenAmt.clone());
            this.setAccGrant(planAst.getAccGrant().clone());
        }
    }
    
    /**
     * 在途流入流出金额变化
     */
    private void changeOnWay(PlanAst planAst, AstFlowTypeEnum flowType, Money happenAmt, int sign) {
        if (happenAmt.isZero()){
            return;
        }
//        // 排除资本调整  其他场景下不会有在途情况 所以 不需要排除
//        if (this.getTransCode() == TransCodeEnum.CAP_INVEST
//            || this.getTransCode() == TransCodeEnum.CAP_ADJ_ADD
//            || this.getTransCode() == TransCodeEnum.CAP_ADJ_SUB) {
//            return;
//        }
        switch (flowType) {
            case FLOW_IN:
                planAst.getRepoOnWay().addTo(happenAmt.multiply(sign));
                this.setRepoOnWay(planAst.getRepoOnWay().clone());
                break;
            case FLOW_OUT:
                planAst.getPayOnWay().addTo(happenAmt.multiply(sign));
                this.setPayOnWay(planAst.getPayOnWay().clone());
                break;
            default:
                // 无变化
        }
    }
    
    /**
     * 流入流出待结算金额变化
     * @param planAst
     * @param happenAmt
     * @param sign
     */
    private void changeClear(PlanAst planAst, AstFlowTypeEnum flowType, Money happenAmt, int sign) {
        // 判断 分配兑付 资金调整 跳过待结算金额处理
        if (this.getTransCode() == TransCodeEnum.ALLOC
            || this.getTransCode() == TransCodeEnum.CAP_INVEST
            || this.getTransCode() == TransCodeEnum.CAP_ADJ_ADD
            || this.getTransCode() == TransCodeEnum.CAP_ADJ_SUB) {
            return;
        }
        if (happenAmt.isZero()){
            return;
        }
        switch (flowType) {
            case FLOW_IN:
                planAst.getRepoClear().addTo(happenAmt.multiply(sign));
                this.setRepoClear(planAst.getRepoClear().clone());
                if (sign == 1) {
                    // 发生资金流入待结算+时 通过校验水位先增加水位金额
                    planAst.getCheckCash().addTo(happenAmt.clone());
                }
                break;
            case FLOW_OUT:
                planAst.getPayClear().addTo(happenAmt.multiply(sign));
                this.setPayClear(planAst.getPayClear().clone());
                if (sign == 1){
                    // 发生资金流出待结算+时 通过校验水位先减少水位金额
                    planAst.getCheckCash().subtractFrom(happenAmt.clone());
                }
                break;
            default:
                // 无变化
        }
        this.setCheckCash(planAst.getCheckCash().clone());
    }
    
    /**
     * 银行余额变化
     * @param planAst
     * @param happenAmt
     */
    public void bankSyncChangeStageAndLog(PlanAst planAst, Money happenAmt, Money bankBal, String extInf) {
        // 设置银行余额
        planAst.setLastBankBal(planAst.getBankBal().clone());
        planAst.setBankBal(bankBal);
        if (happenAmt.isZero()){
            return;
        }
        switch (this.getTransCode().getFundFlow()) {
            case FLOW_IN:
                planAst.getCash().addTo(happenAmt.clone());
                planAst.getCheckCash().addTo(happenAmt.clone());
                break;
            case FLOW_OUT:
                planAst.getCash().subtractFrom(happenAmt.clone());
                planAst.getCheckCash().subtractFrom(happenAmt.clone());
                break;
            default:
                // 无变化
        }
        this.setCashAmt(happenAmt.clone());
        this.setCash(planAst.getCash().clone());
        this.setExtInf(extInf);
        // 不区分类型记录发生额
        this.setHappenAmt(happenAmt);
        this.setBankBal(bankBal);
        
        this.setCheckCash(planAst.getCheckCash().clone());
    }
    
    /**
     * 冻结水位金额变化 冻结金额减少时 在途金额增加
     * @param planAst 水位
     * @param unFrozeAmt  解冻金额
     * @param happenAmt   实际发生支付金额
     * @param extInf  备注信息
     */
    public void changeFrozenCash(PlanAst planAst, Money unFrozeAmt, Money happenAmt, String extInf) {
        if (unFrozeAmt.isZero()){
            return;
        }
        int sign;
        switch (changeType) {
            case FROZEN:
                // 冻结水位增加 金额
                sign = 1;
                break;
            case UNFROZEN:
                // 分配 释放冻结 使用起初冻结的金额
                sign = -1;
                // 解冻金额时 通过实际发生支付金额扣减校验水位金额
                planAst.getCheckCash().subtractFrom(happenAmt.clone());
                this.setCheckCash(planAst.getCheckCash().clone());
                break;
            default:
                return;
        }
        planAst.getFrozenCash().addTo(unFrozeAmt.multiply(sign));
        this.setFrozenCashAmt(unFrozeAmt.clone());
        this.setFrozenCash(planAst.getFrozenCash().clone());
        this.setExtInf(extInf);
        // 不区分类型记录发生额
        this.setHappenAmt(unFrozeAmt);
    }
    
    
    public Long getId() {
        return id;
    }
    
    public void setId(Long id) {
        this.id = id;
    }
    
    public String getPlanNo() {
        return planNo;
    }
    
    public void setPlanNo(String planNo) {
        this.planNo = planNo;
    }
    
    public TransCodeEnum getTransCode() {
        return transCode;
    }
    
    public void setTransCode(TransCodeEnum transCode) {
        this.transCode = transCode;
    }
    
    public StageChangeTypeEnum getChangeType() {
        return changeType;
    }
    
    public void setChangeType(StageChangeTypeEnum changeType) {
        this.changeType = changeType;
    }
    
    public Date getAccountDate() {
        return accountDate;
    }
    
    public void setAccountDate(Date accountDate) {
        this.accountDate = accountDate;
    }
    
    public Money getCash() {
        return cash;
    }
    
    public void setCash(Money cash) {
        this.cash = cash;
    }
    
    public Money getLastCash() {
        return lastCash;
    }
    
    public void setLastCash(Money lastCash) {
        this.lastCash = lastCash;
    }
    
    public Money getFrozenCash() {
        return frozenCash;
    }
    
    public void setFrozenCash(Money frozenCash) {
        this.frozenCash = frozenCash;
    }
    
    public Money getLastFrozenCash() {
        return lastFrozenCash;
    }
    
    public void setLastFrozenCash(Money lastFrozenCash) {
        this.lastFrozenCash = lastFrozenCash;
    }
    
    public Money getAccGrant() {
        return accGrant;
    }
    
    public void setAccGrant(Money accGrant) {
        this.accGrant = accGrant;
    }
    
    public Money getLastAccGrant() {
        return lastAccGrant;
    }
    
    public void setLastAccGrant(Money lastAccGrant) {
        this.lastAccGrant = lastAccGrant;
    }
    
    public String getExtInf() {
        return extInf;
    }
    
    public void setExtInf(String extInf) {
        this.extInf = extInf;
    }
    
    public Money getCashAmt() {
        return cashAmt;
    }
    
    public void setCashAmt(Money cashAmt) {
        this.cashAmt = cashAmt;
    }
    
    public Money getFrozenCashAmt() {
        return frozenCashAmt;
    }
    
    public void setFrozenCashAmt(Money frozenCashAmt) {
        this.frozenCashAmt = frozenCashAmt;
    }
    
    public Money getAccGrantAmt() {
        return accGrantAmt;
    }
    
    public void setAccGrantAmt(Money accGrantAmt) {
        this.accGrantAmt = accGrantAmt;
    }
    
    public Money getPayOnWay() {
        return payOnWay;
    }
    
    public void setPayOnWay(Money payOnWay) {
        this.payOnWay = payOnWay;
    }
    
    public Money getRepoOnWay() {
        return repoOnWay;
    }
    
    public void setRepoOnWay(Money repoOnWay) {
        this.repoOnWay = repoOnWay;
    }
    
    public Money getLastRepoOnWay() {
        return lastRepoOnWay;
    }
    
    public void setLastRepoOnWay(Money lastRepoOnWay) {
        this.lastRepoOnWay = lastRepoOnWay;
    }
    
    public Money getLastPayOnWay() {
        return lastPayOnWay;
    }
    
    public void setLastPayOnWay(Money lastPayOnWay) {
        this.lastPayOnWay = lastPayOnWay;
    }
    
    public Money getPayClear() {
        return payClear;
    }
    
    public void setPayClear(Money payClear) {
        this.payClear = payClear;
    }
    
    public Money getRepoClear() {
        return repoClear;
    }
    
    public void setRepoClear(Money repoClear) {
        this.repoClear = repoClear;
    }
    
    public Money getLastPayClear() {
        return lastPayClear;
    }
    
    public void setLastPayClear(Money lastPayClear) {
        this.lastPayClear = lastPayClear;
    }
    
    public Money getLastRepoClear() {
        return lastRepoClear;
    }
    
    public void setLastRepoClear(Money lastRepoClear) {
        this.lastRepoClear = lastRepoClear;
    }
    
    public Money getBankBal() {
        return bankBal;
    }
    
    public void setBankBal(Money bankBal) {
        this.bankBal = bankBal;
    }
    
    public Money getLastBankBal() {
        return lastBankBal;
    }
    
    public void setLastBankBal(Money lastBankBal) {
        this.lastBankBal = lastBankBal;
    }
    
    public Money getHappenAmt() {
        return happenAmt;
    }
    
    public void setHappenAmt(Money happenAmt) {
        this.happenAmt = happenAmt;
    }
    
    public String getTransBizNo() {
        return transBizNo;
    }
    
    public void setTransBizNo(String transBizNo) {
        this.transBizNo = transBizNo;
    }
    
    public TraderTypeEnum getTraderType() {
        return traderType;
    }
    
    public void setTraderType(TraderTypeEnum traderType) {
        this.traderType = traderType;
    }
    
    public Money getCheckCash() {
        return checkCash;
    }
    
    public void setCheckCash(Money checkCash) {
        this.checkCash = checkCash;
    }
    
    public Money getLastCheckCash() {
        return lastCheckCash;
    }
    
    public void setLastCheckCash(Money lastCheckCash) {
        this.lastCheckCash = lastCheckCash;
    }
}
