/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */
package com.abssqr.plat.common.model.enums;

import com.abssqr.plat.common.facade.model.report.ReportField;
import com.abssqr.plat.common.facade.model.report.wk.WkAstReportFields;
import com.general.system.common.model.IEnum;

import java.util.HashMap;
import java.util.Map;

/**
 * 统计天数计算枚举
 *
 * @author hanfei
 * @version $Id: TermCntCalcEnum.java, v 0.1 2017-06-07 下午18:38 hanfei Exp $
 */
public enum TermCntCalcEnum implements IEnum {
    
    TM01("TM01", "1期", 1, 1, WkAstReportFields.WKAstT001, WkAstReportFields.WKProdT001),
    
    TM02("TM02", "2-3期", 2, 3, WkAstReportFields.WKAstT002, WkAstReportFields.WKProdT002),
    
    TM03("TM03", "4-6期", 4, 6, WkAstReportFields.WKAstT003, WkAstReportFields.WKProdT003),
    
    TM04("TM04", "7-12期", 7, 12, WkAstReportFields.WKAstT004, WkAstReportFields.WKProdT004),
    
    TM05("TM05", "12期以上", 13, null, WkAstReportFields.WKAstT005, WkAstReportFields.WKProdT005),
    
    ;

    private static final Map<String, TermCntCalcEnum> codeMap = new HashMap<String, TermCntCalcEnum>();

    static {
        for (TermCntCalcEnum item : TermCntCalcEnum.values()) {
            codeMap.put(item.getCode(), item);
        }
    }

    private String code;
    private String desc;
    private Integer termStart;
    private Integer termEnd;
    private ReportField astField;
    private ReportField prodField;

    TermCntCalcEnum(String code, String desc, Integer termStart, Integer termEnd, ReportField astField, ReportField prodField) {
        this.code = code;
        this.desc = desc;
        this.termStart = termStart;
        this.termEnd = termEnd;
        this.astField = astField;
        this.prodField = prodField;
    }
    
    public static TermCntCalcEnum getByCode(String code) {
        return codeMap.get(code);
    }
    
    
    public boolean isMatch(int termCnt) {
        if (this.getTermEnd() == null) {
            return termCnt >= this.getTermStart();
        } else {
            return termCnt >= this.getTermStart() && termCnt <= this.getTermEnd();
        }
    }
    
    public static Integer matchTermStart(ReportField field) {
        for (TermCntCalcEnum item : TermCntCalcEnum.values()) {
            if (item.getAstField() == field || item.getProdField() == field){
                return item.getTermStart();
            }
        }
        return 0;
    }
    
    @Override
    public String getCode() {
        return code;
    }
    
    @Override
    public String getDesc() {
        return desc;
    }
    
    public Integer getTermStart() {
        return termStart;
    }
    
    public Integer getTermEnd() {
        return termEnd;
    }
    
    public ReportField getAstField() {
        return astField;
    }
    
    public ReportField getProdField() {
        return prodField;
    }
    
    
}