/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.model.repo.acct.impl;

import com.abssqr.plat.common.dal.mysql.auto.dao.AccountDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.AccountDO;
import com.abssqr.plat.common.facade.enums.AccountAttributeEnum;
import com.abssqr.plat.common.facade.enums.AccountTypeEnum;
import com.abssqr.plat.common.facade.model.account.Account;
import com.abssqr.plat.common.model.repo.acct.AcctRepo;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @author bangis.wangdf
 * @version com.abssqr.plat.common.model.repo.acctday.impl: AcctRepoImpl.java, v 0.1 2019-06-22 14:39 bangis.wangdf Exp $
 */
@Component
public class AcctRepoImpl implements AcctRepo {
    @Autowired
    private AccountDAO accountDAO;

    @Override
    public void addOrUpdateAccounts(String planNo, List<Account> accounts) {

        if (CollectionUtils.isNotEmpty(accounts)) {
            List list = accounts.stream().filter(account -> account.getActType() == AccountTypeEnum.BANK).collect(
                Collectors.toList());
            if (CollectionUtils.isNotEmpty(list)) {
                accountDAO.deleteByEntityCodeAndActType(AccountTypeEnum.BANK.getCode(), planNo);
            }
            List<AccountDO> accountDOS = Lists.newArrayList();
            accounts.forEach(acct -> {
                AccountDO accountDO = convert2Do(planNo, acct);
                accountDOS.add(accountDO);
            });
            accountDAO.insertBatch(accountDOS);
        }
    }

    @Override
    public List<Account> getAcctByPlanNo(String planNo, AccountAttributeEnum acctAttr,
                                         List<AccountTypeEnum> accountTypes) {
        VarChecker.checkNotNull(accountTypes);
        List<String> actTypes = accountTypes.stream().map(AccountTypeEnum::getCode).collect(Collectors.toList());
        String acctAttrStr = acctAttr != null ? acctAttr.getCode() : null;
        List<AccountDO> accountDOS = accountDAO.selectByEntityCodeAndActType(acctAttrStr, planNo, actTypes);
        List<Account> result = Lists.newArrayList();
        if (CollectionUtils.isNotEmpty(accountDOS)) {
            for (AccountDO accountDO : accountDOS) {
                result.add(convert2domain(accountDO));
            }
        }
        return result;
    }

    @Override
    public List<Account> selectByEntityCodeAndType(String planNo, String actType) {
        List<AccountDO> accountDOs = accountDAO.selectByEntityCodeAndType(actType, planNo);
        List<Account> result = Lists.newArrayList();
        if (CollectionUtils.isNotEmpty(accountDOs)) {
            for (AccountDO accountDO : accountDOs) {
                result.add(convert2domain(accountDO));
            }
        }
        return result;
    }

    @Override
    public int deleteByEntityCode(String entityCode) {
        return accountDAO.deleteByEntityCode(entityCode);
    }

    @Override
    public Account getAcctsByAccountNo(String accountNo) {
        return convert2domain(accountDAO.getByAccountNo(accountNo));
    }

    private Account convert2domain(AccountDO aDo) {
        if (aDo == null) {
            return null;
        }
        Account acct = new Account();
        acct.setPlanNo(aDo.getEntityCode());
        acct.setActNo(aDo.getActNo());
        acct.setActAttr(AccountAttributeEnum.getByCode(aDo.getActAttr()));
        acct.setActType(AccountTypeEnum.getByCode(aDo.getActType()));
        acct.setActName(aDo.getActName());
        acct.setActBank(aDo.getActBank());
        acct.setActProv(aDo.getActProv());
        acct.setActCity(aDo.getActCity());
        acct.setChannelNo(aDo.getChannelNo());
        return acct;
    }

    private AccountDO convert2Do(String planNo, Account acct) {
        AccountDO accountDO = new AccountDO();
        accountDO.setEntityCode(planNo);
        accountDO.setActNo(acct.getActNo());
        accountDO.setActAttr(acct.getActAttr().getCode());
        accountDO.setActType(acct.getActType().getCode());
        accountDO.setActName(acct.getActName());
        accountDO.setActBank(acct.getActBank());
        accountDO.setActProv(acct.getActProv());
        accountDO.setActCity(acct.getActCity());
        accountDO.setChannelNo(acct.getChannelNo());
        return accountDO;
    }

}
