/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */

package com.abssqr.plat.common.model.repo.ast.impl;

import com.abssqr.plat.common.dal.mysql.auto.dao.LoanInstalDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.LoanInstalDO;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.LoanInstalFormationItem;
import com.abssqr.plat.common.facade.enums.LoanStatusEnum;
import com.abssqr.plat.common.model.convertor.LoanInstalConvert;
import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.domain.ast.LoanInstal;
import com.abssqr.plat.common.model.domain.ast.LoanInstalDomain;
import com.abssqr.plat.common.model.repo.ast.LoanInstalRepository;
import com.general.system.common.model.Money;
import com.general.system.tool.util.ToolUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author guozhenhua
 * @version com.abssqr.plat.processor.core.service.loanTrans: AstRepayPlanRepositoryImpl.java, v 0.1 2017-12-12 下午7:16 guozhenhua Exp $
 */
@Component
public class LoanInstalRepositoryImpl implements LoanInstalRepository {
    
    @Autowired
    private LoanInstalDAO loanInstalDAO;
    
    @Override
    public int createOrUpdate(LoanInstal entity) {
        if (entity == null) {
            return 0;
        }
        
        LoanInstalDO loanInstalDO = LoanInstalConvert.convert2Do(entity);
        
        if (entity.isNew()) {
            return loanInstalDAO.insert(loanInstalDO);
        } else {
            return loanInstalDAO.update(loanInstalDO);
        }
    }
    
    @Override
    public List<LoanInstalDomain> getByAstNo(Loan loan) {
        return LoanInstalConvert.convert2Domains(loanInstalDAO.getByAstNo(loan.getAstNo()), loan);
    }
    
    /**
     * 获取 对应资产笔数
     * @param ownOrg
     * @param status
     * @return
     */
    @Override
    public Long getPlanAstCountByStatus(String ownOrg, List<LoanStatusEnum> status){
        List<String> list = null;
        if (ToolUtil.isNotEmpty(status)){
            list = status.stream().map(LoanStatusEnum::getCode).collect(Collectors.toList());
        }
        return loanInstalDAO.getPlanAstCount(ownOrg, list);
    }
    
    /**
     * desc:获取资产当前剩余分期数量及余额总计.<br/>
     * @param ownOrg ownOrg
     * @return List<LoanInstalFormationItem>
     */
    @Override
    public List<LoanInstalFormationItem> queryAstFormation(String ownOrg){
        return loanInstalDAO.queryAstFormation(ownOrg);
    }
    /**
     * desc:获取资产最大分期及余额总计.<br/>
     * @param ownOrg ownOrg
     * @return List<LoanInstalFormationItem>
     */
    @Override
    public List<LoanInstalFormationItem> queryProdFormation(String ownOrg){
        return loanInstalDAO.queryProdFormation(ownOrg);
    }
    
    /**
     * 根据资产原始机构获取 当前资产总规模
     * @param lastOwnOrg
     * @return
     */
    @Override
    public Money getPlanAstScale(String lastOwnOrg) {
        Long amt = loanInstalDAO.getPlanAstScale(lastOwnOrg);
        return Money.createWithCent(amt);
    }
    
    /**
     * desc:获取计划时间区间内应还汇总金额  = 已还本金 + 本金余额 + 账单利息
     * @param startDate startDate
     * @param endDate endDate
     * @param lastOwnOrg lastOwnOrg
     * @return Long
     */
    @Override
    public Money getPlanAstShouldScaleByEndDate(Date startDate, Date endDate, String lastOwnOrg){
        Long amt = loanInstalDAO.getPlanAstShouldScaleByEndDate(endDate, startDate, lastOwnOrg);
        return Money.createWithCent(amt);
    }
}
