/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.model.repo.clearing.impl;

import com.abssqr.plat.common.dal.base.StatEntity;
import com.abssqr.plat.common.dal.mysql.auto.dao.PlanClearingLogDAO;
import com.abssqr.plat.common.dal.mysql.auto.dao.PlanClearingLogHisDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanClearingLogDO;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.ClearingLogCntAndId;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.LoanClearingLogStat;
import com.abssqr.plat.common.facade.enums.CommonErrorCodeEnum;
import com.abssqr.plat.common.facade.enums.TransCodeEnum;
import com.abssqr.plat.common.model.convertor.PlanClearingLogConvert;
import com.abssqr.plat.common.model.domain.clearing.PlanClearingLog;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.repo.clearing.PlanClearingLogRepo;
import com.general.system.common.model.Money;
import com.general.system.common.util.MessageUtil;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

/**
 * @author zhenxuan.luo
 * @version com.abssqr.plat.common.model.repo.clearing.impl: PlanClearingLogRepoImpl.java, v 0.1 2019-06-10 00:27 zhenxuan.luo Exp $
 */
@Component
public class PlanClearingLogRepoImpl implements PlanClearingLogRepo {

    @Autowired
    private PlanClearingLogDAO    planClearingLogDAO;
    @Autowired
    private PlanClearingLogHisDAO planClearingLogHisDAO;

    @Override
    public int createBatch(List<PlanClearingLog> planClearingLogs) {
        if (CollectionUtils.isEmpty(planClearingLogs)) {
            return 0;
        }
        List<PlanClearingLogDO> clearingLogDOS = Lists.newArrayList();
        for (PlanClearingLog clearingLog : planClearingLogs) {
            clearingLogDOS.add(PlanClearingLogConvert.convert2DO(clearingLog));
        }
        return planClearingLogDAO.insertBatch(clearingLogDOS);
    }

    @Override
    public List<LoanClearingLogStat> clearingLogStat(Date accountDate, Long maxId, Long minId, String planNo,
                                                     String execTms) {
        return planClearingLogDAO.stat(accountDate, maxId, minId, planNo, execTms);
    }

    @Override
    public int deleteStated(Date accountDate, Long maxId, Long minId, String planNo, String execTms) {
        return planClearingLogDAO.deleteStated(accountDate, maxId, minId, planNo, execTms);
    }

    @Override
    public StatEntity getStatInx(Date accountDate, String planNo, String execTms) {
        return planClearingLogDAO.getStatInx(accountDate, planNo, execTms);
    }

    @Override
    public Date getMinUnStateAccountDate(String planNo, TransCodeEnum transCode) {
        String transCodeStr = null;
        if (null != transCode) {
            transCodeStr = transCode.getCode();
        }
        return planClearingLogDAO.getMinUnStateAccountDate(planNo, transCodeStr);
    }

    @Override
    public void backStated(String clearingSeqNo, String execTms, LoanClearingLogStat statRes) {
        int bakCnt = planClearingLogHisDAO.insertForBak(execTms, clearingSeqNo, statRes);
        if (bakCnt != statRes.getCnt()) {
            throw new AbssqrBizException(MessageUtil.formatMsg("备份ClearingLog失败，期望行数[{0}],实际行数[{1}]---[{2}]",
                    statRes.getCnt(), bakCnt, statRes), CommonErrorCodeEnum.INVALID_STATUS, true);
        }
    }

    /**
     * 查看清算流水是否都处理完
     *
     * @param planNo
     * @return
     */
    @Override
    public Long cntInitClearingLogByPlanNo(String planNo) {
        return planClearingLogDAO.cntInitClearingLogByPlanNo(planNo);
    }

    @Override
    public Long cntInitClearingLog( TransCodeEnum transCode) {
        return planClearingLogDAO.cntInitClearingLog(transCode.getCode());
    }

    @Override
    public ClearingLogCntAndId getHisMaxIdAndCnt(Date accountDate, String planNo, TransCodeEnum transCode) {
        return planClearingLogHisDAO.getCntAndMaxIdByPlanNoAndActDateAndTransCode(accountDate, planNo, transCode == null ? null : transCode.getCode());
    }

    @Override
    public Money getHisAmtByPlanNo(Date accountDate, String planNo, TransCodeEnum transCode){
       return planClearingLogHisDAO.getClearAmtByPlanNoAndActDateAndTransCode(accountDate, planNo, transCode == null ? null : transCode.getCode());
    }

    @Override
    public Money getClearAmtByCntPtyCode(Date accountDate,String cntPtyCode,TransCodeEnum transCode){
       return planClearingLogHisDAO.getClearAmtByCntPtyCodeAndActDateAndTransCode(accountDate, transCode==null?null:transCode.getCode(), cntPtyCode);
    }
}