/**
 * abssqr.comInc.
 * Copyright(c)2019-2019AllRightsReserved.
 */
package com.abssqr.plat.common.model.repo.dataChange.impl;

import com.abssqr.plat.common.dal.mysql.auto.dao.BizDataChangeDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.BizDataChangeDO;
import com.abssqr.plat.common.facade.enums.DataChangeBizTypeEnum;
import com.abssqr.plat.common.facade.model.BizDataChange;
import com.abssqr.plat.common.model.repo.dataChange.ChangeRepo;
import com.alibaba.fastjson.JSON;
import com.general.system.bpm.enums.ApproveStatusEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author yangcheng
 * @version ChangeRepoImpl.java, v0.12019-06-1410:27 yangchengExp$
 */
@Component
public class ChangeRepoImpl implements ChangeRepo {
    @Autowired
    private BizDataChangeDAO bizDataChangeDAO;

    @Override
    public void addChange(BizDataChange dataChange) {
        BizDataChangeDO dataChangeDO = convert2DO(dataChange);
        bizDataChangeDAO.insert(dataChangeDO);
    }

    /**
     * 根据changeNo获取
     *
     * @param changeNo
     * @return
     */
    @Override
    public BizDataChange getChange(String changeNo) {
        BizDataChangeDO dataChangeDO = bizDataChangeDAO.getByChangeNo(changeNo);
        return convert2Model(dataChangeDO);
    }


    /**
     * desc:根据bizNo、status和bizType查询审批数量.<br/>
     *
     * @param changeNo
     * @param approveStatus
     * @return
     */
    @Override
    public Long cntByBizNoAndStatusAndBizType(String bizType,String changeNo,String approveStatus) {
        return bizDataChangeDAO.cntByBizNoAndStatusAndBizType(bizType, changeNo, approveStatus);
    }

    @Override
    public boolean updateStatus(ApproveStatusEnum status, String changeNo, ApproveStatusEnum oldStatus) {
        int i = bizDataChangeDAO.updateStatus(changeNo, status.getCode(), oldStatus.getCode());
        return i > 0;
    }

    private BizDataChangeDO convert2DO(BizDataChange dataChange) {
        BizDataChangeDO entity = new BizDataChangeDO();
        //name 变更名称-同审批流名称
        entity.setName(dataChange.getName());
        //bizNo 业务编号
        entity.setBizNo(dataChange.getBizNo());
        //datas 变更后数据
        entity.setDatas(JSON.toJSONString(dataChange.getDatas()));
        //bizType 业务类型
        entity.setBizType(dataChange.getBizType().getCode());
        //changeNo 变更编号
        entity.setChangeNo(dataChange.getChangeNo());
        //approveStatus 最终审批状态   审批中，已通过，未通过
        entity.setApproveStatus(dataChange.getApproveStatus());
        return entity;
    }

    private BizDataChange convert2Model(BizDataChangeDO dataChange) {
        if (dataChange == null) {
            return null;
        }
        BizDataChange model = new BizDataChange();
        //name 变更名称-同审批流名称
        model.setName(dataChange.getName());
        //bizNo 业务编号
        model.setBizNo(dataChange.getBizNo());
        //bizType 业务类型
        model.setBizType(DataChangeBizTypeEnum.getEnum(dataChange.getBizType()));
        //datas 变更后数据
        model.setDatas(JSON.parseObject(dataChange.getDatas(), model.getBizType().getClazz()));
        //changeNo 变更编号
        model.setChangeNo(dataChange.getChangeNo());
        //approveStatus 最终审批状态   审批中，已通过，未通过
        model.setApproveStatus(dataChange.getApproveStatus());
        return model;
    }
}
