/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */

package com.abssqr.plat.common.model.repo.job;

import com.abssqr.plat.common.dal.mysql.auto.dao.JobControlDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.JobControlDO;
import com.abssqr.plat.common.facade.enums.YNEnum;
import com.abssqr.plat.common.model.domain.job.JobControl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * @author guozhenhua
 * @version com.abssqr.plat.common.model.repo.job: JobControlRepositoryImpl.java, v 0.1 2018-02-02 上午10:59 guozhenhua
 * Exp $
 */
@Component
public class JobControlRepositoryImpl implements JobControlRepository {

    @Autowired
    private JobControlDAO jobControlDAO;

    @Override
    public JobControl selectByJobName(String jobName) {

        JobControlDO jobControlDO = jobControlDAO.getByJobName(jobName);
        JobControl jobControl = convert2BO(jobControlDO);
        if (jobControl == null) {
            jobControl = new JobControl();
            jobControl.setJobName(jobName);
            jobControl.setIsEff(YNEnum.Y);
            insert(jobControl);
        }
        return jobControl;
    }
    
    @Override
    public JobControl lockByJobName(String jobName) {
        
        JobControlDO jobControlDO = jobControlDAO.lockByJobName(jobName);
        JobControl jobControl = convert2BO(jobControlDO);
        if (jobControl == null) {
            jobControl = new JobControl();
            jobControl.setJobName(jobName);
            jobControl.setIsEff(YNEnum.Y);
            insert(jobControl);
            jobControlDO = jobControlDAO.lockByJobName(jobName);
            jobControl = convert2BO(jobControlDO);
        }
        return jobControl;
    }

    @Override
    public int insert(JobControl jobControl) {
        if (null == jobControl) {
            return 0;
        }

        JobControlDO jobControlDO = new JobControlDO();
        jobControlDO.setJobName(jobControl.getJobName());
        jobControlDO.setIsEff(jobControl.getIsEff().isYes());

        return jobControlDAO.insert(jobControlDO);
    }

    @Override
    public int update(JobControl jobControl) {
        if (null == jobControl) {
            return 0;
        }

        JobControlDO jobControlDO = new JobControlDO();
        jobControlDO.setJobName(jobControl.getJobName());
        jobControlDO.setIsEff(jobControl.getIsEff().isYes());

        return jobControlDAO.update(jobControlDO);
    }

    private JobControl convert2BO(JobControlDO jobControlDO) {
        if (jobControlDO == null) {
            return null;
        }

        JobControl jobControl = new JobControl();
        jobControl.setJobName(jobControlDO.getJobName());
        jobControl.setIsEff(YNEnum.getByCode(jobControlDO.getIsEff()));
        return jobControl;
    }

}
