/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.model.repo.plan.impl;

import com.abssqr.plat.common.dal.mysql.auto.dao.PlanAllocDAO;
import com.abssqr.plat.common.dal.mysql.auto.dao.PlanAllocItemDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanAllocDO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanAllocItemDO;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.PlanAllocItemResultMap;
import com.abssqr.plat.common.facade.enums.AllocTypeEmun;
import com.abssqr.plat.common.facade.enums.PlanAllocStatusEnum;
import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.model.plan.PlanAcctTitle;
import com.abssqr.plat.common.facade.model.plan.PlanAlloc;
import com.abssqr.plat.common.facade.model.plan.PlanAllocItem;
import com.abssqr.plat.common.model.convertor.PlanAllocConvert;
import com.abssqr.plat.common.model.repo.plan.PlanAcctTitleRepo;
import com.abssqr.plat.common.model.repo.plan.PlanAllocRepo;
import com.general.system.bpm.enums.ApproveStatusEnum;
import com.general.system.common.model.Money;
import com.general.system.common.util.VarChecker;
import com.general.system.tool.util.ToolUtil;
import com.google.common.collect.Maps;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author bangis.wangdf
 * @version com.abssqr.plat.common.model.repo.plan.impl: PlanAllocRepoImpl.java, v 0.1 2019-06-03 16:55 bangis.wangdf
 * Exp $
 */
@Repository
public class PlanAllocRepoImpl implements PlanAllocRepo {
    @Autowired
    private PlanAllocDAO      planAllocDAO;
    @Autowired
    private PlanAllocItemDAO  planAllocItemDAO;
    @Autowired
    private PlanAcctTitleRepo planAcctTitleRepo;

    /**
     * 获取分配计划
     *
     * @param planNo
     * @return
     */
    @Override
    public List<PlanAlloc> queryByPlanNo(String planNo) {
        List<PlanAllocDO> planAllocDOS = planAllocDAO.queryByPlanNo(planNo);
        List<PlanAlloc> result = PlanAllocConvert.convertDomains(planAllocDOS);
        this.fullItems(planNo, result);
        return result;
    }
    
    /**
     * 获取未分配的分配计划
     *
     * @param planNo
     * @return
     */
    @Override
    public List<PlanAlloc> queryUnassigneAllocByPlanNo(String planNo) {
        List<PlanAllocDO> planAllocDOS = planAllocDAO.queryAllocByPlanNo(null, null, planNo,
                PlanAllocStatusEnum.UNASSIGNE.getCode());
        List<PlanAlloc> result = PlanAllocConvert.convertDomains(planAllocDOS);
        this.fullItems(planNo, result);
        return result;
    }
    
    @Override
    public void insertPlanAllocs(List<PlanAlloc> allocs) {
        if(CollectionUtils.isNotEmpty(allocs)) {
            List<PlanAllocDO> entites = PlanAllocConvert.convertDos(allocs);
            planAllocDAO.insertBatch(entites);

            List<PlanAllocItemDO> itemEntites = PlanAllocConvert.convertItemDos(allocs);
            planAllocItemDAO.insertBatch(itemEntites);
        }
    }

    @Override
    public PlanAlloc getPlanAllocDetail(String allocNo) {
        PlanAllocDO planAllocDO = planAllocDAO.getByAllocNo(allocNo);
        VarChecker.checkNotNull(planAllocDO, "[{0}]找不到对应的分配计划", allocNo);
        List<PlanAllocItemResultMap> itemResultMaps = planAllocItemDAO.getByAllocNoList(allocNo);
        PlanAlloc planAlloc = PlanAllocConvert.convertDomain(planAllocDO);
        if (CollectionUtils.isNotEmpty(itemResultMaps)) {
            planAlloc.setPlanAllocItemList(
                itemResultMaps.stream().map(resultMap -> PlanAllocConvert.convertItemDomain(resultMap)).collect(
                    Collectors.toList()));
        }
        return planAlloc;
    }

    @Override
    public void addOrUpdate(PlanAlloc alloc, boolean upd) {
        PlanAllocDO planAllocDO = PlanAllocConvert.convert2Do(alloc);
        if (upd) {
            planAllocDAO.updateByAllocNo(planAllocDO);
        } else {
            planAllocDAO.insert(planAllocDO);
        }
        //分配事项不为空则更新事项
        if (CollectionUtils.isNotEmpty(alloc.getPlanAllocItemList())) {
            //更新事项列表
            this.allocTitleItemUpdate(alloc);
        }
    }
    
    /**
     * 更新事项
     * @param alloc
     */
    private void allocTitleItemUpdate(PlanAlloc alloc){
        //先获取之前金额，在删除
        Map<String, Long> titleSettleAmtMap = planAllocItemDAO.getByAllocNo(alloc.getAllocNo());
        planAllocItemDAO.deleteByAllocNo(alloc.getAllocNo());
        if (CollectionUtils.isNotEmpty(alloc.getPlanAllocItemList())) {
            List<PlanAllocItemDO> itemEntites = PlanAllocConvert.convertItemDomain2Dos(alloc.getPlanNo(),
                    alloc.getAllocNo(), alloc.getPlanAllocItemList(), titleSettleAmtMap);
            planAllocItemDAO.insertBatch(itemEntites);
        }
    }
    
    /**
     * 删除分配方案
     *
     * @param allocNo
     * @return
     */
    @Override
    public int delete(String allocNo) {
        int itemCnt = planAllocItemDAO.deleteByAllocNo(allocNo);
        int cnt = planAllocDAO.deleteByAllocNo(allocNo);
        return itemCnt + cnt;
    }

    /**
     * 删除分配事项
     *
     * @param itemNo
     * @return
     */
    @Override
    public int deleteAllocItem(String itemNo) {
        return planAllocItemDAO.deleteByItemNo(itemNo);
    }
    /**
     * 分配计划改为已分配
     *
     * @param allocNo
     * @param allocStatus
     * @return
     */
    @Override
    public Boolean planAllocDetailAssigned(String allocNo, PlanAllocStatusEnum allocStatus) {
        PlanAllocDO allocDO = planAllocDAO.getByAllocNo(allocNo);
        int i = planAllocDAO.updateStatus(allocStatus.getCode(), allocNo, allocDO.getStatus());
        return i == 1;
    }
    
    @Override
    public PlanAlloc queryRecentlyPlans(PlanTypeEnum planType) {
        PlanAllocDO allocDO = planAllocDAO.queryRecentlyPlans(planType.getCode());
        PlanAlloc planAlloc = PlanAllocConvert.convertDomain(allocDO);
        return planAlloc;
    }
    
    @Override
    public int updateAllocStatus(PlanAlloc alloc){
        PlanAllocDO planAllocDO = PlanAllocConvert.convert2Do(alloc);
        return planAllocDAO.updateByAllocNo(planAllocDO);
    }
    
    @Override
    public int querySameClearDate(Date clearDate,String planNo, String allocNo){
        return planAllocDAO.querySameClearDate(clearDate, planNo, allocNo);
    }
    
    @Override
    public PlanAlloc lockByAllocNo(String allocNo) {
        PlanAllocDO planAllocDO = planAllocDAO.lockByAllocNo(allocNo);
        return PlanAllocConvert.convertDomain(planAllocDO);
    }
    
    /**
     * 查询审批通过并且未分配的时间段分配金额统计
     * @param startDate
     * @param endDate
     * @param planNo
     * @return
     */
    @Override
    public Money queryStatAllocAmtByPlanNoAndPayDate(Date startDate, Date endDate, String planNo) {
        Long amt = planAllocDAO.queryStatAllocAmtByPlanNoAndPayDate(endDate, startDate, planNo,
                PlanAllocStatusEnum.UNASSIGNE.getCode(),
                ApproveStatusEnum.PASSED.getCode());
        return Money.createWithCent(amt);
    }
    
    /**
     * 根据计划编号获取累计分配金额
     * @param planNos
     * @return
     */
    @Override
    public Money queryStatTotalAllocAmtByPlanNos(List<String> planNos) {
        if (ToolUtil.isEmpty(planNos)){
            return new Money();
        }
        Long amt = planAllocDAO.queryStatTotalAllocAmtByPlanNos(null, null,
                PlanAllocStatusEnum.UNASSIGNE.getCode(),
                ApproveStatusEnum.PASSED.getCode(), planNos);
        return Money.createWithCent(amt);
    }

    @Override
    public void insertPlanAllocItem(PlanAllocItem allocItem, String planNo, String allocNo) {
        PlanAllocItemDO planAllocItemDO = PlanAllocConvert.convertItemDo(allocItem, planNo, allocNo);
        planAllocItemDAO.insert(planAllocItemDO);
    }


    @Override
    public void cleanByPlanNo(String planNo){
        if (planNo != null) {
            planAllocDAO.deleteByPlanNo(planNo);
            planAllocItemDAO.deleteByPlanNo(planNo);
        }
    }

    @Override
    public List<PlanAlloc> queryUnMatchedApprovedAlloc(String planNo) {
        List<PlanAllocDO> planAllocDOS = planAllocDAO.queryUnMatchedApprovedAlloc(planNo);
        List<PlanAlloc> planAllocs = PlanAllocConvert.convertDomains(planAllocDOS);
        return planAllocs;
    }


    private void fullItems(String planNo, List<PlanAlloc> result){
        if (CollectionUtils.isNotEmpty(result)) {
            //获取分配明细
            Map<String, List<PlanAllocItemResultMap>> allocItemMapList = planAllocItemDAO.getByPlanNo(planNo);
            result.forEach(planAlloc -> {
                List<PlanAllocItemResultMap> itemResultMaps = allocItemMapList.get(planAlloc.getAllocNo());
                if (CollectionUtils.isNotEmpty(itemResultMaps)) {
                    planAlloc.setPlanAllocItemList(
                            itemResultMaps.stream().map(resultMap -> PlanAllocConvert.convertItemDomain(resultMap)).collect(
                                    Collectors.toList()));
                }
            });
        }
    }


    @Override
    public Map<String,Money> queryApprovedNotPayMoneyGroupByLayerNo(String planNo){
        List<PlanAcctTitle> byEntityNo = planAcctTitleRepo.getByEntityNo(planNo);
        Map<String, PlanAcctTitle> titleNosMap = byEntityNo.stream()
                .collect(Collectors.toMap(PlanAcctTitle::getTitleNo,item->item));
        Map<String,Money> titlePayMoneyMap = Maps.newHashMap();
        List<PlanAllocDO> planAllocDOS = planAllocDAO.queryApprovedNotPayAlloc(planNo);
        if (CollectionUtils.isNotEmpty(planAllocDOS)) {
            List<PlanAlloc> planAllocs = PlanAllocConvert.convertDomains(planAllocDOS);
            fullItems(planNo, planAllocs);
            planAllocs.forEach(alloc -> {
                alloc.getPlanAllocItemList().forEach(item -> {
                    if (item.getAllocType() == AllocTypeEmun.PRIN) {

                        Money money = titlePayMoneyMap.get(item.getRelNo());
                        if (money == null) {
                            money = new Money();
                            PlanAcctTitle planAcctTitle = titleNosMap.get(item.getTitleNo());
                            VarChecker.checkNotNull(planAcctTitle, "[PlanAllocRepoImpl] - 科目[{0}]不存在", item.getTitleNo());
                            titlePayMoneyMap.put(planAcctTitle.getRefNo(), money);
                        }
                        money.addTo(item.getConfirmAmt());
                    }

                });
            });
        }
        return titlePayMoneyMap;
    }
}
