/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */

package com.abssqr.plat.common.model.repo.plan.impl;

import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import com.abssqr.plat.common.dal.mysql.auto.dao.AstScreenRuleDAO;
import com.abssqr.plat.common.dal.mysql.auto.dao.PlanTranScreenRelDAO;
import com.abssqr.plat.common.dal.mysql.auto.dao.PlanTransferRuleDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.AstScreenRuleDO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanTranScreenRelDO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.PlanTransferRuleDO;
import com.abssqr.plat.common.facade.model.plan.PlanTransferRule;
import com.abssqr.plat.common.facade.model.rule.AstScreenRule;
import com.abssqr.plat.common.model.convertor.PlanTransferRuleConvert;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.common.model.repo.plan.PlanTransferRuleRepo;
import com.abssqr.plat.common.model.repo.rule.AstScreenRuleRepo;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import org.apache.commons.collections.CollectionUtils;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

/**
 * @author yaphet
 * @version com.abssqr.admin.common.model.repo.rule: TransferRuleRepoImpl.java, v 0.1 2018-02-27 下午7:40 yaphet Exp $
 */
@Repository
public class PlanTransferRuleRepoImpl implements PlanTransferRuleRepo {
    @Autowired
    PlanTransferRuleDAO  planTransferRuleDAO;
    @Autowired
    PlanTranScreenRelDAO planTranScreenRelDAO;
    @Autowired
    AstScreenRuleDAO     astScreenRuleDAO;
    @Autowired
    AstScreenRuleRepo    astScreenRuleRepo;
    @Autowired
    PlanRepo             planRepo;

    @Override
    public PlanTransferRule getByPlanNo(String planNo) {
        PlanTransferRuleDO planTransferRuleDO = planTransferRuleDAO.getByPlanNo(planNo);
        if (planTransferRuleDO == null) {
            return null;
        }
        List<PlanTranScreenRelDO> tranScreenRelDOS = planTranScreenRelDAO.getByTransferNo(
            planTransferRuleDO.getTransferNo());
        List<AstScreenRule> astScreenRules = Lists.newArrayList();
        if (CollectionUtils.isNotEmpty(tranScreenRelDOS)) {
            List<String> screenRuleNos = tranScreenRelDOS.stream().map(PlanTranScreenRelDO::getScreenRuleNo).collect(
                Collectors.toList());
            astScreenRules = astScreenRuleRepo.getByNos(screenRuleNos);
        }

        return PlanTransferRuleConvert.convert2TransferRuleDomain(planTransferRuleDO, astScreenRules);
    }


    @Override
    public void add(String planNo, PlanTransferRule transferRule) {
        if (transferRule != null) {
            PlanTransferRuleDO planTransferRuleDO = PlanTransferRuleConvert.convert2Do(planNo, transferRule);
            planTransferRuleDAO.insert(planTransferRuleDO);
            dealRel(planNo, planTransferRuleDO.getTransferNo(), transferRule.getAstScreenRuleList());
        }
    }

    @Override
    public void update(String planNo, PlanTransferRule transferRule) {
        PlanTransferRuleDO planTransferRuleDO = PlanTransferRuleConvert.convert2Do(planNo, transferRule);
        int cnt = planTransferRuleDAO.update(planTransferRuleDO);
        VarChecker.checkEquals(cnt, 1, "计划[{0}],transferNo未提交", planNo);
        dealRel(planNo, planTransferRuleDO.getTransferNo(), transferRule.getAstScreenRuleList());
    }

    @Override
    public void deleteByPlanNo(String planNo){
        PlanTransferRuleDO planTransferRuleDO = planTransferRuleDAO.getByPlanNo(planNo);
        if (planTransferRuleDO == null) {
            return ;
        }
        planTransferRuleDAO.deleteByPlanNo(planNo);
        dealRel(planNo,planTransferRuleDO.getTransferNo(),null);
    }
    /**
     * 计划筛选规则处理
     *
     * @param planNo
     * @param screeningRules
     */
    private void dealRel(String planNo, String transferNo, List<AstScreenRule> screeningRules) {
        planTranScreenRelDAO.deleteByTransferNo(transferNo);
        if (CollectionUtils.isNotEmpty(screeningRules)) {
            List<String> collect = screeningRules.stream().map(AstScreenRule::getScreenRuleNo).collect(
                Collectors.toList());
            Map<String, AstScreenRuleDO> screenNoPordMap = astScreenRuleDAO.getByScreenRuleNos(collect);
            Set<String> managerOrgCodeSet = Sets.newHashSet();
            List<PlanTranScreenRelDO> planTranScreenRelDOS = collect.stream().map(
                screenNo -> {
                    PlanTranScreenRelDO relDO = new PlanTranScreenRelDO();
                    relDO.setPlanNo(planNo);
                    relDO.setScreenRuleNo(screenNo);
                    relDO.setTransferNo(transferNo);
                    VarChecker.checkNotNull(screenNoPordMap.get(screenNo), "[{0}]筛选规则找不到", screenNo);
                    relDO.setProdCode(screenNoPordMap.get(screenNo).getAbssqrProdCode());
                    relDO.setManagerOrgCode(screenNoPordMap.get(screenNo).getManagerOrgCode());
                    managerOrgCodeSet.add(relDO.getManagerOrgCode());
                    return relDO;
                }).collect(Collectors.toList());
            VarChecker.checkEquals(1, managerOrgCodeSet.size(), "仅支持同一服务机构下转让计划筛选规则");
            planTranScreenRelDAO.insertBatch(planTranScreenRelDOS);
        }
    }
}
