/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */

package com.abssqr.plat.common.model.repo.prod.impl;

import com.abssqr.plat.common.dal.mysql.auto.dao.OrganizationDAO;
import com.abssqr.plat.common.dal.mysql.auto.dao.ProductDAO;
import com.abssqr.plat.common.dal.mysql.auto.dao.ProductRiskClDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.ProductDO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.ProductRiskClDO;
import com.abssqr.plat.common.dal.mysql.auto.paging.ProductListPage;
import com.abssqr.plat.common.facade.model.prod.LoanProduct;
import com.abssqr.plat.common.facade.model.prod.ProductRiskCl;
import com.abssqr.plat.common.model.convertor.ProdRiskClConvert;
import com.abssqr.plat.common.model.convertor.ProductConvert;
import com.abssqr.plat.common.model.repo.prod.LoanProdRepository;
import com.abssqr.plat.common.util.utils.BeanCopierUtil;
import com.google.common.collect.Lists;
import com.google.common.collect.Sets;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 产品仓储
 *
 * @author guozhenhua
 * @version com.abssqr.plat.common.model.repo.prod: LoanProdRepositoryImpl.java, v 0.1 2018-01-05 下午7:56 guozhenhua
 * Exp $
 */
@Component
public class LoanProdRepositoryImpl implements LoanProdRepository {
    private Logger logger = LoggerFactory.getLogger(getClass());
    
    @Autowired
    private ProductDAO         productDAO;
    @Autowired
    private ProductRiskClDAO   productRiskClDAO;
    @Autowired
    private OrganizationDAO    organizationDAO;

    /**
     * 查询产品分页
     * @param param
     * @return
     */
    @Override
    public ProductListPage getProdList(ProductListPage param) {
        return productDAO.getProductList(param);
    }
    
    @Override
    public LoanProduct getLoanProdByPK(String prodCode, String managerOrgCode) {
        ProductDO productDO = productDAO.getByOrgCodeProdCode(prodCode, managerOrgCode);
        return fillLoanProductRiskCl(productDO);
    }
    
    /**
     * 组装五级分类的产品
     * @param abssqrProdCode
     * @return
     */
    @Override
    public LoanProduct getBybssqrProdCode(String abssqrProdCode) {
        ProductDO productDO = productDAO.getByAbssqrProdCode(abssqrProdCode);
        return fillLoanProductRiskCl(productDO);
    }
    
    @Override
    public List<LoanProduct> getBybssqrProdCodes(List<String> abssqrProdCodes) {
        if (CollectionUtils.isEmpty(abssqrProdCodes)) {
            return Lists.newArrayList();
        }
        List<ProductDO> dos = productDAO.getByAbssqrProdCodes(abssqrProdCodes);
        return ProductConvert.convertDTOs(dos);
    }
    
    /**
     * 新增产品
     * @param loanProduct
     * @return
     */
    @Override
    public Integer addProduct(LoanProduct loanProduct) {
        logger.info("新增产品" + loanProduct.getAbssqrProdCode());
        ProductDO productDO = loanProduct2DO(loanProduct);
        return productDAO.insert(productDO);
    }
    
    /**
     * 修改产品
     * @param loanProduct
     * @return
     */
    @Override
    public Integer update(LoanProduct loanProduct) {
        logger.info("修改产品" + loanProduct.getAbssqrProdCode());
        ProductDO productDO = loanProduct2DO(loanProduct);
        return productDAO.update(productDO);
    }
    
    @Override
    public Integer deleteProductByCode(String absProdCode) {
        return productDAO.deleteByAbssqrProdCode(absProdCode);
    }

    @Override
    public LoanProduct getProductByAbsProductNo(String absProductNo) {
        logger.info("查询产品" + absProductNo);
        ProductDO productDO = productDAO.getByAbssqrProdCode(absProductNo);
        LoanProduct loanProduct = productDO2BO(productDO);
        return loanProduct;
    }
    
    /**
     * 查看是否存在该产品
     * @param loanProduct
     * @return
     */
    @Override
    public Integer getSameProdCnt(LoanProduct loanProduct) {
        ProductDO productDO = loanProduct2DO(loanProduct);
        return productDAO.getSameProdCnt(productDO);
    }
    
    private ProductDO loanProduct2DO(LoanProduct loanProduct) {
        ProductDO productDO = BeanCopierUtil.copy(ProductDO.class, loanProduct);
        return productDO;
    }

    private LoanProduct productDO2BO(ProductDO productDO) {
        LoanProduct loanProduct = BeanCopierUtil.copy(LoanProduct.class, productDO);
        return loanProduct;
    }

    private LoanProduct fillLoanProductRiskCl(ProductDO productDO) {
        if (productDO==null){
            return null;
        }
        LoanProduct loanProduct = ProductConvert.convertDTO(productDO);
        Set<String> orgCodes = Sets.newHashSet();
        orgCodes.add(productDO.getManagerOrgCode());
        if (StringUtils.isNotBlank(productDO.getLenderOrgCode())) {
            orgCodes.add(productDO.getLenderOrgCode());
        }
        Map<String, String> orgNames = organizationDAO.getOrgNames(Lists.newArrayList(orgCodes));
        loanProduct.setLenderOrgName(orgNames.get(productDO.getLenderOrgCode()));
        loanProduct.setManagerOrgName(orgNames.get(productDO.getManagerOrgCode()));
        if (loanProduct != null) {
            List<ProductRiskClDO> productRiskClDOS = productRiskClDAO.queryByAbsProdCode(
                loanProduct.getAbssqrProdCode());
            List<ProductRiskCl> productRiskCls = ProdRiskClConvert.convertDTOs(productRiskClDOS);
            loanProduct.setProductRiskCls(productRiskCls);
        }
        return loanProduct;
    }
}
