/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.common.model.repo.report;

import com.abssqr.plat.common.facade.enums.ReportTypeEnum;
import com.abssqr.plat.common.facade.model.report.PlanReport;
import com.abssqr.plat.common.facade.model.report.PlanReportItem;
import com.abssqr.plat.common.facade.model.report.ReportField;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.common.model.repo.report: PlanReportRepo.java, v 0.1 2019-06-22 18:46 zhenxuan.luo Exp $ 
 */
public interface PlanReportRepo {
    
    /**
     * 创建计划报表
     * @param planReport
     * @return
     */
    int create(PlanReport planReport);
    
    /**
     * 获取最新报告
     *
     * @param planNo
     * @param rptType
     * @return
     */
    PlanReport getLatestReport(String planNo, ReportTypeEnum rptType);
    
    /**
     * 按报表日期获取计划 日报或月报
     * @param planNo
     * @param rptType
     * @param rptDate
     * @return
     */
    PlanReport getReport(String planNo, ReportTypeEnum rptType, String rptDate);
    
    /**
     * 获取计划日期时间段数据
     * @param planNo
     * @param startDate
     * @param endDate
     * @return
     */
    List<PlanReportItem> getItemsByDate(String planNo, Date startDate, Date endDate);
    
    /**
     *  获取报表list
     *
     * @param planNo
     * @param startDt
     * @param endDt
     * @return
     */
    List<PlanReport> getPlansByDate(String planNo, String startDt, String endDt, ReportTypeEnum rptType);
    
    /**
     *
     * @param planNo
     * @param rptType
     * @param dateList
     * @return
     */
    List<PlanReport> getPlansByDateList(String planNo, ReportTypeEnum rptType, List<String> dateList);
    
    /**
     * 根据计划和日期获取多个项统计值
     * @param accountDate
     * @param planNo
     * @param fields
     * @return
     */
    Map<String,PlanReportItem> queryRptItemsByPlanNoAndAccDate(Date accountDate, String planNo, List<ReportField> fields);
    
    /**
     * 根据计划和日期获取区间多个项统计值
     * @param startDate
     * @param endDate
     * @param planNo
     * @param fields
     * @return
     */
    Map<String,PlanReportItem> queryRptItemsByPlanNoSection(Date startDate, Date endDate, String planNo,
                                                            List<ReportField> fields);
    
    /**
     * 根据计划和日期区间获取多个统计项的每日数据
     * @param startDate
     * @param endDate
     * @param fields
     * @param listPlanNo
     * @return
     */
    List<PlanReportItem> queryDayRptItemsByPlanNoSection(Date startDate, Date endDate, List<ReportField> fields,
                                                           List<String> listPlanNo);
    
    /**
     * 根据计划和日期获取区间最小日期
     * @param endDate
     * @param startDate
     * @param planNo
     * @param fields
     * @return
     */
    Date queryReportMinDateByPlanNoSection(Date startDate, Date endDate, String planNo, List<ReportField> fields);
    
    /**
     * 给定计划是否完成
     * @param planNos
     * @param rptDate
     * @return
     */
    boolean allFinished(List<String> planNos, String rptDate,ReportTypeEnum rptType);
    
    /**
     * 初始化报表基础数据
     * @param planNo
     * @param accountDate
     * @return
     */
    int defaultRPT001Init(String planNo, Date accountDate);
}