/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2017 All Rights Reserved.
 */
package com.abssqr.plat.common.model.seq;

import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.StringFormatUtil;
import com.general.system.common.util.SystemDateUtil;
import org.springframework.stereotype.Component;

import java.text.NumberFormat;
import java.util.Date;

import static com.general.system.common.util.MessageUtil.formatMsg;

/**
 * 流水号生成器
 *
 * @author zhenxuan.luo
 * @version $Id: SequenceGenerator.java, v 0.1 2017-06-22 下午15:06 zhenxuan.luo Exp $
 */
@Component
class SequenceGenerator {
    private static final NumberFormat LOANID_FORMAT = NumberFormat.getNumberInstance();
    
    private static final NumberFormat LOANID_SEQ_FORMAT = NumberFormat.getNumberInstance();
    
    static {
        LOANID_FORMAT.setMinimumIntegerDigits(16);
        LOANID_FORMAT.setMaximumIntegerDigits(16);
        LOANID_FORMAT.setGroupingUsed(false);
        LOANID_SEQ_FORMAT.setMinimumIntegerDigits(6);
        LOANID_SEQ_FORMAT.setMaximumIntegerDigits(6);
        LOANID_SEQ_FORMAT.setGroupingUsed(false);
    }
    
    public String genLoanSeqNo(String astNo, Long id, String eventCode) {
        
        LoanSeqStructure seqStrut = new LoanSeqStructure(astNo, SystemDateUtil.getSystemDate(), eventCode, id);
        return seqStrut.toString();
    }
    
    public String genLoanId(String astNo, Long id) {
        LoanIdStructure idStut = new LoanIdStructure(astNo, SystemDateUtil.getSystemDate(), id);
        return idStut.toString();
    }
    
    /**
     * 生成流水号
     *
     * @param seqName
     * @param id
     * @param eventCode
     * @return
     */
    public String genSeqNo(String seqName, Long id, String eventCode) {
        
        SeqStructure seqStrut = new SeqStructure(SystemDateUtil.getSystemDate(),
                seqName, eventCode, id);
        
        return seqStrut.toString();
    }
    
    /**
     * 生成id
     *
     * @param idName
     * @return
     */
    public String genId(String idName, Long id) {
        IdStructure idStut = new IdStructure(SystemDateUtil.getSystemDate(),
                idName, id);
        return idStut.toString();
    }
    
    class IdStructure {
        protected Date   curTime;
        protected String idName;
        protected long   seqNo;
        
        public IdStructure(Date curTime, String idName,
                           long seqNo) {
            
            this.curTime = curTime;
            this.idName = idName;
            this.seqNo = seqNo;
        }
        
        @Override
        public String toString() {
            return formatMsg("{0}{1}{2}",
                    // id名称
                    idName,
                    // 时间元素yyyyMMddHHmmss, 14位
                    DateTimeUtil.formatYMDHMSShort(curTime),
                    // 随机滚动流水，一般为8位
                    StringFormatUtil.formatTransSeqByNumber(seqNo));
        }
    }
    
    class SeqStructure {
        
        protected Date   curTime;
        protected String seqName;
        protected String transCode;
        protected long   seqNo;
        
        public SeqStructure(Date curTime, String seqName, String transCode,
                            long seqNo) {
            this.curTime = curTime;
            this.seqName = seqName;
            this.transCode = transCode;
            this.seqNo = seqNo;
        }
        
        @Override
        public String toString() {
            return formatMsg("{0}{1}{2}{3}",
                    // code名称
                    seqName,
                    transCode,
                    DateTimeUtil.formatYMDHMSShort(curTime),
                    // 随机滚动流水，一般为8位
                    StringFormatUtil.formatTransSeqByNumber(seqNo));
        }
    }
    
    class LoanIdStructure extends IdStructure {
        private String astNo;
        
        public LoanIdStructure(String astNo, Date curTime, long seqNo) {
            super(curTime, "", seqNo);
            this.astNo = astNo;
        }
        
        @Override
        public String toString() {
            return formatMsg("{0}{1}{2}",
                    astNo,
                    // 时间元素yyyyMMddHH, 8位
                    DateTimeUtil.format(curTime, "yyyyMMddHH"),
                    // 随机滚动流水，一般为6位
                    LOANID_SEQ_FORMAT.format(seqNo));
        }
    }
    
    class LoanSeqStructure extends SeqStructure {
        
        private String astNo;
        
        public LoanSeqStructure(String astNo, Date curTime, String transCode,
                                long seqNo) {
            super(curTime, "", transCode, seqNo);
            this.astNo = astNo;
        }
        
        @Override
        public String toString() {
            return formatMsg("{0}{1}{2}{3}",
                    astNo,
                    transCode,
                    // 时间元素yyyyMMddHH, 8位
                    DateTimeUtil.format(curTime, "yyyyMMddHH"),
                    // 随机滚动流水，一般为6位
                    LOANID_SEQ_FORMAT.format(seqNo));
        }
    }
}