/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2018 All Rights Reserved. 
 */
package com.abssqr.plat.common.util.interceptor;

import com.google.common.collect.Lists;
import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;

import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 抽象方法拦截器
 *
 * @author zhenxuan.luo
 * @version com.loan.common.util: AbstractInterceptor.java, v 0.1 2018-12-19 AM11:39 zhenxuan.luo Exp $ 
 */
public abstract class AbstractInterceptor implements MethodInterceptor {
    
    /** Object原生方法 */
    private static final Map<String, Method> OBJECT_METHOD_MAP = new HashMap<String, Method>();
    
    private static final List<Class<?>> PRICLZ = Lists.newArrayList();
    
    static {
        //初始化Object 默认方法
        for (Method method : Object.class.getMethods()) {
            OBJECT_METHOD_MAP.put(method.getName(), method);
        }
    
        PRICLZ.add(String.class);
        PRICLZ.add(Integer.class);
        PRICLZ.add(Byte.class);
        PRICLZ.add(Long.class);
        PRICLZ.add(Double.class);
        PRICLZ.add(Float.class);
        PRICLZ.add(Character.class);
        PRICLZ.add(Short.class);
        PRICLZ.add(BigDecimal.class);
        PRICLZ.add(BigInteger.class);
        PRICLZ.add(Boolean.class);
    }
    
    /**
     *
     * @see MethodInterceptor#invoke(MethodInvocation)
     */
    @Override
    public Object invoke(MethodInvocation invocation) throws Throwable {
        if (OBJECT_METHOD_MAP.containsKey(invocation.getMethod().getName())) {
            return invocation.proceed();
        }
        //调用子类逻辑
        return businessInvoke(invocation);
    }
    
    public boolean isPrimitive(Class<?> clz) {
        return PRICLZ.contains(clz) || clz.isPrimitive();
    }
    
    /**
     * 子类具体拦截逻辑
     *
     * @param invocation 方法调用
     * @return 调用结果
     * @throws Throwable 任何异常
     */
    public abstract Object businessInvoke(MethodInvocation invocation) throws Throwable;
}