/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.core.service.ast;

import com.abssqr.plat.common.facade.base.DebtMoney;
import com.abssqr.plat.common.facade.enums.LoanStatusEnum;
import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.domain.ast.LoanInstal;
import com.abssqr.plat.common.model.domain.ast.LoanInstalDomain;
import com.abssqr.plat.common.model.domain.astSync.BaseTsfRes;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncRepayPlanTrans;
import com.abssqr.plat.common.model.domain.astSync.LoanAstSyncTrans;
import com.abssqr.plat.common.model.domain.loanTrans.LoanInstalTransDetail;
import com.abssqr.plat.common.model.domain.loanTrans.LoanTransDetail;
import com.abssqr.plat.core.service.repo.AstFlowOutComponent;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.MessageUtil;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.Map;

/**
 * 减免处理
 *
 * @author zhenxuan.luo
 * @version LoanAstExmptManager.java, v 0.1 2019-06-17 20:07 zhenxuan.luo Exp $
 */
@Component
public class LoanAstExmptManager extends LoanAstBaseManager {
    
    @Autowired
    private LoanStatusManager loanStatusManager;
    
    @Autowired
    private AstFlowOutComponent astFlowOutComponent;
    
    @Override
    protected Loan doProcess(BaseTsfRes baseTsfRes, LoanAstSyncTrans loanAstSyncTrans,
                             LoanTransDetail loanTransDetail) {
        
        Loan loan = loanTransDetail.getLoan();
        if (CollectionUtils.isEmpty(loanAstSyncTrans.getRepayPlanTransList())) {
            return loan;
        }
        Map<Integer, LoanInstalDomain> loanInstalMap = loan.getLoanInstalMap();
        
        for (LoanAstSyncRepayPlanTrans repayPlanTrans : loanAstSyncTrans.getRepayPlanTransList()) {
            LogUtil.info(LOGGER, "减免息费[{0}]", repayPlanTrans);
            LoanInstalDomain refInstal = loanInstalMap.get(repayPlanTrans.getTermNo());
            
            if (refInstal.getStatus() == LoanStatusEnum.CLEAR) {
                continue;
            }
            
            LoanInstalTransDetail loanInstalTransDetail = loanTransDetail.addLoanInstalTransDtl(refInstal);
            
            // 1. 处理金额
            loanInstalTransDetail.processTransAmt(repayPlanTrans.getAmt());
            astFlowOutComponent.flowOutInstal(loanInstalTransDetail, repayPlanTrans.getAmt(), new DebtMoney(), true);
            
            // 2. 处理结清
            if (isNeedSetClear(refInstal, loanTransDetail.getActiveLog().getAccountDate())) {
                refInstal.getBal().checkZero(MessageUtil.formatMsg(
                        "结清分期的余额必须为0. [{0}-{1}]", refInstal.getAstNo(), refInstal.getTermNo()));
                loanInstalTransDetail.setClear(loanAstSyncTrans.getAccountDate());
            }
        }
        
        loanStatusManager.calcLoanRiskStatus(loanTransDetail, false);
        return loanTransDetail.getLoan();
    }
    
    private boolean isNeedSetClear(LoanInstal instal, Date accountDate) {
        
        // 资产未到期，不设置结清
        if (DateTimeUtil.calcDayDiff(accountDate, instal.getAstPeriod().getEndDate()) < 0) {
            return false;
        }
        
        if (instal.getBal().getSum().isZero()) {
            return true;
        }
        
        return false;
    }
}