/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */

package com.abssqr.plat.core.service.plan;

import com.abssqr.plat.common.facade.enums.DataChangeBizTypeEnum;
import com.abssqr.plat.common.facade.enums.PriceTypeEnum;
import com.abssqr.plat.common.facade.model.BizDataChange;
import com.abssqr.plat.common.facade.model.plan.Plan;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.param.plan.priceRule.PlanAstPriceRuleParam;
import com.abssqr.plat.common.model.repo.plan.PlanAstPriceRuleRepo;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.common.model.utils.ApproveChangeUtil;
import com.abssqr.plat.core.service.change.ApproveCallBack;
import com.abssqr.plat.core.service.change.ChangeManager;
import com.general.system.bpm.enums.ApproveStatusEnum;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * 资产定价规则
 *
 * @author xiachenxiang
 * @version com.abssqr.plat.core.service.plan: PlanAstPriceRuleManager.java, v 0.1 2019-11-07 2:47 PM xiachenxiang Exp $
 */
@Component
public class PlanAstPriceRuleManager implements ApproveCallBack<PlanBase> {
    @Autowired
    private PlanRepo planRepo;
    @Autowired
    private PlanManager planManager;
    @Autowired
    private PlanAstPriceRuleRepo planAstPriceRuleRepo;
    @Autowired
    private ChangeManager changeManager;

    /**
     * 更新规则
     *
     * @param param
     * @param userCode
     * @param orgCode
     * @return
     */
    public Void updateRule(PlanAstPriceRuleParam param, String userCode, String orgCode) {

        PlanBase planBase = planManager.getPlanFullByNo(param.getPlanNo());
        VarChecker.checkNotNull(planBase, "计划[{0}]不存在", param.getPlanNo());
        if (planBase instanceof Plan) {
            // 调用数据变更 提交审批
            DataChangeBizTypeEnum dataChangeBizType = DataChangeBizTypeEnum.PanAstPriceRule;
            if (param.getPlanAstPriceRuleList() != null) {
                param.getPlanAstPriceRuleList().forEach(item -> item.setPlanNo(param.getPlanNo()));
            }
            planBase.setPriceType(PriceTypeEnum.getByCode(param.getPriceType()));
            planBase.setPlanAstPriceRuleList(param.getPlanAstPriceRuleList());
            //提交审批
            changeManager
                    .addApprove(dataChangeBizType, planBase.getPlanNo(), planBase.getPlanName(), planBase, userCode, orgCode);
        }
        return null;
    }


    @Override
    public void approveCallBack(BizDataChange<PlanBase> dataChange, ApproveStatusEnum approveStatus) {
        //审批通过后 创建
        PlanBase param = dataChange.getDatas();
        PlanBase dbPlan = planRepo.lockPlanByNo(param.getPlanNo());

        VarChecker.checkNotNull(dbPlan, "[{0}]计划信息不存在-审批失败", param.getPlanNo());

        if (ApproveChangeUtil.callBackSucceed(approveStatus)) {
            // 1. 更新定价类型
            planRepo.updatePirceType(param.getPlanNo(), param.getPriceType());
            // 2.批量删除
            planAstPriceRuleRepo.deleteByPlanNo(dbPlan.getPlanNo());
            // 3.批量插入
            if (CollectionUtils.isNotEmpty(param.getPlanAstPriceRuleList())) {
                planAstPriceRuleRepo.insertBatch(param.getPlanAstPriceRuleList());
            }
        }
    }

    @Override
    public DataChangeBizTypeEnum getChangeType() {
        return DataChangeBizTypeEnum.PanAstPriceRule;
    }

    @Override
    public List<DataChangeBizTypeEnum> getChangeTypes() {
        return Lists.newArrayList(DataChangeBizTypeEnum.PanAstPriceRule);
    }
}
