/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.core.service.report;

import com.abssqr.plat.common.facade.enums.DebtMoneyFieldEnum;
import com.abssqr.plat.common.facade.enums.EventCodeEnum;
import com.abssqr.plat.common.facade.enums.ReportAstStatTypeEnum;
import com.abssqr.plat.common.facade.enums.ReportDimTypeEnum;
import com.abssqr.plat.common.facade.model.report.ReportEntity;
import com.abssqr.plat.common.facade.model.report.ReportItem;
import com.abssqr.plat.common.model.domain.ast.Loan;
import com.abssqr.plat.common.model.domain.ast.LoanAst;
import com.abssqr.plat.common.model.domain.ast.LoanAstProfit;
import com.abssqr.plat.common.model.domain.ast.LoanInstal;
import com.abssqr.plat.common.model.domain.ast.LoanInstalDomain;
import com.abssqr.plat.common.model.domain.loanTrans.LoanAstLog;
import com.abssqr.plat.common.model.domain.loanTrans.LoanAstProfitLog;
import com.abssqr.plat.common.model.domain.loanTrans.LoanAstProfitTransDetail;
import com.abssqr.plat.common.model.domain.loanTrans.LoanAstTransDetail;
import com.abssqr.plat.common.model.domain.loanTrans.LoanInstalLog;
import com.abssqr.plat.common.model.domain.loanTrans.LoanInstalTransDetail;
import com.abssqr.plat.common.model.domain.loanTrans.LoanLog;
import com.abssqr.plat.common.model.domain.loanTrans.LoanTransDetail;
import com.abssqr.plat.core.service.report.calc.ReportItemCalc;
import com.abssqr.plat.core.service.report.calc.ReportItemCalcFactory;
import com.general.system.common.model.BaseEntity;
import com.general.system.common.util.LogUtil;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 计划统计报表
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.core.service.plan: PlanReportEntityManager.java, v 0.1 2019-06-26 17:42 zhenxuan.luo Exp $
 */
@Component
public class PlanReportEntityManager {
    
    private final static Logger LOGGER = LoggerFactory.getLogger(PlanReportEntityManager.class);
    
    private final static List<ReportAstStatTypeEnum> reportItems = Lists.newArrayList(ReportAstStatTypeEnum.values());
    
    private final static List<ReportAstStatTypeEnum> logReportItems       =
            Lists.newArrayList(ReportAstStatTypeEnum.ALL, ReportAstStatTypeEnum.STATUS, ReportAstStatTypeEnum.WTF_STATUS);
    /**
     * 转出、转入、结息
     */
    private static final List<EventCodeEnum>         SETT_TRF_EVENT_CODES =
            Lists.newArrayList(EventCodeEnum.TRANS_OUT, EventCodeEnum.TRANS_IN, EventCodeEnum.SETTLE);
    @Autowired
    private              ReportItemCalcFactory       reportItemCalcFactory;
    
    /**
     * 根据分期生成统计项
     *
     * @param loan
     * @param eventCode
     * @param accountDate
     * @return
     */
    public List<ReportEntity> getReportItemsForInit(Loan loan, EventCodeEnum eventCode,
                                                    Date accountDate) {
        
        List<ReportEntity> reportEntities = Lists.newArrayList();
        List<LoanInstalDomain> loanInstals = loan.getLoanInstalList();
        Map<ReportEntityKey, ReportEntity> reportEntityMap = Maps.newHashMap();
        // 统计全部分类
        for (ReportAstStatTypeEnum type : reportItems) {
            ReportItemCalc calc = reportItemCalcFactory.get(type);
            
            if (null == calc) {
                LogUtil.info(LOGGER, "没有找到对应的统计项计算器[{0}]", type);
                continue;
            }
            
            for (LoanInstal loanInstal : loanInstals) {
                // 非正常与逾期忽略
                if (!loanInstal.getStatus().isActive()) {
                    continue;
                }
                
                //生成对应类型报表统计item
                List<ReportItem> reportItems = calc.calcByInstal(loan, loanInstal, accountDate);
                
                for (ReportItem reportItem : reportItems) {
                    if (null == reportItem) {
                        continue;
                    }
                    ReportEntityKey key = new ReportEntityKey(loanInstal.getAstOrg().getAstOwnership().getOwnOrg(),
                            eventCode, reportItem);
                    ReportEntity reportEntity = reportEntityMap.get(key);
                    if (null == reportEntity) {
                        reportEntity = build(loanInstal, eventCode, accountDate);
                        reportEntity.setReportItem(reportItem);
                        reportEntity.setReportDimType(ReportDimTypeEnum.BAL);
                        reportEntities.add(reportEntity);
                        reportEntityMap.put(key, reportEntity);
                    }
                    //余额汇总
                    reportEntity.getAmt().addAll(loanInstal.getBal());
                }
            }
        }
        return reportEntities;
    }
    
    private ReportEntity build(LoanInstal loanInstal, EventCodeEnum eventCode, Date accountDate) {
        ReportEntity reportEntity = new ReportEntity();
        reportEntity.setPlanNo(loanInstal.getAstOrg().getAstOwnership().getOwnOrg());
        reportEntity.setEventCode(eventCode);
        reportEntity.setAstChg(1);
        reportEntity.setUserId(loanInstal.getUserId());
        reportEntity.setAstNo(loanInstal.getAstNo());
        reportEntity.setAccountDate(accountDate);
        
        return reportEntity;
    }
    
    /**
     * 根据分期生成统计项
     *
     * @param loan
     * @param eventCode
     * @param accountDate
     * @return
     */
    public List<ReportEntity> getReportItems(Loan loan, EventCodeEnum eventCode,
                                             Date accountDate) {
        
        List<LoanInstalDomain> loanInstals = loan.getLoanInstalList();
        Map<ReportEntityKey, ReportEntity> reportEntityMap = Maps.newHashMap();
        // 统计全部分类
        for (ReportAstStatTypeEnum type : reportItems) {
            ReportItemCalc calc = reportItemCalcFactory.get(type);
            
            if (null == calc) {
                LogUtil.info(LOGGER, "没有找到对应的统计项计算器[{0}]", type);
                continue;
            }
            
            for (LoanInstalDomain loanInstal : loanInstals) {
                // 非正常与逾期忽略
                if (!loanInstal.getStatus().isActive()) {
                    continue;
                }
                
                //生成对应类型报表统计item
                List<ReportItem> reportItems = calc.calcByInstal(loan, loanInstal, accountDate);
                
                for (ReportItem reportItem : reportItems) {
                    if (null == reportItem) {
                        continue;
                    }
                    
                    for (LoanAst loanAst : loanInstal.getLoanAsts()) {
                        if (loanAst.getPrinBal().isGreaterThanZero()) {
                            ReportEntity reportEntity = tryGetReportEntity(accountDate, loanAst.getOwnOrg(),
                                    loanAst.getAstNo(), loanInstal.getUserId(), eventCode, reportItem, reportEntityMap,
                                    ReportDimTypeEnum.BAL);
                            reportEntity.getAmt().addByFieldType(DebtMoneyFieldEnum.PRIN, loanAst.getPrinBal());
                        }
                    }
                    
                    for (LoanAstProfit loanAstProfit : loanInstal.getSourcedProfits()) {
                        if (loanAstProfit.getBal().getSum().isGreaterThanZero()) {
                            ReportEntity reportEntity = tryGetReportEntity(accountDate, loanAstProfit.getOwnOrg(),
                                    loanAstProfit.getAstNo(), loanInstal.getUserId(), eventCode, reportItem,
                                    reportEntityMap, ReportDimTypeEnum.BAL);
                            reportEntity.getAmt().addAll(loanAstProfit.getBal());
                        }
                    }
                }
            }
        }
        return Lists.newArrayList(reportEntityMap.values());
    }
    
    /**
     * 根据分期流水生成统计项
     *
     * @param loanTransDetail
     * @return
     */
    public List<ReportEntity> getReportItemsByLog(LoanTransDetail loanTransDetail) {
        Map<ReportEntityKey, ReportEntity> reportEntityMap = Maps.newHashMap();
        // 统计 不分类 状态 核销状态
        for (ReportAstStatTypeEnum type : logReportItems) {
            ReportItemCalc calc = reportItemCalcFactory.get(type);
            
            if (null == calc) {
                LogUtil.info(LOGGER, "没有找到对应的统计项计算器[{0}]", type);
                continue;
            }
            
            LoanLog loanLog = loanTransDetail.getActiveLog();
            List<LoanInstalTransDetail> loanInstalTransDetails = loanTransDetail.getLoanInstalTransDetails();
            for (LoanInstalTransDetail loanInstalTransDetail : loanInstalTransDetails) {
                LoanInstalLog loanInstalLog = loanInstalTransDetail.getActiveLog();
                List<ReportItem> reportItems = calc.calcByLog(loanLog, loanInstalLog);
                
                for (ReportItem reportItem : reportItems) {
                    if (null == reportItem) {
                        continue;
                    }
                    
                    for (LoanAstTransDetail astTransDetail : loanInstalTransDetail.getLoanAstTransDetails()) {
                        LoanAstLog loanAstLog = astTransDetail.getActiveLog();
                        LoanAst loanAst = astTransDetail.getLoanAst();
                        if (loanAstLog.getPrinAmt().isGreaterThanZero()) {
                            ReportEntity reportEntity = tryGetReportEntity(loanAstLog.getAccountDate(),
                                    loanAstLog.getOwnOrg(), loanAstLog.getAstNo(), loanInstalLog.getUserId(),
                                    loanInstalLog.getEventCode(), reportItem, reportEntityMap, ReportDimTypeEnum.AMT);
                            reportEntity.getAmt().addByFieldType(DebtMoneyFieldEnum.PRIN, loanAstLog.getPrinAmt());
                            reportEntity.getCost().addTo(loanAst.getCostAmt());
                        }
                    }
                    
                    for (LoanAstProfitTransDetail astProfitTransDetail : loanInstalTransDetail
                                                                                 .getLoanAstProfitTransDetails()) {
                        LoanAstProfitLog astProfitLog = astProfitTransDetail.getActiveLog();
                        if (astProfitLog.getAmt().getSum().isGreaterThanZero()) {
                            ReportEntity reportEntity = tryGetReportEntity(astProfitLog.getAccountDate(),
                                    astProfitLog.getOwnOrg(), astProfitLog.getAstNo(), loanInstalLog.getUserId(),
                                    astProfitLog.getEventCode(), reportItem, reportEntityMap, ReportDimTypeEnum.AMT);
                            reportEntity.getAmt().addAll(astProfitLog.getAmt());
                            reportEntity.getOrgDistPftAmt().addTo(astProfitLog.calcManagerFee());
                        }
                    }
                }
            }
        }
        return Lists.newArrayList(reportEntityMap.values());
    }
    
    /**
     * 发生结息生成日初信息
     * @param loanTransDetail
     * @param accountDate
     */
    public List<ReportEntity> getRptItemsBySettOrTrf(LoanTransDetail loanTransDetail, Date accountDate) {
        Map<ReportEntityKey, ReportEntity> reportEntityMap = Maps.newHashMap();
        // 根据特定事件码 生成item
        if (SETT_TRF_EVENT_CODES.contains(loanTransDetail.getActiveLog().getEventCode())) {
            // 到期应计计算器
            ReportItemCalc calc = reportItemCalcFactory.get(ReportAstStatTypeEnum.DUE);
            for (LoanInstalTransDetail instalTransDetail : loanTransDetail.getLoanInstalTransDetails()) {
                LoanInstalDomain loanInstal = instalTransDetail.getLoanInstal();
                LoanInstalLog loanInstalLog = instalTransDetail.getActiveLog();
                
                // 非正常与逾期忽略
                if (!loanInstal.getStatus().isActive()) {
                    continue;
                }
                // 生成对应报表统计 到期应计item
                List<ReportItem> reportItems = calc.calcByInstal(loanTransDetail.getLoan(), loanInstal, accountDate);
                
                for (ReportItem reportItem : reportItems) {
                    if (null == reportItem) {
                        continue;
                    }
                    
                    // 非结息交易 转让交易
                    if (loanInstalLog.getEventCode() != EventCodeEnum.SETTLE) {
                        for (LoanAstTransDetail astTransDetail : instalTransDetail.getLoanAstTransDetails()) {
                            LoanAstLog loanAstLog = astTransDetail.getActiveLog();
                            LoanAst loanAst = astTransDetail.getLoanAst();
                            EventCodeEnum eventCode = null;
                            if (loanAstLog.getPrinAmt().isGreaterThanZero()) {
                                switch (loanAstLog.getEventCode()) {
                                    case TRANS_OUT:
                                        eventCode = EventCodeEnum.STAT_SUB;
                                        break;
                                    case TRANS_IN:
                                        eventCode = EventCodeEnum.STAT_ADD;
                                        break;
                                    default:
                                        break;
                                }
                                
                                if (null == eventCode) {
                                    continue;
                                }
                                
                                ReportEntity reportEntity = tryGetReportEntity(loanAstLog.getAccountDate(),
                                        loanAstLog.getOwnOrg(), loanAstLog.getAstNo(), loanInstalLog.getUserId(),
                                        eventCode, reportItem, reportEntityMap,
                                        ReportDimTypeEnum.ACCRUED);
                                reportEntity.getAmt().addByFieldType(DebtMoneyFieldEnum.PRIN, loanAst.getPrinBal());
                            }
                        }
                    } else {
                        // 结息交易
                        for (LoanAst loanAst : loanInstal.getLoanAsts()) {
                            ReportEntity reportEntity = tryGetReportEntity(loanInstalLog.getAccountDate(),
                                    loanAst.getOwnOrg(), loanAst.getAstNo(), loanInstalLog.getUserId(),
                                    EventCodeEnum.STAT_ADD, reportItem, reportEntityMap,
                                    ReportDimTypeEnum.ACCRUED);
                            reportEntity.getAmt().addByFieldType(DebtMoneyFieldEnum.PRIN, loanAst.getPrinBal());
                        }
                        
                        for (LoanAstProfit loanAstProfit : loanInstal.getSourcedProfits()) {
                            if (loanAstProfit.getBal().getSum().isGreaterThanZero()) {
                                ReportEntity reportEntity = tryGetReportEntity(accountDate, loanAstProfit.getOwnOrg(),
                                        loanAstProfit.getAstNo(), loanInstal.getUserId(), EventCodeEnum.STAT_ADD,
                                        reportItem, reportEntityMap, ReportDimTypeEnum.ACCRUED);
                                reportEntity.getAmt().addAll(loanAstProfit.getBal());
                            }
                        }
                    }
                }
            }
        }
        
        return Lists.newArrayList(reportEntityMap.values());
    }
    
    private ReportEntity tryGetReportEntity(Date accountDate, String ownOrg, String astNo, String userId,
                                            EventCodeEnum eventCode, ReportItem reportItem,
                                            Map<ReportEntityKey, ReportEntity> reportEntityMap,
                                            ReportDimTypeEnum dimType) {
        ReportEntityKey key = new ReportEntityKey(ownOrg, eventCode, reportItem);
        ReportEntity reportEntity = reportEntityMap.get(key);
        if (null == reportEntity) {
            reportEntity = build(ownOrg, astNo, userId, eventCode, accountDate);
            reportEntity.setReportItem(reportItem);
            reportEntity.setReportDimType(dimType);
            reportEntityMap.put(key, reportEntity);
        }
        return reportEntity;
    }
    
    private ReportEntity build(String ownOrg, String astNo, String userId, EventCodeEnum eventCode, Date accountDate) {
        ReportEntity reportEntity = new ReportEntity();
        reportEntity.setPlanNo(ownOrg);
        reportEntity.setEventCode(eventCode);
        reportEntity.setAstChg(1);
        reportEntity.setUserId(userId);
        reportEntity.setAstNo(astNo);
        reportEntity.setAccountDate(accountDate);
        
        return reportEntity;
    }
    
    private class ReportEntityKey extends BaseEntity {
        String planNo;
        
        EventCodeEnum eventCode;
        
        ReportItem reportItem;
        
        public ReportEntityKey(String planNo, EventCodeEnum eventCode,
                               ReportItem reportItem) {
            this.planNo = planNo;
            this.eventCode = eventCode;
            this.reportItem = reportItem;
        }
    }
    
}