/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.core.service.util;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.Map;

import com.google.common.collect.Maps;
import freemarker.template.Configuration;
import freemarker.template.Template;
import freemarker.template.TemplateExceptionHandler;

/**
 * @author bangis.wangdf
 * @version com.abssqr.plat.core.service.util: FreeMarkerUtil.java, v 0.1 2019-11-28 22:35 bangis.wangdf Exp $
 */
public class FreeMarkerUtil {
    private static Configuration cfg;

    static {
        cfg = new Configuration(Configuration.VERSION_2_3_29);
        cfg.setClassForTemplateLoading(FreeMarkerUtil.class, "/template");
        // 设置默认编码
        cfg.setDefaultEncoding("utf-8");
        cfg.setTemplateExceptionHandler(TemplateExceptionHandler.RETHROW_HANDLER);
    }

    public static void genFile(Object dataModel, String templateFile, String fileName) {
        try {
            createDirs(fileName);
            Template template = cfg.getTemplate(templateFile);
            File file = new File(fileName);
            if(file.exists()){
                file.deleteOnExit();
            }
            Writer out = new BufferedWriter(new OutputStreamWriter(
                new FileOutputStream(file)));
            template.process(dataModel, out);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    /**
     * 创建目录
     * @return 存储文件全目录
     */
    private   static  void createDirs(String parentDir) {

        File parentFile = new File(parentDir);
        if (!parentFile.getParentFile().exists()) {
            parentFile.getParentFile().mkdirs();
        }
    }
}
