/**
 * abssqr.comInc.
 * Copyright(c)2019-2019AllRightsReserved.
 */
package com.abssqr.plat.core.service.workbenchquery;

import com.abssqr.plat.common.facade.enums.PlanStatusEnum;
import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.enums.ReportTypeEnum;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.model.report.PlanReport;
import com.abssqr.plat.common.facade.model.report.wk.WkAstReportFields;
import com.abssqr.plat.common.facade.result.workbench.OrgPlanInfo;
import com.abssqr.plat.common.model.repo.plan.PlanAstRepository;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.common.model.repo.report.PlanReportRepo;
import com.abssqr.plat.common.model.repo.task.AstSyncTaskCtrRepo;
import com.abssqr.plat.core.service.plan.PlanAllocManager;
import com.abssqr.plat.core.service.util.RptBuilderUtil;
import com.general.system.common.model.Money;
import com.general.system.tool.util.ToolUtil;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * 工作台 展示
 *
 * @author yangcheng
 * @version WorkBenchQueryManager.java, v0.12019-06-2614:44 yangchengExp$
 */
@Component
public class WorkBenchQueryManager {
    
    @Autowired
    private PlanRepo           planRepo;
    @Autowired
    private PlanAllocManager   planAllocManager;
    @Autowired
    private PlanAstRepository  planAstRepository;
    @Autowired
    private PlanReportRepo     planReportRepo;
    @Autowired
    private AstSyncTaskCtrRepo astSyncTaskCtrRepo;
    
    /**
     * 获取机构信息总览
     * @param planType
     * @return
     */
    public OrgPlanInfo getOrgProductInfo(PlanTypeEnum planType) {
        OrgPlanInfo orgPlanInfo = new OrgPlanInfo();
        //planAmount 存续期计划总规模
        orgPlanInfo.setPlanAmount(
                Money.createWithCent(planRepo.getAllPlanAmt(planType, PlanStatusEnum.ONGOING)));
        // 获取存续期计划
        List<PlanBase> planBases = planRepo.getPlansByStatus(planType, PlanStatusEnum.ONGOING);
        if(ToolUtil.isEmpty(planBases)){
            return orgPlanInfo;
        }
        if (PlanTypeEnum.SPV == planType){
            // 改为获取最大报表数据进行汇总
            Money allPlanTotal = new Money();
            planBases.forEach(one -> {
                // 获取最大日的报表
                PlanReport report = planReportRepo.getLatestReport(one.getPlanNo(), ReportTypeEnum.RPT001);
                // 获取累计放款
                allPlanTotal.addTo(new Money(RptBuilderUtil.getReportVal( report, WkAstReportFields.WK116.getCode())));
            });
            orgPlanInfo.setTrfAmount(allPlanTotal);
        }else{
            // 承接池 统计 累计转让规模
            orgPlanInfo.setTrfAmount(Money.createWithCent(planAstRepository.getAllPlanAmt(planType)));
        }
        //planOngoingCount 产品数量(存续期)
        if (CollectionUtils.isNotEmpty(planBases)) {
            orgPlanInfo.setPlanOngoingCount(planBases.size());
        }
        //recentlyPlans 最近支付产品列表
        orgPlanInfo.setRecentlyPlan(planAllocManager.queryRecentlyPlans(planType));
        return orgPlanInfo;
    }
    
}
