/**
 * abssqr.comInc.
 * Copyright(c)2019-2019AllRightsReserved.
 */
package com.abssqr.plat.web.controller.workOrder;

import com.abssqr.plat.common.facade.enums.DataChangeBizTypeEnum;
import com.abssqr.plat.common.facade.param.workOrder.WorkOrderPageParam;
import com.abssqr.plat.common.facade.param.workOrder.WorkOrderQprParam;
import com.abssqr.plat.common.facade.param.workOrder.WorkOrderQryParam;
import com.abssqr.plat.common.facade.result.BasePage;
import com.abssqr.plat.common.facade.result.workOrder.WorkOrderBtn;
import com.abssqr.plat.common.facade.result.workOrder.WorkOrderVo;
import com.abssqr.plat.common.facade.service.WorkOrderService;
import com.abssqr.plat.common.facade.validation.Valid;
import com.general.enhanced.controller.annotation.RestJsonController;
import com.general.system.common.model.BaseResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;

import java.util.List;

/**
 * 工单管理controller
 *
 * @author yangcheng
 * @version WorkOrderController.java, v0.12019-06-1009:52 yangchengExp$
 */
@RestJsonController
public class WorkOrderController {
    
    @Autowired
    private WorkOrderService workOrderService;
    /**
     * @param param
     * @return
     * @description 我的待办
     */
    @RequestMapping(path = "/workOrder/my.json", method = RequestMethod.GET)
    public BasePage<WorkOrderVo> myWorkOrder(@Valid WorkOrderPageParam param) {
        return workOrderService.myWorkOrder(param);
    }
    /**
     * @param param
     * @return
     * @description 我的申请
     */
    @RequestMapping(path = "/workOrder/apply.json", method = RequestMethod.GET)
    public BasePage<WorkOrderVo> applyWorkOrder(@Valid WorkOrderPageParam param) {
        return workOrderService.applyWorkOrder(param);
    }
    /**
     * @param param
     * @return
     * @description 已办事项
     */
    @RequestMapping(path = "/workOrder/history.json", method = RequestMethod.GET)
    public BasePage<WorkOrderVo> hisWorkOrder(@Valid WorkOrderPageParam param) {
        return workOrderService.hisWorkOrder(param);
    }
    /**
     * 查看已申请信息 包含 实例信息、历史记录及追踪图
     */
    @RequestMapping(path = "/workOrder/detail.json", method = RequestMethod.GET)
    public List<WorkOrderVo> detail(WorkOrderQryParam param) {
        return workOrderService.getWorkOrderTasks(param);
    }
    
    /**
     *
     * @return
     * @description 详情页对应的按钮数据
     */
    @RequestMapping(path = "/workOrder/btn.json", method = RequestMethod.GET)
    public List<WorkOrderBtn> getBtns(WorkOrderQryParam param) {
        return workOrderService.getBtns(param);
    }
    
    /**
     * 工单统一操作入口
     * @return
     * @description 详情页对应的按钮数据
     */
    @RequestMapping(path = "/workOrder/operate/{operate}.json", method = RequestMethod.GET)
    public BaseResult operate(WorkOrderQprParam param) {
        workOrderService.operate(param);
        return BaseResult.success("操作成功");
    }

    /**
     * 获取工单业务的业务类型枚举
     * @return
     */
    @RequestMapping(path = "/workOrder/getBizType.json", method = RequestMethod.GET)
    public DataChangeBizTypeEnum getBizType(WorkOrderQryParam param) {
        return workOrderService.getBizType(param);
    }
    
    /**
     * 工单初始化流程文件
     */
    @RequestMapping(path = "/workOrder/init.json", method = RequestMethod.GET)
    public BaseResult init() {
        BaseResult result=  BaseResult.success("操作成功");
        result.setDataObject(workOrderService.init());
        return result;
    }

}
