/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.web.fileExport;

import cn.hutool.core.io.IoUtil;
import com.abssqr.plat.common.facade.enums.CommonErrorCodeEnum;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.general.system.common.util.MessageUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Map;

/**
 *
 * @author zhenxuan.luo
 * @version com.ruiyi.rdassist.web.util: FileExportUtil.java, v 0.1 2019-03-01 15:12 zhenxuan.luo Exp $ 
 */
@Component
public class FileExportUtil {
    
    @Autowired
    private XlsExportUtil xlsExportUtil;
    
    public void exportXls(HttpServletResponse response, FileExportTypeEnum fileType, String fileKey,
                          Map<String, Object> model)
            throws IOException {
        String fileName = fileType.getFilePrefix().concat(fileKey).concat(fileType.getFilePostfix());
        
        InputStream inputStream = Thread.currentThread().getContextClassLoader()
                                          .getResourceAsStream(fileType.getTmplPath());
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        
        try {
            
            xlsExportUtil.export(inputStream, byteArrayOutputStream, model);
            byteArrayOutputStream.flush();
            
            response.reset();
            response.setCharacterEncoding("UTF-8");
            response.setContentType("multipart/form-data");
            String outfileName = new String(fileName.getBytes(), "ISO-8859-1");
            response.setHeader("Content-Disposition", "attachment;fileName=" + outfileName);
            response.getOutputStream().write(byteArrayOutputStream.toByteArray());
            response.getOutputStream().flush();
            
        } catch (Throwable e) {
            throw new AbssqrBizException(MessageUtil.formatMsg("文件[{0}]导出失败", fileName),
                    CommonErrorCodeEnum.FILE_EXPORT_ERROR, e);
        } finally {
            IoUtil.close(response.getOutputStream());
            IoUtil.close(byteArrayOutputStream);
            IoUtil.close(inputStream);
        }
    }
}